@@ -228,7 +228,10 @@ sub unsupported_devices() {
 		if (/15b3:(
 			1002| 6368| 6732| 673c| 6740| 6750| 6764| # CTX2
 			1003| 1004| 1007|	# ConnectX-3
+			1023|			# ConnectX-8
+			1025|			# ConnectX-9
 			1011| 1012|		# Connect-IB
+			a2dc|			# BlueField-3
 			____
 			)/x) {
 			push @devices, $_;
@@ -243,7 +246,8 @@ sub unsupported_devices_check() {
 	return unless (@devices);
 
 	my $error = "System has one or more unsupported device, see below.\n";
-	$error .= "MLNX_OFED / mlnx_en 5.1 and above supports only ConnectX-4 or newer devices.\n";
+	$error .= "ConnectX-8/BlueField-3 and newer devices are only supported in DOCA-host and not in MLNX_OFED / mlnx_en\n";
+	$error .= "ConnectX-3 devices are only supported in MLNX_OFED / mlnx_en 4.9 and below.\n";
 	$error .= "This device could become unavailable which might result in loss of connectivity.\n" ;
 	$error .= "Use --skip-unsupported-devices-check to skip this check.\n" ;
 	$error .= "Aborting.";
@@ -434,11 +438,12 @@ sub parse_rpm_dist($$) {
 	} elsif ($dist_rpm =~ /redhat-release-.*-7.9|sl-release-7.9|centos-release-7-9/) {
 		$DISTRO = "RHEL7.9";
 		$rpm_distro = "rhel7u9";
-	} elsif ($dist_rpm =~ /(redhat|centos|centos-linux|almalinux|rocky|centos-stream)-release-([89])\.(\d+)-/) {
+	} elsif ($dist_rpm =~ /(redhat|centos|centos-linux|almalinux|rocky|centos-stream)-release-([89]|10)\.(\d+)-/) {
 		my ($major, $minor) = ($2, $3);
 		if (
 			(($major eq "8") and ($minor <= 10)) ||
-			(($major eq "9") and ($minor <= 4))
+			(($major eq "9") and ($minor <= 6)) ||
+			(($major eq "10") and ($minor <= 0))
 		) {
 			$DISTRO = "RHEL$2.$3";
 			$rpm_distro = "rhel$2u$3";
@@ -495,6 +500,9 @@ sub parse_rpm_dist($$) {
 	} elsif ($dist_rpm =~ /bclinux-release-(21)\.(10)/) {
 		$DISTRO = "BCLINUX$1.$2SP2";
 		$rpm_distro = "bclinux${1}u${2}sp2";
+	} elsif ($dist_rpm =~ /bclinux-release-22\.10/) {
+		$DISTRO = "BCLINUX22.10";
+		 $rpm_distro = "bclinux22u10.oe2203sp2.bclinux";
 	} elsif ($dist_rpm =~ /centos-release-5-10.el5.centos/) {
 		$DISTRO = "XenServer6.5";
 		$rpm_distro = "xenserver6u5";
@@ -572,7 +580,7 @@ sub parse_rpm_dist($$) {
 		my $sp = $1;
 		$DISTRO = "EULEROS2.0SP${sp}";
 		$rpm_distro = "euleros2u0sp${sp}";
-	} elsif ($dist_rpm =~ /openEuler-release-(20\.03)LTS_SP(1|3)/) {
+	} elsif ($dist_rpm =~ /openEuler-release-(2[02]\.03)LTS_SP(1|3)/) {
 		my $dist_num = $1;
 		my $sp = $2;
 		$DISTRO = "OPENEULER${dist_num}SP${sp}";
@@ -595,9 +603,12 @@ sub parse_rpm_dist($$) {
 	} elsif ($dist_rpm =~ /kylin-release-10/) {
 		$DISTRO = "KYLIN10";
 		$rpm_distro = "kylin10";
-	} elsif ($dist_rpm =~ /UnionTech-release-1020/) {
-		$DISTRO = "UOS20.1020";
-		$rpm_distro = "uos20.1020";
+	} elsif ($dist_rpm =~ /UnionTech-release-(10[26]0)/) {
+		$DISTRO = "UOS20.$1";
+		$rpm_distro = "uos20.$1";
+	} elsif ($dist_rpm =~ /UnionTech-release-20-1060a/) {
+		$DISTRO = "UOS20.1060a";
+		$rpm_distro = "uos20.1060a";
 	} elsif ($dist_rpm =~ /photon-release-3.0-/) {
 		$DISTRO = "PHOTON3";
 		$rpm_distro = "photon3";
@@ -606,16 +617,22 @@ sub parse_rpm_dist($$) {
 		$rpm_distro = "mariner1.0";
 	} elsif ($dist_rpm =~ /mariner-release-2.0/) {
 		$DISTRO = "MARINER2.0";
-		$rpm_distro = "mariner2.0";
+		$rpm_distro = "mariner2u0";
+	} elsif ($dist_rpm =~ /azurelinux-release-3.0/) {
+		$DISTRO = "AZURELINUX3.0";
+		$rpm_distro = "azurelinux3u0";
 	} elsif ($dist_rpm =~ /anolis-release-8.([46])/) {
 		$DISTRO = "ANOLIS8.$1";
 		$rpm_distro = "anolis8u$1";
-	} elsif ($dist_rpm =~ /ctyunos-release-2.0/) {
-		$DISTRO = "CTYUNOS2.0";
-		$rpm_distro = "ctyunos2.0";
+	} elsif ($dist_rpm =~ /ctyunos-release-2.22.06/) {
+		$DISTRO = "CTYUNOS22.06";
+		$rpm_distro = "ctyunos22.06";
 	} elsif ($dist_rpm =~ /ctyunos-release-23.01/) {
 		$DISTRO = "CTYUNOS23.01";
 		$rpm_distro = "ctyunos23.01";
+	} elsif ($dist_rpm =~ /tencentos-release-3.3/) {
+		$DISTRO = "TENCENTOS3.3";
+		$rpm_distro = "tencentos3u3";
 	} elsif ($dist_rpm =~ /tencentos-release/) {
 		$DISTRO = "TENCENT";
 		$rpm_distro = "tencent";
