@@ -35,8 +35,8 @@
 #include <linux/errno.h>
 
 #define DRV_NAME	"iw_cxgb4"
-#define DRV_VERSION	"23.10-6.1.6"
-#define DRV_RELDATE	"01 Jan 2026"
+#define DRV_VERSION	"24.10-3.2.5"
+#define DRV_RELDATE	"29 Jun 2025"
 
 MODULE_AUTHOR("Alaa Hleihel");
 MODULE_DESCRIPTION("iw_cxgb4 dummy kernel module");
