@@ -6,7 +6,6 @@
 #include "fw_reset.h"
 #include "diag/fw_tracer.h"
 #include "lib/tout.h"
-#include "sf/sf.h"
 
 enum {
 	MLX5_FW_RESET_FLAGS_RESET_REQUESTED,
@@ -28,6 +27,7 @@ struct mlx5_fw_reset {
 	struct work_struct reset_abort_work;
 	struct delayed_work reset_timeout_work;
 	unsigned long reset_flags;
+	u8 reset_method;
 	struct timer_list timer;
 	struct completion done;
 	int ret;
@@ -36,6 +36,7 @@ struct mlx5_fw_reset {
 enum {
 	MLX5_FW_RST_STATE_IDLE = 0,
 	MLX5_FW_RST_STATE_TOGGLE_REQ = 4,
+	MLX5_FW_RST_STATE_DROP_MODE = 5,
 };
 
 enum {
@@ -96,7 +97,8 @@ static int mlx5_reg_mfrl_set(struct mlx5_core_dev *dev, u8 reset_level,
 }
 
 static int mlx5_reg_mfrl_query(struct mlx5_core_dev *dev, u8 *reset_level,
-			       u8 *reset_type, u8 *reset_state)
+			       u8 *reset_type, u8 *reset_state, u8 *reset_method,
+			       u8 *sync_flow)
 {
 	u32 out[MLX5_ST_SZ_DW(mfrl_reg)] = {};
 	u32 in[MLX5_ST_SZ_DW(mfrl_reg)] = {};
@@ -106,19 +108,36 @@ static int mlx5_reg_mfrl_query(struct mlx5_core_dev *dev, u8 *reset_level,
 	if (err)
 		return err;
 
+	if (sync_flow)
+		*sync_flow = MLX5_GET(mfrl_reg, out, pci_sync_for_fw_update_start);
 	if (reset_level)
 		*reset_level = MLX5_GET(mfrl_reg, out, reset_level);
 	if (reset_type)
 		*reset_type = MLX5_GET(mfrl_reg, out, reset_type);
 	if (reset_state)
 		*reset_state = MLX5_GET(mfrl_reg, out, reset_state);
+	if (reset_method)
+		*reset_method = MLX5_GET(mfrl_reg, out, pci_reset_req_method);
 
 	return 0;
 }
 
 int mlx5_fw_reset_query(struct mlx5_core_dev *dev, u8 *reset_level, u8 *reset_type)
 {
-	return mlx5_reg_mfrl_query(dev, reset_level, reset_type, NULL);
+	return mlx5_reg_mfrl_query(dev, reset_level, reset_type, NULL, NULL, NULL);
+}
+
+static int mlx5_fw_reset_get_request_info(struct mlx5_core_dev *dev,
+					  u8 *reset_method, u8 *sync_flow)
+{
+	if (!MLX5_CAP_GEN(dev, pcie_reset_using_hotreset_method) &&
+	    !MLX5_CAP_GEN(dev, pci_sync_for_fw_update_sync2_using_tool_reset)) {
+		*reset_method = MLX5_MFRL_REG_PCI_RESET_METHOD_LINK_TOGGLE;
+		*sync_flow = MLX5_MFRL_REG_SYNCED_DRIVER_FLOW;
+		return 0;
+	}
+
+	return mlx5_reg_mfrl_query(dev, NULL, NULL, NULL, reset_method, sync_flow);
 }
 
 static int mlx5_fw_reset_get_reset_state_err(struct mlx5_core_dev *dev,
@@ -126,20 +145,26 @@ static int mlx5_fw_reset_get_reset_state_err(struct mlx5_core_dev *dev,
 {
 	u8 reset_state;
 
-	if (mlx5_reg_mfrl_query(dev, NULL, NULL, &reset_state))
+	if (mlx5_reg_mfrl_query(dev, NULL, NULL, &reset_state, NULL, NULL))
 		goto out;
 
+	if (!reset_state)
+		return 0;
+
 	switch (reset_state) {
 	case MLX5_MFRL_REG_RESET_STATE_IN_NEGOTIATION:
 	case MLX5_MFRL_REG_RESET_STATE_RESET_IN_PROGRESS:
-		NL_SET_ERR_MSG_MOD(extack, "Sync reset was already triggered");
+		NL_SET_ERR_MSG_MOD(extack, "Sync reset still in progress");
 		return -EBUSY;
-	case MLX5_MFRL_REG_RESET_STATE_TIMEOUT:
-		NL_SET_ERR_MSG_MOD(extack, "Sync reset got timeout");
+	case MLX5_MFRL_REG_RESET_STATE_NEG_TIMEOUT:
+		NL_SET_ERR_MSG_MOD(extack, "Sync reset negotiation timeout");
 		return -ETIMEDOUT;
 	case MLX5_MFRL_REG_RESET_STATE_NACK:
 		NL_SET_ERR_MSG_MOD(extack, "One of the hosts disabled reset");
 		return -EPERM;
+	case MLX5_MFRL_REG_RESET_STATE_UNLOAD_TIMEOUT:
+		NL_SET_ERR_MSG_MOD(extack, "Sync reset unload timeout");
+		return -ETIMEDOUT;
 	}
 
 out:
@@ -153,7 +178,7 @@ int mlx5_fw_reset_set_reset_sync(struct mlx5_core_dev *dev, u8 reset_type_sel,
 	struct mlx5_fw_reset *fw_reset = dev->priv.fw_reset;
 	u32 out[MLX5_ST_SZ_DW(mfrl_reg)] = {};
 	u32 in[MLX5_ST_SZ_DW(mfrl_reg)] = {};
-	int err;
+	int err, rst_res;
 
 	set_bit(MLX5_FW_RESET_FLAGS_PENDING_COMP, &fw_reset->reset_flags);
 
@@ -166,13 +191,34 @@ int mlx5_fw_reset_set_reset_sync(struct mlx5_core_dev *dev, u8 reset_type_sel,
 		return 0;
 
 	clear_bit(MLX5_FW_RESET_FLAGS_PENDING_COMP, &fw_reset->reset_flags);
-	if (err == -EREMOTEIO && MLX5_CAP_MCAM_FEATURE(dev, reset_state))
-		return mlx5_fw_reset_get_reset_state_err(dev, extack);
+	if (err == -EREMOTEIO && MLX5_CAP_MCAM_FEATURE(dev, reset_state)) {
+		rst_res = mlx5_fw_reset_get_reset_state_err(dev, extack);
+		return rst_res ? rst_res : err;
+	}
 
 	NL_SET_ERR_MSG_MOD(extack, "Sync reset command failed");
 	return mlx5_cmd_check(dev, err, in, out);
 }
 
+int mlx5_fw_reset_verify_fw_complete(struct mlx5_core_dev *dev,
+				     struct netlink_ext_ack *extack)
+{
+	u8 rst_state;
+	int err;
+
+	err = mlx5_fw_reset_get_reset_state_err(dev, extack);
+	if (err)
+		return err;
+
+	rst_state = mlx5_get_fw_rst_state(dev);
+	if (!rst_state)
+		return 0;
+
+	mlx5_core_err(dev, "Sync reset did not complete, state=%d\n", rst_state);
+	NL_SET_ERR_MSG_MOD(extack, "Sync reset did not complete successfully");
+	return rst_state;
+}
+
 int mlx5_fw_reset_set_live_patch(struct mlx5_core_dev *dev)
 {
 	return mlx5_reg_mfrl_set(dev, MLX5_MFRL_REG_RESET_LEVEL0, 0, 0, false);
@@ -181,6 +227,7 @@ int mlx5_fw_reset_set_live_patch(struct mlx5_core_dev *dev)
 static void mlx5_fw_reset_complete_reload(struct mlx5_core_dev *dev, bool unloaded)
 {
 	struct mlx5_fw_reset *fw_reset = dev->priv.fw_reset;
+	struct devlink *devlink = priv_to_devlink(dev);
 
 	/* if this is the driver that initiated the fw reset, devlink completed the reload */
 	if (test_bit(MLX5_FW_RESET_FLAGS_PENDING_COMP, &fw_reset->reset_flags)) {
@@ -192,9 +239,11 @@ static void mlx5_fw_reset_complete_reload(struct mlx5_core_dev *dev, bool unload
 			mlx5_core_err(dev, "reset reload flow aborted, PCI reads still not working\n");
 		else
 			mlx5_load_one(dev, true);
-		devlink_remote_reload_actions_performed(priv_to_devlink(dev), 0,
+		devl_lock(devlink);
+		devlink_remote_reload_actions_performed(devlink, 0,
 							BIT(DEVLINK_RELOAD_ACTION_DRIVER_REINIT) |
 							BIT(DEVLINK_RELOAD_ACTION_FW_ACTIVATE));
+		devl_unlock(devlink);
 	}
 }
 
@@ -301,25 +350,6 @@ static void mlx5_fw_live_patch_event(struct work_struct *work)
 		mlx5_core_err(dev, "Failed to reload FW tracer\n");
 }
 
-static const struct pci_device_id rshim_device_ids[] = {
-	{ PCI_VDEVICE(MELLANOX, 0xc2d2) }, /* BlueField1 RShim device ID */
-	{ PCI_VDEVICE(MELLANOX, 0xc2d3) }, /* BlueField2 RShim device ID */
-	{ PCI_VDEVICE(MELLANOX, 0xc2d6) }, /* BlueField2 RShim device ID2 */
-	{ PCI_VDEVICE(MELLANOX, 0xc2d5) }  /* BlueField3 RShim device ID */
-};
-
-static bool mlx5_is_rshim_pci_device(struct mlx5_core_dev *dev, u16 dev_id)
-{
-	int i;
-
-	for (i = 0; i < ARRAY_SIZE(rshim_device_ids); ++i) {
-		if (rshim_device_ids[i].device == dev_id)
-			return true;
-	}
-
-	return false;
-}
-
 #if IS_ENABLED(CONFIG_HOTPLUG_PCI_PCIE)
 static int mlx5_check_hotplug_interrupt(struct mlx5_core_dev *dev)
 {
@@ -343,6 +373,25 @@ static int mlx5_check_hotplug_interrupt(struct mlx5_core_dev *dev)
 }
 #endif
 
+static const struct pci_device_id mgt_ifc_device_ids[] = {
+	{ PCI_VDEVICE(MELLANOX, 0xc2d2) }, /* BlueField1 MGT interface device ID */
+	{ PCI_VDEVICE(MELLANOX, 0xc2d3) }, /* BlueField2 MGT interface device ID */
+	{ PCI_VDEVICE(MELLANOX, 0xc2d4) }, /* BlueField3-Lx MGT interface device ID */
+	{ PCI_VDEVICE(MELLANOX, 0xc2d5) }, /* BlueField3 MGT interface device ID */
+	{ PCI_VDEVICE(MELLANOX, 0xc2d6) }, /* BlueField4 MGT interface device ID */
+};
+
+static bool mlx5_is_mgt_ifc_pci_device(struct mlx5_core_dev *dev, u16 dev_id)
+{
+	int i;
+
+	for (i = 0; i < ARRAY_SIZE(mgt_ifc_device_ids); ++i)
+		if (mgt_ifc_device_ids[i].device == dev_id)
+			return true;
+
+	return false;
+}
+
 static int mlx5_check_dev_ids(struct mlx5_core_dev *dev, u16 dev_id)
 {
 	struct pci_bus *bridge_bus = dev->pdev->bus;
@@ -357,10 +406,11 @@ static int mlx5_check_dev_ids(struct mlx5_core_dev *dev, u16 dev_id)
 		err = pci_read_config_word(sdev, PCI_DEVICE_ID, &sdev_id);
 		if (err)
 			return pcibios_err_to_errno(err);
+
 		if (sdev_id == dev_id)
 			continue;
 
-		if (mlx5_is_rshim_pci_device(dev, sdev_id))
+		if (mlx5_is_mgt_ifc_pci_device(dev, sdev_id))
 			continue;
 
 		mlx5_core_warn(dev, "unrecognized dev_id (0x%x)\n", sdev_id);
@@ -369,7 +419,8 @@ static int mlx5_check_dev_ids(struct mlx5_core_dev *dev, u16 dev_id)
 	return 0;
 }
 
-static bool mlx5_is_reset_now_capable(struct mlx5_core_dev *dev)
+static bool mlx5_is_reset_now_capable(struct mlx5_core_dev *dev,
+				      u8 reset_method)
 {
 	u16 dev_id;
 	int err;
@@ -379,15 +430,12 @@ static bool mlx5_is_reset_now_capable(struct mlx5_core_dev *dev)
 		return false;
 	}
 
-	if (!mlx5_core_is_ecpf(dev) && !mlx5_sf_table_empty(dev)) {
-		mlx5_core_warn(dev, "SFs should be removed before reset\n");
-		return false;
-	}
-
 #if IS_ENABLED(CONFIG_HOTPLUG_PCI_PCIE)
-	err = mlx5_check_hotplug_interrupt(dev);
-	if (err)
-		return false;
+	if (reset_method != MLX5_MFRL_REG_PCI_RESET_METHOD_HOT_RESET) {
+		err = mlx5_check_hotplug_interrupt(dev);
+		if (err)
+			return false;
+	}
 #endif
 
 	err = pci_read_config_word(dev->pdev, PCI_DEVICE_ID, &dev_id);
@@ -402,10 +450,22 @@ static void mlx5_sync_reset_request_event(struct work_struct *work)
 						      reset_request_work);
 	struct mlx5_core_dev *dev = fw_reset->dev;
 	unsigned long sync_reset_timeout;
+	bool nack_request = false;
+	u8 sync_flow;
 	int err;
 
-	if (test_bit(MLX5_FW_RESET_FLAGS_NACK_RESET_REQUEST, &fw_reset->reset_flags) ||
-	    !mlx5_is_reset_now_capable(dev)) {
+	err = mlx5_fw_reset_get_request_info(dev, &fw_reset->reset_method,
+					     &sync_flow);
+	if (err) {
+		mlx5_core_warn(dev, "Failed reading MFRL, err %d\n", err);
+		nack_request = true;
+	} else {
+		if (sync_flow == MLX5_MFRL_REG_SYNCED_DRIVER_FLOW &&
+		    !mlx5_is_reset_now_capable(dev, fw_reset->reset_method))
+			nack_request = true;
+	}
+
+	if (nack_request || test_bit(MLX5_FW_RESET_FLAGS_NACK_RESET_REQUEST, &fw_reset->reset_flags)) {
 		err = mlx5_fw_reset_set_reset_sync_nack(dev);
 		mlx5_core_warn(dev, "PCI Sync FW Update Reset Nack %s",
 			       err ? "Failed" : "Sent");
@@ -423,213 +483,15 @@ static void mlx5_sync_reset_request_event(struct work_struct *work)
 		mlx5_core_warn(dev, "PCI Sync FW Update Reset Ack. Device reset is expected.\n");
 }
 
-static int mlx5_pci_config_hw_control(struct pci_dev *root_port,
-				      bool new_val, bool *prev_val)
-{
-	u16 root_ctl, root_cap;
-	bool curr_val;
-	int ret;
-
-	pcie_capability_read_word(root_port, PCI_EXP_RTCAP, &root_cap);
-	if (!(root_cap & PCI_EXP_RTCAP_CRSVIS))
-		return 0;
-
-	pcie_capability_read_word(root_port, PCI_EXP_RTCTL, &root_ctl);
-	curr_val = !(root_ctl & PCI_EXP_RTCTL_CRSSVE);
-
-	if (prev_val)
-		*prev_val = curr_val;
-
-	if (curr_val == new_val)
-		return 0;
-
-	if (new_val)
-		ret = pcie_capability_clear_word(root_port, PCI_EXP_RTCTL,
-						 PCI_EXP_RTCTL_CRSSVE);
-	else
-		ret = pcie_capability_set_word(root_port, PCI_EXP_RTCTL,
-					       PCI_EXP_RTCTL_CRSSVE);
-
-	if (ret) {
-		dev_err(&root_port->dev, "Failed to config CRSSVE bit, err(%d)\n", ret);
-		return ret;
-	}
-
-	return 0;
-}
-
-static void mlx5_pci_restore(struct pci_dev *dev)
-{
-	struct pci_dev *pdev;
-
-	pci_restore_state(dev);
-	if (!dev->subordinate)
-		return;
-	list_for_each_entry(pdev, &dev->subordinate->devices, bus_list)
-		mlx5_pci_restore(pdev);
-}
-
-static void mlx5_pci_unlock(struct pci_dev *dev)
-{
-	struct pci_dev *pdev;
-
-	pci_cfg_access_unlock(dev);
-	if (!dev->subordinate)
-		return;
-	list_for_each_entry(pdev, &dev->subordinate->devices, bus_list)
-		mlx5_pci_unlock(pdev);
-}
-
-static bool mlx5_pci_is_dev_same_id(struct pci_dev *pdev, u16 dev_id)
-{
-	u16 pdev_id;
-
-	if (pci_read_config_word(pdev, PCI_DEVICE_ID, &pdev_id))
-		return false;
-	if (pdev_id != dev_id)
-		return false;
-	return true;
-}
-
-static bool mlx5_pci_are_all_devs_same_id(struct pci_dev *dev, u16 nic_dev_id)
-{
-	struct pci_dev *pdev;
-	int type;
-
-	if (!dev->subordinate) {
-		type = pci_pcie_type(dev);
-		if (type != PCI_EXP_TYPE_ENDPOINT &&
-		    type != PCI_EXP_TYPE_LEG_END)
-			return true;
-		return mlx5_pci_is_dev_same_id(dev, nic_dev_id);
-	}
-
-	list_for_each_entry(pdev, &dev->subordinate->devices, bus_list)
-		if (!mlx5_pci_are_all_devs_same_id(pdev, nic_dev_id))
-			return false;
-
-	return true;
-}
-
-static void mlx5_pci_save_locked(struct pci_dev *dev)
-{
-	struct pci_dev *pdev;
-
-	pci_cfg_access_lock(dev);
-	pci_save_state(dev);
-	if (!dev->subordinate)
-		return;
-	list_for_each_entry(pdev, &dev->subordinate->devices, bus_list)
-		mlx5_pci_save_locked(pdev);
-}
-
-static int mlx5_check_reset_criteria(struct mlx5_core_dev *dev,
-				     struct pci_dev *root_port)
-{
-	u16 dev_id;
-	int err;
-
-	err = pci_read_config_word(dev->pdev, PCI_DEVICE_ID, &dev_id);
-	if (err)
-		return err;
-
-	if (!mlx5_pci_are_all_devs_same_id(root_port, dev_id))
-		return -EPERM;
-
-	return 0;
-}
-
-static int mlx5_reset_pci_topology(struct mlx5_core_dev *dev,
-				   struct pci_dev *root_port)
-{
-	struct pci_dev *bridge;
-	unsigned long timeout;
-	int err;
-
-	bridge = dev->pdev->bus->self;
-
-	/* Disable PCI link */
-	err = pcie_capability_set_word(bridge, PCI_EXP_LNKCTL,
-				       PCI_EXP_LNKCTL_LD);
-	if (err) {
-		mlx5_core_err(dev, "Failed to disable pci link, err (%d)\n", err);
-		return err;
-	}
-	msleep(500);
-
-	/* Wait for PCI link comes back with timeout */
-	timeout = jiffies + msecs_to_jiffies(mlx5_tout_ms(dev, PCI_TOGGLE));
-	do {
-		if (pci_device_is_present(root_port))
-			break;
-		msleep(20);
-	} while (!time_after(jiffies, timeout));
-
-	if (pci_device_is_present(root_port)) {
-		mlx5_core_info(dev, "PCIe topology is ready\n");
-	} else {
-		mlx5_core_err(dev, "PCIe topology is not ready after %lu ms\n",
-				msecs_to_jiffies(mlx5_tout_ms(dev, PCI_TOGGLE)));
-		return -ETIMEDOUT;
-	}
-	return 0;
-}
-
-static int mlx5_pci_link_toggle_ecpf(struct mlx5_core_dev *dev)
-{
-	struct pci_dev *root_port;
-	bool prev;
-	int err;
-
-	root_port = pcie_find_root_port(dev->pdev);
-	if (!root_port) {
-		mlx5_core_err(dev, "Failed to find root port\n");
-		return -ENODEV;
-	}
-
-	err = mlx5_check_reset_criteria(dev, root_port);
-	if (err) {
-		mlx5_core_err(dev, "Device does not match reset criteria, err(%d)\n",
-			      err);
-		return err;
-	}
-
-	err = mlx5_pci_config_hw_control(root_port, true, &prev);
-	if (err) {
-		mlx5_core_err(dev, "Failed to enable hardware retry\n");
-		return -EIO;
-	}
-
-	mlx5_pci_save_locked(root_port);
-
-	err = mlx5_reset_pci_topology(dev, root_port);
-	if (err)
-		goto err_reset;
-
-	mlx5_pci_restore(root_port);
-err_reset:
-	mlx5_pci_unlock(root_port);
-	mlx5_pci_config_hw_control(root_port, prev, NULL);
-	return err;
-}
-
-static int mlx5_pci_link_toggle_pf(struct mlx5_core_dev *dev)
+static int mlx5_pci_link_toggle(struct mlx5_core_dev *dev, u16 dev_id)
 {
 	struct pci_bus *bridge_bus = dev->pdev->bus;
 	struct pci_dev *bridge = bridge_bus->self;
 	unsigned long timeout;
 	struct pci_dev *sdev;
-	u16 reg16, dev_id;
 	int cap, err;
 	u32 reg32;
-
-	err = pci_read_config_word(dev->pdev, PCI_DEVICE_ID, &dev_id);
-	if (err)
-		return pcibios_err_to_errno(err);
-
-	err = mlx5_check_dev_ids(dev, dev_id);
-	if (err)
-		return err;
+	u16 reg16;
 
 	if (!bridge)
 		return -EOPNOTSUPP;
@@ -646,17 +508,14 @@ static int mlx5_pci_link_toggle_pf(struct mlx5_core_dev *dev)
 	err = pcie_capability_set_word(bridge, PCI_EXP_LNKCTL, PCI_EXP_LNKCTL_LD);
 	if (err)
 		return pcibios_err_to_errno(err);
-	msleep(2500);
+	msleep(500);
 	err = pcie_capability_clear_word(bridge, PCI_EXP_LNKCTL, PCI_EXP_LNKCTL_LD);
 	if (err)
-		return err;
+		return pcibios_err_to_errno(err);
 
 	/* Check link */
-	err = pci_read_config_dword(bridge, cap + PCI_EXP_LNKCAP, &reg32);
-	if (err)
-		return pcibios_err_to_errno(err);
-	if (!(reg32 & PCI_EXP_LNKCAP_DLLLARC)) {
-		mlx5_core_warn(dev, "No PCI link reporting capability (0x%08x)\n", reg32);
+	if (!bridge->link_active_reporting) {
+		mlx5_core_warn(dev, "No PCI link reporting capability\n");
 		msleep(1000);
 		goto restore;
 	}
@@ -691,6 +550,7 @@ static int mlx5_pci_link_toggle_pf(struct mlx5_core_dev *dev)
 
 	if (reg16 == dev_id) {
 		mlx5_core_info(dev, "Firmware responds to PCI config cycles again\n");
+		msleep(200);
 	} else {
 		mlx5_core_err(dev, "Firmware is not responsive (0x%04x) after %llu ms\n",
 			      reg16, mlx5_tout_ms(dev, PCI_TOGGLE));
@@ -698,20 +558,81 @@ static int mlx5_pci_link_toggle_pf(struct mlx5_core_dev *dev)
 	}
 
 restore:
-	list_for_each_entry(sdev, &bridge_bus->devices, bus_list) {
-		pci_cfg_access_unlock(sdev);
-		pci_restore_state(sdev);
-	}
+	timeout = jiffies + msecs_to_jiffies(mlx5_tout_ms(dev, PCI_TOGGLE));
 
+	list_for_each_entry(sdev, &bridge_bus->devices, bus_list) {
+		u64 reg64, reg64_comp;
+		int retry_ctr = 0;
+
+		pci_cfg_access_unlock(sdev);
+		reg64_comp = ((u64)sdev->saved_config_space[5] << 32) |
+			      ((u64)sdev->saved_config_space[4] & 0xfff00000);
+		do {
+			msleep(10);
+			sdev->state_saved = true;
+			pci_restore_state(sdev);
+			err = pci_read_config_dword(sdev, 0x14, &reg32);
+			if (err)
+				return pcibios_err_to_errno(err);
+			reg64 = (u64)reg32 << 32;
+			err = pci_read_config_dword(sdev, 0x10, &reg32);
+			if (err)
+				return pcibios_err_to_errno(err);
+			reg64 |= (u64)reg32 & 0xfff00000;
+			retry_ctr++;
+		} while ((reg64_comp != reg64) && !time_after(jiffies, timeout));
+		if (1 != retry_ctr)
+			mlx5_core_warn(dev, "devfn=0x%.8x restore had to retry %d times\n",
+				       sdev->devfn, retry_ctr);
+		if (reg64_comp != reg64) {
+			mlx5_core_err(dev, "devfn=0x%.8x BAR not set after %llu ms\n",
+				      sdev->devfn, mlx5_tout_ms(dev, PCI_TOGGLE));
+			err = -ETIMEDOUT;
+			break;
+		}
+	}
 	return err;
 }
 
-static int mlx5_pci_link_toggle(struct mlx5_core_dev *dev)
+static int mlx5_pci_reset_bus(struct mlx5_core_dev *dev)
 {
+	if (!MLX5_CAP_GEN(dev, pcie_reset_using_hotreset_method))
+		return -EOPNOTSUPP;
+
+	return pci_reset_bus(dev->pdev);
+}
+
+static int mlx5_sync_pci_reset(struct mlx5_core_dev *dev, u8 reset_method)
+{
+	u16 dev_id;
+	int err;
+
 	if (mlx5_core_is_ecpf(dev))
-		return mlx5_pci_link_toggle_ecpf(dev);
-	else
-		return mlx5_pci_link_toggle_pf(dev);
+		return -ENOTSUPP;
+
+	err = pci_read_config_word(dev->pdev, PCI_DEVICE_ID, &dev_id);
+	if (err)
+		return pcibios_err_to_errno(err);
+	err = mlx5_check_dev_ids(dev, dev_id);
+	if (err)
+		return err;
+
+	switch (reset_method) {
+	case MLX5_MFRL_REG_PCI_RESET_METHOD_LINK_TOGGLE:
+		err = mlx5_pci_link_toggle(dev, dev_id);
+		if (err)
+			mlx5_core_warn(dev, "mlx5_pci_link_toggle failed\n");
+		break;
+	case MLX5_MFRL_REG_PCI_RESET_METHOD_HOT_RESET:
+		err = mlx5_pci_reset_bus(dev);
+		if (err)
+			mlx5_core_warn(dev, "mlx5_pci_reset_bus failed\n");
+		break;
+	default:
+		return -EOPNOTSUPP;
+	}
+
+	return err;
 }
 
 static void mlx5_sync_reset_now_event(struct work_struct *work)
@@ -733,9 +654,9 @@ static void mlx5_sync_reset_now_event(struct work_struct *work)
 		goto done;
 	}
 
-	err = mlx5_pci_link_toggle(dev);
+	err = mlx5_sync_pci_reset(dev, fw_reset->reset_method);
 	if (err) {
-		mlx5_core_warn(dev, "mlx5_pci_link_toggle failed, no reset done, err %d\n", err);
+		mlx5_core_warn(dev, "mlx5_sync_pci_reset failed, no reset done, err %d\n", err);
 		set_bit(MLX5_FW_RESET_FLAGS_RELOAD_REQUIRED, &fw_reset->reset_flags);
 	}
 
@@ -750,6 +671,7 @@ static void mlx5_sync_reset_unload_event(struct work_struct *work)
 	struct mlx5_fw_reset *fw_reset;
 	struct mlx5_core_dev *dev;
 	unsigned long timeout;
+	int poll_freq = 20;
 	bool reset_action;
 	u8 rst_state;
 	int err;
@@ -757,7 +679,6 @@ static void mlx5_sync_reset_unload_event(struct work_struct *work)
 	fw_reset = container_of(work, struct mlx5_fw_reset, reset_unload_work);
 	dev = fw_reset->dev;
 
-	cancel_delayed_work(&fw_reset->reset_timeout_work);
 	if (mlx5_sync_reset_clear_reset_requested(dev, false))
 		return;
 
@@ -786,7 +707,12 @@ static void mlx5_sync_reset_unload_event(struct work_struct *work)
 			reset_action = true;
 			break;
 		}
-		msleep(20);
+		if (rst_state == MLX5_FW_RST_STATE_DROP_MODE) {
+			mlx5_core_info(dev, "Sync Reset Drop mode ack\n");
+			mlx5_set_fw_rst_ack(dev);
+			poll_freq = 1000;
+		}
+		msleep(poll_freq);
 	} while (!time_after(jiffies, timeout));
 
 	if (!reset_action) {
@@ -798,9 +724,9 @@ static void mlx5_sync_reset_unload_event(struct work_struct *work)
 
 	mlx5_core_warn(dev, "Sync Reset, got reset action. rst_state = %u\n", rst_state);
 	if (rst_state == MLX5_FW_RST_STATE_TOGGLE_REQ) {
-		err = mlx5_pci_link_toggle(dev);
+		err = mlx5_sync_pci_reset(dev, fw_reset->reset_method);
 		if (err) {
-			mlx5_core_warn(dev, "mlx5_pci_link_toggle failed, err %d\n", err);
+			mlx5_core_warn(dev, "mlx5_sync_pci_reset failed, err %d\n", err);
 			fw_reset->ret = err;
 		}
 	}
@@ -909,19 +835,30 @@ void mlx5_fw_reset_events_start(struct mlx5_core_dev *dev)
 {
 	struct mlx5_fw_reset *fw_reset = dev->priv.fw_reset;
 
+	if (!fw_reset)
+		return;
+
 	MLX5_NB_INIT(&fw_reset->nb, fw_reset_event_notifier, GENERAL_EVENT);
 	mlx5_eq_notifier_register(dev, &fw_reset->nb);
 }
 
 void mlx5_fw_reset_events_stop(struct mlx5_core_dev *dev)
 {
-	mlx5_eq_notifier_unregister(dev, &dev->priv.fw_reset->nb);
+	struct mlx5_fw_reset *fw_reset = dev->priv.fw_reset;
+
+	if (!fw_reset)
+		return;
+
+	mlx5_eq_notifier_unregister(dev, &fw_reset->nb);
 }
 
 void mlx5_drain_fw_reset(struct mlx5_core_dev *dev)
 {
 	struct mlx5_fw_reset *fw_reset = dev->priv.fw_reset;
 
+	if (!fw_reset)
+		return;
+
 	set_bit(MLX5_FW_RESET_FLAGS_DROP_NEW_REQUESTS, &fw_reset->reset_flags);
 	cancel_work_sync(&fw_reset->fw_live_patch_work);
 	cancel_work_sync(&fw_reset->reset_request_work);
@@ -940,9 +877,13 @@ static const struct devlink_param mlx5_fw_reset_devlink_params[] = {
 
 int mlx5_fw_reset_init(struct mlx5_core_dev *dev)
 {
-	struct mlx5_fw_reset *fw_reset = kzalloc(sizeof(*fw_reset), GFP_KERNEL);
+	struct mlx5_fw_reset *fw_reset;
 	int err;
 
+	if (!MLX5_CAP_MCAM_REG(dev, mfrl))
+		return 0;
+
+	fw_reset = kzalloc(sizeof(*fw_reset), GFP_KERNEL);
 	if (!fw_reset)
 		return -ENOMEM;
 	fw_reset->wq = create_singlethread_workqueue("mlx5_fw_reset_events");
@@ -979,6 +920,9 @@ void mlx5_fw_reset_cleanup(struct mlx5_core_dev *dev)
 {
 	struct mlx5_fw_reset *fw_reset = dev->priv.fw_reset;
 
+	if (!fw_reset)
+		return;
+
 	devl_params_unregister(priv_to_devlink(dev),
 			       mlx5_fw_reset_devlink_params,
 			       ARRAY_SIZE(mlx5_fw_reset_devlink_params));
