@@ -3,10 +3,16 @@
  * Copyright (c) 2011-2014, Intel Corporation.
  * Copyright (c) 2017-2021 Christoph Hellwig.
  */
+#ifdef HAVE_BIO_INTEGRITY_H
+#include <linux/bio-integrity.h>
+#endif
+#ifdef HAVE_BLK_INTEGRITY_H
+#include <linux/blk-integrity.h>
+#endif
 #include <linux/ptrace.h>	/* for force_successful_syscall_return */
 #include <linux/nvme_ioctl.h>
-#ifdef HAVE_IO_URING_H
-#include <linux/io_uring.h>
+#ifdef HAVE_IO_URING_CMD_H
+#include <linux/io_uring/cmd.h>
 #endif
 #include "nvme.h"
 
@@ -24,16 +30,13 @@ static bool nvme_cmd_allowed(struct nvme_ns *ns, struct nvme_command *c,
 {
 	u32 effects;
 
-	if (capable(CAP_SYS_ADMIN))
-		return true;
-
 #ifdef HAVE_BDEV_IS_PARTITION
 	/*
 	 * Do not allow unprivileged passthrough on partitions, as that allows an
 	 * escape from the containment of the partition.
 	 */
 	if (flags & NVME_IOCTL_PARTITION)
-		return false;
+		goto admin;
 #endif
 
 	/*
@@ -42,7 +45,7 @@ static bool nvme_cmd_allowed(struct nvme_ns *ns, struct nvme_command *c,
 	 */
 	if (c->common.opcode >= nvme_cmd_vendor_start ||
 	    c->common.opcode == nvme_fabrics_command)
-		return false;
+		goto admin;
 
 	/*
 	 * Do not allow unprivileged passthrough of admin commands except
@@ -61,7 +64,7 @@ static bool nvme_cmd_allowed(struct nvme_ns *ns, struct nvme_command *c,
 				return true;
 			}
 		}
-		return false;
+		goto admin;
 	}
 
 	/*
@@ -71,7 +74,7 @@ static bool nvme_cmd_allowed(struct nvme_ns *ns, struct nvme_command *c,
 	 */
 	effects = nvme_command_effects(ns->ctrl, ns, c->common.opcode);
 	if (!(effects & NVME_CMD_EFFECTS_CSUPP))
-		return false;
+		goto admin;
 
 	/*
 	 * Don't allow passthrough for command that have intrusive (or unknown)
@@ -80,16 +83,20 @@ static bool nvme_cmd_allowed(struct nvme_ns *ns, struct nvme_command *c,
 	if (effects & ~(NVME_CMD_EFFECTS_CSUPP | NVME_CMD_EFFECTS_LBCC |
 			NVME_CMD_EFFECTS_UUID_SEL |
 			NVME_CMD_EFFECTS_SCOPE_MASK))
-		return false;
+		goto admin;
 
 	/*
 	 * Only allow I/O commands that transfer data to the controller or that
 	 * change the logical block contents if the file descriptor is open for
 	 * writing.
 	 */
-	if (nvme_is_write(c) || (effects & NVME_CMD_EFFECTS_LBCC))
-		return open_for_write;
+	if ((nvme_is_write(c) || (effects & NVME_CMD_EFFECTS_LBCC)) &&
+	    !open_for_write)
+		goto admin;
+
 	return true;
+admin:
+	return capable(CAP_SYS_ADMIN);
 }
 
 /*
@@ -106,74 +113,64 @@ static void __user *nvme_to_user_ptr(uintptr_t ptrval)
 	return (void __user *)ptrval;
 }
 
-
+#ifndef HAVE_BIO_INTEGRITY_MAP_USER
 static void *nvme_add_user_metadata(struct request *req, void __user *ubuf,
-#if defined HAVE_BLK_TYPES_REQ_INTEGRITY && defined HAVE_REQ_OPF_REQ_OP_DRV_OUT
-		unsigned len, u32 seed)
-#else
-		unsigned len, u32 seed, bool write)
-#endif
+	       unsigned len, u32 seed)
 {
-	struct bio_integrity_payload *bip;
-	int ret = -ENOMEM;
-	void *buf;
-	struct bio *bio = req->bio;
+       struct bio_integrity_payload *bip;
+       int ret = -ENOMEM;
+       void *buf;
+       struct bio *bio = req->bio;
 
-	buf = kmalloc(len, GFP_KERNEL);
-	if (!buf)
-		goto out;
+       buf = kmalloc(len, GFP_KERNEL);
+       if (!buf)
+	       goto out;
 
-	ret = -EFAULT;
-#if defined HAVE_BLK_TYPES_REQ_INTEGRITY && defined HAVE_REQ_OPF_REQ_OP_DRV_OUT
-	if ((req_op(req) == REQ_OP_DRV_OUT) && copy_from_user(buf, ubuf, len))
-#else
-	if (write && copy_from_user(buf, ubuf, len))
-#endif
-		goto out_free_meta;
+       if (req_op(req) == REQ_OP_DRV_OUT) {
+	       ret = -EFAULT;
+	       if (copy_from_user(buf, ubuf, len))
+		       goto out_free_meta;
+       } else {
+	       memset(buf, 0, len);
+       }
 
-	bip = bio_integrity_alloc(bio, GFP_KERNEL, 1);
-	if (IS_ERR(bip)) {
-		ret = PTR_ERR(bip);
-		goto out_free_meta;
-	}
+       bip = bio_integrity_alloc(bio, GFP_KERNEL, 1);
+       if (IS_ERR(bip)) {
+	       ret = PTR_ERR(bip);
+	       goto out_free_meta;
+       }
 
-#ifdef HAVE_BIO_INTEGRITY_PYLD_BIP_ITER
+/* Using 5e70d0acf082 instead of 80814b8e359 as it has no new symbols */
+#ifndef HAVE_PCIE_CAPABILITY_CLEAR_AND_SET_WORD_LOCKED
 	bip->bip_iter.bi_size = len;
-	bip->bip_iter.bi_sector = seed;
-#endif
-	ret = bio_integrity_add_page(bio, virt_to_page(buf), len,
-			offset_in_page(buf));
-#if defined HAVE_BLK_TYPES_REQ_INTEGRITY && defined HAVE_REQ_OPF_REQ_OP_DRV_OUT
-	if (ret != len) {
-		ret = -ENOMEM;
-		goto out_free_meta;
-	}
-
-	req->cmd_flags |= REQ_INTEGRITY;
-	return buf;
-#else
-	if (ret == len)
-		return buf;
-	ret = -ENOMEM;
 #endif
+       bip->bip_iter.bi_sector = seed;
+       ret = bio_integrity_add_page(bio, virt_to_page(buf), len,
+		       offset_in_page(buf));
+       if (ret != len) {
+	       ret = -ENOMEM;
+	       goto out_free_meta;
+       }
 
+       req->cmd_flags |= REQ_INTEGRITY;
+       return buf;
 out_free_meta:
-	kfree(buf);
+       kfree(buf);
 out:
-	return ERR_PTR(ret);
+       return ERR_PTR(ret);
 }
 
-#ifdef HAVE_BLK_TYPES_REQ_OP_DRV_OUT
 static int nvme_finish_user_metadata(struct request *req, void __user *ubuf,
-		void *meta, unsigned len, int ret)
+	       void *meta, unsigned len, int ret)
 {
-	if (!ret && req_op(req) == REQ_OP_DRV_IN &&
-	    copy_to_user(ubuf, meta, len))
-		ret = -EFAULT;
-	kfree(meta);
-	return ret;
+       if (!ret && req_op(req) == REQ_OP_DRV_IN &&
+	   copy_to_user(ubuf, meta, len))
+	       ret = -EFAULT;
+       kfree(meta);
+       return ret;
 }
 #endif
+
 static struct request *nvme_alloc_user_request(struct request_queue *q,
 		struct nvme_command *cmd,
 #ifdef HAVE_BLK_OPF_T
@@ -184,40 +181,25 @@ static struct request *nvme_alloc_user_request(struct request_queue *q,
 {
 	struct request *req;
 
-#ifdef HAVE_BLK_TYPES_REQ_OP_DRV_OUT
 	req = blk_mq_alloc_request(q, nvme_req_op(cmd) | rq_flags, blk_flags);
 	if (IS_ERR(req))
 		return req;
 	nvme_init_request(req, cmd);
-#else
-#ifdef HAVE_BLK_MQ_ALLOC_REQUEST_HAS_3_PARAMS
-	req = nvme_alloc_request(q, cmd, 0);
-#else
-	req = nvme_alloc_request(q, cmd, GFP_KERNEL, false);
-#endif
-	if (IS_ERR(req))
-		return req;
-#endif
 	nvme_req(req)->flags |= NVME_REQ_USERCMD;
 	return req;
 }
 
 static int nvme_map_user_request(struct request *req, u64 ubuffer,
 		unsigned bufflen, void __user *meta_buffer, unsigned meta_len,
-#ifdef HAVE_IO_URING_CMD
-		u32 meta_seed, void **metap, struct io_uring_cmd *ioucmd,
-#else
-		u32 meta_seed, void **metap,
+		u32 meta_seed,
+#ifdef HAVE_IO_URING_CMD_H
+		struct io_uring_cmd *ioucmd,
+#endif
+#ifndef HAVE_BIO_INTEGRITY_MAP_USER
+		void **metap,
 #endif
-#if defined HAVE_BLK_TYPES_REQ_INTEGRITY && defined HAVE_REQ_OPF_REQ_OP_DRV_OUT
 		unsigned int flags)
-#else
-		unsigned int flags, struct nvme_command *cmd)
-#endif
 {
-#if !(defined HAVE_BLK_TYPES_REQ_INTEGRITY && defined HAVE_REQ_OPF_REQ_OP_DRV_OUT)
-	bool write = nvme_is_write(cmd);
-#endif
 	struct request_queue *q = req->q;
 	struct nvme_ns *ns = q->queuedata;
 #ifdef HAVE_ENUM_BIO_REMAPPED
@@ -226,12 +208,14 @@ static int nvme_map_user_request(struct request *req, u64 ubuffer,
 	struct gendisk *disk = ns ? ns->disk : NULL;
 #endif
 	struct bio *bio = NULL;
+#ifndef HAVE_BIO_INTEGRITY_MAP_USER
 	void *meta = NULL;
+#endif
 	int ret;
 
 #ifdef HAVE_NVME_IOCTL_IO64_CMD_VEC
 #ifdef HAVE_BLK_RQ_MAP_USER_IO
-#ifdef HAVE_IO_URING_CMD
+#ifdef HAVE_IO_URING_CMD_H
 	if (ioucmd && (ioucmd->flags & IORING_URING_CMD_FIXED)) {
 		struct iov_iter iter;
 
@@ -250,7 +234,7 @@ static int nvme_map_user_request(struct request *req, u64 ubuffer,
 	}
 #else
 	ret = blk_rq_map_user_io(req, NULL, nvme_to_user_ptr(ubuffer), bufflen,
-			GFP_KERNEL, vec, 0, 0, rq_data_dir(req));
+			GFP_KERNEL, flags & NVME_IOCTL_VEC, 0, 0, rq_data_dir(req));
 #endif
 #else
 	if (!((flags & NVME_IOCTL_VEC)))
@@ -269,39 +253,59 @@ static int nvme_map_user_request(struct request *req, u64 ubuffer,
 				GFP_KERNEL);
 		kfree(iov);
 	}
-#endif
+#endif /* HAVE_BLK_RQ_MAP_USER_IO */
 #else
 	ret = blk_rq_map_user(q, req, NULL, nvme_to_user_ptr(ubuffer), bufflen,
 			GFP_KERNEL);
-#endif
+#endif /* HAVE_NVME_IOCTL_IO64_CMD_VEC */
 
 	if (ret)
 		goto out;
+
 	bio = req->bio;
+#ifdef HAVE_BIO_INTEGRITY_MAP_USER
+	if (bdev) {
+		bio_set_dev(bio, bdev);
+		if (meta_buffer && meta_len) {
+#ifdef HAVE_BLK_RQ_INTEGRITY_MAP_USER
+#ifdef HAVE_BLK_RQ_INTEGRITY_MAP_USER_GET_3_PARAM
+			ret = blk_rq_integrity_map_user(req, meta_buffer, meta_len);
+#else
+			ret = blk_rq_integrity_map_user(req, meta_buffer, meta_len,
+							meta_seed);
+#endif
+#else
+			ret = bio_integrity_map_user(bio, meta_buffer, meta_len,
+						     meta_seed);
+#endif
+			if (ret)
+				goto out_unmap;
+#ifndef HAVE_BLK_RQ_INTEGRITY_MAP_USER
+			req->cmd_flags |= REQ_INTEGRITY;
+#endif
+		}
+	}
+#else
 #ifdef HAVE_BIO_BI_DISK
-		bio->bi_disk = disk;
-		if (disk && meta_buffer && meta_len) {
+	bio->bi_disk = disk;
+	if (disk && meta_buffer && meta_len) {
 #elif defined HAVE_ENUM_BIO_REMAPPED
 	if (bdev)
 		bio_set_dev(bio, bdev);
 
 	if (bdev && meta_buffer && meta_len) {
 #else
-		if (disk) {
-			bio->bi_bdev = bdget_disk(disk, 0);
-			if (!bio->bi_bdev) {
-				ret = -ENODEV;
-				goto out_unmap;
-			}
+	if (disk) {
+		bio->bi_bdev = bdget_disk(disk, 0);
+		if (!bio->bi_bdev) {
+			ret = -ENODEV;
+			goto out_unmap;
 		}
-		if (disk && meta_buffer && meta_len) {
-#endif
+	}
+	if (disk && meta_buffer && meta_len) {
+#endif /* HAVE_BIO_BI_DISK */
 		meta = nvme_add_user_metadata(req, meta_buffer, meta_len,
-#if defined HAVE_BLK_TYPES_REQ_INTEGRITY && defined HAVE_REQ_OPF_REQ_OP_DRV_OUT
 					meta_seed);
-#else
-					meta_seed, write);
-#endif
 
 		if (IS_ERR(meta)) {
 			ret = PTR_ERR(meta);
@@ -309,6 +313,7 @@ static int nvme_map_user_request(struct request *req, u64 ubuffer,
 		}
 		*metap = meta;
 	}
+#endif /* HAVE_BIO_INTEGRITY_MAP_USER */
 
 	return ret;
 
@@ -330,7 +335,9 @@ static int nvme_submit_user_cmd(struct request_queue *q,
 #endif
 	struct nvme_ctrl *ctrl;
 	struct request *req;
+#ifndef HAVE_BIO_INTEGRITY_MAP_USER
 	void *meta = NULL;
+#endif
 	struct bio *bio;
 #if defined(HAVE_BLK_EXECUTE_RQ_2_PARAM) || defined(HAVE_BLK_EXECUTE_RQ_3_PARAM)
 	u32 effects;
@@ -344,16 +351,15 @@ static int nvme_submit_user_cmd(struct request_queue *q,
 	req->timeout = timeout;
 	if (ubuffer && bufflen) {
 		ret = nvme_map_user_request(req, ubuffer, bufflen, meta_buffer,
-#ifdef HAVE_IO_URING_CMD
-				meta_len, meta_seed, &meta, NULL,
-#else
-				meta_len, meta_seed, &meta,
+				meta_len, meta_seed,
+#ifdef HAVE_IO_URING_CMD_H
+				NULL,
+#endif
+#ifndef HAVE_BIO_INTEGRITY_MAP_USER
+				&meta,
 #endif
-#if defined HAVE_BLK_TYPES_REQ_INTEGRITY && defined HAVE_REQ_OPF_REQ_OP_DRV_OUT
 				flags);
-#else
-				flags, cmd);
-#endif
+
 		if (ret)
 			return ret;
 	}
@@ -377,18 +383,12 @@ static int nvme_submit_user_cmd(struct request_queue *q,
 #endif
 	if (result)
 		*result = le64_to_cpu(nvme_req(req)->result.u64);
+#ifndef HAVE_BIO_INTEGRITY_MAP_USER
 	if (meta) {
-#ifdef HAVE_BLK_TYPES_REQ_OP_DRV_OUT
 		ret = nvme_finish_user_metadata(req, meta_buffer, meta,
 						meta_len, ret);
-#else
-		if (!ret && ! nvme_is_write(cmd)) {
-			if (copy_to_user(meta_buffer, meta, meta_len))
-				ret = -EFAULT;
-		}
-		kfree(meta);
-#endif
 	}
+#endif
 #if defined HAVE_BIO_BI_DISK || defined HAVE_ENUM_BIO_REMAPPED
 	if (bio)
 		blk_rq_unmap_user(bio);
@@ -401,11 +401,12 @@ static int nvme_submit_user_cmd(struct request_queue *q,
 		blk_rq_unmap_user(bio);
 	}
 #endif
+	blk_mq_free_request(req);
+
 #if defined(HAVE_BLK_EXECUTE_RQ_2_PARAM) || defined(HAVE_BLK_EXECUTE_RQ_3_PARAM)
 	if (effects)
 		nvme_passthru_end(ctrl, ns, effects, cmd, ret);
 #endif
-	blk_mq_free_request(req);
 
 	return ret;
 }
@@ -431,10 +432,10 @@ static int nvme_submit_io(struct nvme_ns *ns, struct nvme_user_io __user *uio)
 		return -EINVAL;
 	}
 
-	length = (io.nblocks + 1) << ns->lba_shift;
+	length = (io.nblocks + 1) << ns->head->lba_shift;
 
 	if ((io.control & NVME_RW_PRINFO_PRACT) &&
-	    ns->ms == sizeof(struct t10_pi_tuple)) {
+	    (ns->head->ms == ns->head->pi_size)) {
 		/*
 		 * Protection information is stripped/inserted by the
 		 * controller.
@@ -444,11 +445,11 @@ static int nvme_submit_io(struct nvme_ns *ns, struct nvme_user_io __user *uio)
 		meta_len = 0;
 		metadata = NULL;
 	} else {
-		meta_len = (io.nblocks + 1) * ns->ms;
+		meta_len = (io.nblocks + 1) * ns->head->ms;
 		metadata = nvme_to_user_ptr(io.metadata);
 	}
 
-	if (ns->features & NVME_NS_EXT_LBAS) {
+	if (ns->head->features & NVME_NS_EXT_LBAS) {
 		length += meta_len;
 		meta_len = 0;
 	} else if (meta_len) {
@@ -493,7 +494,7 @@ static int nvme_user_cmd(struct nvme_ctrl *ctrl, struct nvme_ns *ns,
 	struct nvme_passthru_cmd cmd;
 	struct nvme_command c;
 	unsigned timeout = 0;
-	u64 result = 0;
+	u64 result;
 	int status;
 
 	if (copy_from_user(&cmd, ucmd, sizeof(cmd)))
@@ -597,25 +598,16 @@ struct nvme_uring_data {
 	__u32	timeout_ms;
 };
 
+#if defined(HAVE_IO_URING_CMD_H) && defined(HAVE_BIO_INTEGRITY_MAP_USER)
 /*
  * This overlays struct io_uring_cmd pdu.
  * Expect build errors if this grows larger than that.
  */
-#ifdef HAVE_FILE_OPERATIONS_URING_CMD
 struct nvme_uring_cmd_pdu {
-	union {
-		struct bio *bio;
-		struct request *req;
-	};
-	u32 meta_len;
-	u32 nvme_status;
-	union {
-		struct {
-			void *meta; /* kernel-resident buffer */
-			void __user *meta_buffer;
-		};
-		u64 result;
-	} u;
+	struct request *req;
+	struct bio *bio;
+	u64 result;
+	int status;
 };
 
 static inline struct nvme_uring_cmd_pdu *nvme_uring_cmd_pdu(
@@ -624,156 +616,51 @@ static inline struct nvme_uring_cmd_pdu *nvme_uring_cmd_pdu(
 	return (struct nvme_uring_cmd_pdu *)&ioucmd->pdu;
 }
 
-#ifdef HAVE_IO_URING_CMD_DONE_4_PARAMS
-static void nvme_uring_task_meta_cb(struct io_uring_cmd *ioucmd,
-				    unsigned issue_flags)
-#else
-static void nvme_uring_task_meta_cb(struct io_uring_cmd *ioucmd)
-#endif
-{
-	struct nvme_uring_cmd_pdu *pdu = nvme_uring_cmd_pdu(ioucmd);
-	struct request *req = pdu->req;
-	int status;
-	u64 result;
-
-	if (nvme_req(req)->flags & NVME_REQ_CANCELLED)
-		status = -EINTR;
-	else
-		status = nvme_req(req)->status;
-
-	result = le64_to_cpu(nvme_req(req)->result.u64);
-
-	if (pdu->meta_len)
-		status = nvme_finish_user_metadata(req, pdu->u.meta_buffer,
-					pdu->u.meta, pdu->meta_len, status);
-	if (req->bio)
-		blk_rq_unmap_user(req->bio);
-	blk_mq_free_request(req);
-#ifdef HAVE_IO_URING_CMD_DONE_4_PARAMS
-	io_uring_cmd_done(ioucmd, status, result, issue_flags);
-#else
-	io_uring_cmd_done(ioucmd, status, result);
-#endif
-}
-
-#ifdef HAVE_IO_URING_CMD_DONE_4_PARAMS
 static void nvme_uring_task_cb(struct io_uring_cmd *ioucmd,
 			       unsigned issue_flags)
-#else
-static void nvme_uring_task_cb(struct io_uring_cmd *ioucmd)
-#endif
 {
 	struct nvme_uring_cmd_pdu *pdu = nvme_uring_cmd_pdu(ioucmd);
 
 	if (pdu->bio)
 		blk_rq_unmap_user(pdu->bio);
-
-#ifdef HAVE_IO_URING_CMD_DONE_4_PARAMS
-	io_uring_cmd_done(ioucmd, pdu->nvme_status, pdu->u.result, issue_flags);
-#else
-	io_uring_cmd_done(ioucmd, pdu->nvme_status, pdu->u.result);
-#endif
+	io_uring_cmd_done(ioucmd, pdu->status, pdu->result, issue_flags);
 }
 
-#ifdef HAVE_RQ_END_IO_RET
 static enum rq_end_io_ret nvme_uring_cmd_end_io(struct request *req,
 						blk_status_t err)
-#else
-static void nvme_uring_cmd_end_io(struct request *req, blk_status_t err)
-#endif
 {
 	struct io_uring_cmd *ioucmd = req->end_io_data;
 	struct nvme_uring_cmd_pdu *pdu = nvme_uring_cmd_pdu(ioucmd);
-#ifdef HAVE_FILE_OPERATIONS_URING_CMD_IOPOLL
-	void *cookie = READ_ONCE(ioucmd->cookie);
-#endif
 
-	req->bio = pdu->bio;
 	if (nvme_req(req)->flags & NVME_REQ_CANCELLED)
-		pdu->nvme_status = -EINTR;
+		pdu->status = -EINTR;
 	else
-		pdu->nvme_status = nvme_req(req)->status;
-	pdu->u.result = le64_to_cpu(nvme_req(req)->result.u64);
+		pdu->status = nvme_req(req)->status;
+	pdu->result = le64_to_cpu(nvme_req(req)->result.u64);
 
-#ifdef HAVE_FILE_OPERATIONS_URING_CMD_IOPOLL
 	/*
 	 * For iopoll, complete it directly.
 	 * Otherwise, move the completion to task work.
 	 */
-	if (cookie != NULL && blk_rq_is_poll(req))
-#ifdef HAVE_IO_URING_CMD_DONE_4_PARAMS
+	if (blk_rq_is_poll(req))
 		nvme_uring_task_cb(ioucmd, IO_URING_F_UNLOCKED);
-#else
-		nvme_uring_task_cb(ioucmd);
-#endif
 	else
-		io_uring_cmd_complete_in_task(ioucmd, nvme_uring_task_cb);
-#else
-	io_uring_cmd_complete_in_task(ioucmd, nvme_uring_task_cb);
-#endif
+		io_uring_cmd_do_in_task_lazy(ioucmd, nvme_uring_task_cb);
 
-#ifdef HAVE_RQ_END_IO_RET
 	return RQ_END_IO_FREE;
-#endif
-}
-
-#ifdef HAVE_RQ_END_IO_RET
-static enum rq_end_io_ret nvme_uring_cmd_end_io_meta(struct request *req,
-#else
-static void nvme_uring_cmd_end_io_meta(struct request *req,
-#endif
-						     blk_status_t err)
-{
-	struct io_uring_cmd *ioucmd = req->end_io_data;
-	struct nvme_uring_cmd_pdu *pdu = nvme_uring_cmd_pdu(ioucmd);
-#ifdef HAVE_FILE_OPERATIONS_URING_CMD_IOPOLL
-	void *cookie = READ_ONCE(ioucmd->cookie);
-#endif
-
-	req->bio = pdu->bio;
-	pdu->req = req;
-
-#ifdef HAVE_FILE_OPERATIONS_URING_CMD_IOPOLL
-	/*
-	 * For iopoll, complete it directly.
-	 * Otherwise, move the completion to task work.
-	 */
-	if (cookie != NULL && blk_rq_is_poll(req))
-#ifdef HAVE_IO_URING_CMD_DONE_4_PARAMS
-		nvme_uring_task_meta_cb(ioucmd, IO_URING_F_UNLOCKED);
-#else
-		nvme_uring_task_meta_cb(ioucmd);
-#endif
-	else
-		io_uring_cmd_complete_in_task(ioucmd, nvme_uring_task_meta_cb);
-#else
-		io_uring_cmd_complete_in_task(ioucmd, nvme_uring_task_meta_cb);
-#endif
-#ifdef HAVE_RQ_END_IO_RET
-	return RQ_END_IO_NONE;
-#endif
 }
 
 static int nvme_uring_cmd_io(struct nvme_ctrl *ctrl, struct nvme_ns *ns,
 		struct io_uring_cmd *ioucmd, unsigned int issue_flags, bool vec)
 {
 	struct nvme_uring_cmd_pdu *pdu = nvme_uring_cmd_pdu(ioucmd);
-#ifdef HAVE_IO_URING_SQE_CMD
 	const struct nvme_uring_cmd *cmd = io_uring_sqe_cmd(ioucmd->sqe);
-#else
-	const struct nvme_uring_cmd *cmd = ioucmd->cmd;
-#endif
 	struct request_queue *q = ns ? ns->queue : ctrl->admin_q;
 	struct nvme_uring_data d;
 	struct nvme_command c;
 	struct request *req;
-#ifdef HAVE_BLK_OPF_T
 	blk_opf_t rq_flags = REQ_ALLOC_CACHE;
-#else
-	unsigned int rq_flags = 0;
-#endif
 	blk_mq_req_flags_t blk_flags = 0;
-	void *meta = NULL;
 	int ret;
 
 	c.common.opcode = READ_ONCE(cmd->opcode);
@@ -814,12 +701,9 @@ static int nvme_uring_cmd_io(struct nvme_ctrl *ctrl, struct nvme_ns *ns,
 		rq_flags |= REQ_NOWAIT;
 		blk_flags = BLK_MQ_REQ_NOWAIT;
 	}
-#ifdef HAVE_FILE_OPERATIONS_URING_CMD_IOPOLL
 	if (issue_flags & IO_URING_F_IOPOLL)
 		rq_flags |= REQ_POLLED;
 
-retry:
-#endif
 	req = nvme_alloc_user_request(q, &c, rq_flags, blk_flags);
 	if (IS_ERR(req))
 		return PTR_ERR(req);
@@ -828,61 +712,20 @@ retry:
 	if (d.addr && d.data_len) {
 		ret = nvme_map_user_request(req, d.addr,
 			d.data_len, nvme_to_user_ptr(d.metadata),
-#ifdef HAVE_IO_URING_CMD
-			d.metadata_len, 0, &meta, ioucmd,
-#else
-			d.metadata_len, 0, &meta,
-#endif
-#if defined HAVE_BLK_TYPES_REQ_INTEGRITY && defined HAVE_REQ_OPF_REQ_OP_DRV_OUT
-				vec);
-#else
-				vec, &c);
-#endif
+			d.metadata_len, 0, ioucmd, vec);
 		if (ret)
 			return ret;
 	}
-#ifdef HAVE_FILE_OPERATIONS_URING_CMD_IOPOLL
-	if (issue_flags & IO_URING_F_IOPOLL && rq_flags & REQ_POLLED) {
-		if (unlikely(!req->bio)) {
-			/* we can't poll this, so alloc regular req instead */
-			blk_mq_free_request(req);
-			rq_flags &= ~REQ_POLLED;
-			goto retry;
-		} else {
-			WRITE_ONCE(ioucmd->cookie, req->bio);
-			req->bio->bi_opf |= REQ_POLLED;
-		}
-	}
-#endif
+
 	/* to free bio on completion, as req->bio will be null at that time */
 	pdu->bio = req->bio;
-	pdu->meta_len = d.metadata_len;
+	pdu->req = req;
 	req->end_io_data = ioucmd;
-	if (pdu->meta_len) {
-		pdu->u.meta = meta;
-		pdu->u.meta_buffer = nvme_to_user_ptr(d.metadata);
-		req->end_io = nvme_uring_cmd_end_io_meta;
-	} else {
-#ifdef HAVE_BLK_EXECUTE_RQ_NOWAIT_2_PARAM
-		req->end_io = nvme_uring_cmd_end_io;
-#endif
-	}
-#ifdef HAVE_BLK_EXECUTE_RQ_NOWAIT_2_PARAM
+	req->end_io = nvme_uring_cmd_end_io;
 	blk_execute_rq_nowait(req, false);
-#else
-#ifdef HAVE_BLK_EXECUTE_RQ_NOWAIT_5_PARAM
-	blk_execute_rq_nowait(req->q, NULL, req, 0, nvme_uring_cmd_end_io);
-#else
-#ifdef HAVE_BLK_EXECUTE_RQ_NOWAIT_3_PARAM
-	blk_execute_rq_nowait(req, 0, nvme_uring_cmd_end_io);
-#else
-	blk_execute_rq_nowait(NULL, req, 0, nvme_uring_cmd_end_io);
-#endif
-#endif
-#endif
 	return -EIOCBQUEUED;
 }
-#endif //HAVE_FILE_OPERATIONS_URING_CMD
+#endif // HAVE_IO_URING_CMD_H
 
 static bool is_ctrl_ioctl(unsigned int cmd)
 {
@@ -1001,14 +844,10 @@ long nvme_ns_chr_ioctl(struct file *file, unsigned int cmd, unsigned long arg)
 	return nvme_ns_ioctl(ns, cmd, argp, 0, open_for_write);
 }
 
-#ifdef HAVE_FILE_OPERATIONS_URING_CMD
+#if defined(HAVE_IO_URING_CMD_H) && defined(HAVE_BIO_INTEGRITY_MAP_USER)
 static int nvme_uring_cmd_checks(unsigned int issue_flags)
 {
-#ifndef HAVE_FILE_OPERATIONS_URING_CMD_IOPOLL
-	/* IOPOLL not supported yet */
-	if (issue_flags & IO_URING_F_IOPOLL)
-		return -EOPNOTSUPP;
-#endif
+
 	/* NVMe passthrough requires big SQE/CQE support */
 	if ((issue_flags & (IO_URING_F_SQE128|IO_URING_F_CQE32)) !=
 	    (IO_URING_F_SQE128|IO_URING_F_CQE32))
@@ -1049,27 +888,17 @@ int nvme_ns_chr_uring_cmd(struct io_uring_cmd *ioucmd, unsigned int issue_flags)
 
 	return nvme_ns_uring_cmd(ns, ioucmd, issue_flags);
 }
-#endif
 
-#ifdef HAVE_FILE_OPERATIONS_URING_CMD_IOPOLL
 int nvme_ns_chr_uring_cmd_iopoll(struct io_uring_cmd *ioucmd,
 				 struct io_comp_batch *iob,
 				 unsigned int poll_flags)
 {
-	struct bio *bio;
-	int ret = 0;
-	struct nvme_ns *ns;
-	struct request_queue *q;
+	struct nvme_uring_cmd_pdu *pdu = nvme_uring_cmd_pdu(ioucmd);
+	struct request *req = pdu->req;
 
-	rcu_read_lock();
-	bio = READ_ONCE(ioucmd->cookie);
-	ns = container_of(file_inode(ioucmd->file)->i_cdev,
-			struct nvme_ns, cdev);
-	q = ns->queue;
-	if (test_bit(QUEUE_FLAG_POLL, &q->queue_flags) && bio && bio->bi_bdev)
-		ret = bio_poll(bio, iob, poll_flags);
-	rcu_read_unlock();
-	return ret;
+	if (req && blk_rq_is_poll(req))
+		return blk_rq_poll(req, iob, poll_flags);
+	return 0;
 }
 #endif
 #ifdef CONFIG_NVME_MULTIPATH
@@ -1158,9 +987,8 @@ out_unlock:
 	srcu_read_unlock(&head->srcu, srcu_idx);
 	return ret;
 }
-#endif /* CONFIG_NVME_MULTIPATH */
 
-#ifdef HAVE_FILE_OPERATIONS_URING_CMD
+#if defined(HAVE_IO_URING_CMD_H) && defined(HAVE_BIO_INTEGRITY_MAP_USER)
 int nvme_ns_head_chr_uring_cmd(struct io_uring_cmd *ioucmd,
 		unsigned int issue_flags)
 {
@@ -1175,44 +1003,18 @@ int nvme_ns_head_chr_uring_cmd(struct io_uring_cmd *ioucmd,
 	srcu_read_unlock(&head->srcu, srcu_idx);
 	return ret;
 }
+#endif /* HAVE_IO_URING_CMD_H */
+#endif /* CONFIG_NVME_MULTIPATH */
 
-#ifdef HAVE_FILE_OPERATIONS_URING_CMD_IOPOLL
-int nvme_ns_head_chr_uring_cmd_iopoll(struct io_uring_cmd *ioucmd,
-				      struct io_comp_batch *iob,
-				      unsigned int poll_flags)
-{
-	struct cdev *cdev = file_inode(ioucmd->file)->i_cdev;
-	struct nvme_ns_head *head = container_of(cdev, struct nvme_ns_head, cdev);
-	int srcu_idx = srcu_read_lock(&head->srcu);
-	struct nvme_ns *ns = nvme_find_path(head);
-	struct bio *bio;
-	int ret = 0;
-	struct request_queue *q;
-
-	if (ns) {
-		rcu_read_lock();
-		bio = READ_ONCE(ioucmd->cookie);
-		q = ns->queue;
-		if (test_bit(QUEUE_FLAG_POLL, &q->queue_flags) && bio
-				&& bio->bi_bdev)
-			ret = bio_poll(bio, iob, poll_flags);
-		rcu_read_unlock();
-	}
-	srcu_read_unlock(&head->srcu, srcu_idx);
-	return ret;
-}
-#endif
-
+#if defined(HAVE_IO_URING_CMD_H) && defined(HAVE_BIO_INTEGRITY_MAP_USER)
 int nvme_dev_uring_cmd(struct io_uring_cmd *ioucmd, unsigned int issue_flags)
 {
 	struct nvme_ctrl *ctrl = ioucmd->file->private_data;
 	int ret;
 
-#ifdef HAVE_FILE_OPERATIONS_URING_CMD_IOPOLL
 	/* IOPOLL not supported yet */
 	if (issue_flags & IO_URING_F_IOPOLL)
 		return -EOPNOTSUPP;
-#endif
 
 	ret = nvme_uring_cmd_checks(issue_flags);
 	if (ret)
@@ -1231,21 +1033,21 @@ int nvme_dev_uring_cmd(struct io_uring_cmd *ioucmd, unsigned int issue_flags)
 
 	return ret;
 }
-#endif /* HAVE_FILE_OPERATIONS_URING_CMD */
+#endif /* HAVE_IO_URING_CMD_H */
 
 static int nvme_dev_user_cmd(struct nvme_ctrl *ctrl, void __user *argp,
 		bool open_for_write)
 {
 	struct nvme_ns *ns;
-	int ret;
+	int ret, srcu_idx;
 
-	down_read(&ctrl->namespaces_rwsem);
+	srcu_idx = srcu_read_lock(&ctrl->srcu);
 	if (list_empty(&ctrl->namespaces)) {
 		ret = -ENOTTY;
 		goto out_unlock;
 	}
 
-	ns = list_first_entry(&ctrl->namespaces, struct nvme_ns, list);
+	ns = list_first_or_null_rcu(&ctrl->namespaces, struct nvme_ns, list);
 	if (ns != list_last_entry(&ctrl->namespaces, struct nvme_ns, list)) {
 		dev_warn(ctrl->device,
 			"NVME_IOCTL_IO_CMD not supported when multiple namespaces present!\n");
@@ -1255,15 +1057,18 @@ static int nvme_dev_user_cmd(struct nvme_ctrl *ctrl, void __user *argp,
 
 	dev_warn(ctrl->device,
 		"using deprecated NVME_IOCTL_IO_CMD ioctl on the char device!\n");
-	kref_get(&ns->kref);
-	up_read(&ctrl->namespaces_rwsem);
+	if (!nvme_get_ns(ns)) {
+		ret = -ENXIO;
+		goto out_unlock;
+	}
+	srcu_read_unlock(&ctrl->srcu, srcu_idx);
 
 	ret = nvme_user_cmd(ctrl, ns, argp, 0, open_for_write);
 	nvme_put_ns(ns);
 	return ret;
 
 out_unlock:
-	up_read(&ctrl->namespaces_rwsem);
+	srcu_read_unlock(&ctrl->srcu, srcu_idx);
 	return ret;
 }
 
