@@ -3,7 +3,6 @@
  * Copyright (c) 2017-2018 Christoph Hellwig.
  */
 
-#ifdef HAVE_BLK_TYPES_REQ_DRV
 #include <linux/backing-dev.h>
 #include <linux/moduleparam.h>
 #include <linux/vmalloc.h>
@@ -22,11 +21,7 @@ static const char *nvme_iopolicy_names[] = {
 
 static int iopolicy = NVME_IOPOLICY_NUMA;
 
-#ifdef HAVE_CHECK_OLD_SET_PARAM
-static int nvme_set_iopolicy(const char *val, struct kernel_param *kp)
-#else
 static int nvme_set_iopolicy(const char *val, const struct kernel_param *kp)
-#endif
 {
 	if (!val)
 		return -EINVAL;
@@ -62,7 +57,11 @@ void nvme_mpath_unfreeze(struct nvme_subsystem *subsys)
 	lockdep_assert_held(&subsys->lock);
 	list_for_each_entry(h, &subsys->nsheads, entry)
 		if (h->disk)
+#ifdef HAVE_BLK_MQ_UNFREEZE_QUEUE_2_PARAMS
+			blk_mq_unfreeze_queue_nomemrestore(h->disk->queue);
+#else
 			blk_mq_unfreeze_queue(h->disk->queue);
+#endif
 }
 
 void nvme_mpath_wait_freeze(struct nvme_subsystem *subsys)
@@ -117,11 +116,20 @@ void nvme_failover_req(struct request *req)
 			bio->bi_cookie = BLK_QC_T_NONE;
 		}
 #endif
+		/*
+		 * The alternate request queue that we may end up submitting
+		 * the bio to may be frozen temporarily, in this case REQ_NOWAIT
+		 * will fail the I/O immediately with EAGAIN to the issuer.
+		 * We are not in the issuer context which cannot block. Clear
+		 * the flag to avoid spurious EAGAIN I/O failures.
+		 */
+		bio->bi_opf &= ~REQ_NOWAIT;
 	}
 	blk_steal_bios(&ns->head->requeue_list, req);
 	spin_unlock_irqrestore(&ns->head->requeue_lock, flags);
 
-	blk_mq_end_request(req, 0);
+	nvme_req(req)->status = 0;
+	nvme_end_req(req);
 	kblockd_schedule_work(&ns->head->requeue_work);
 }
 
@@ -167,21 +175,22 @@ void nvme_mpath_end_request(struct request *rq)
 void nvme_kick_requeue_lists(struct nvme_ctrl *ctrl)
 {
 	struct nvme_ns *ns;
+	int srcu_idx;
 
-	down_read(&ctrl->namespaces_rwsem);
-	list_for_each_entry(ns, &ctrl->namespaces, list) {
+	srcu_idx = srcu_read_lock(&ctrl->srcu);
+	list_for_each_entry_rcu(ns, &ctrl->namespaces, list) {
 #ifdef HAVE_DISK_UEVENT
 		if (!ns->head->disk)
 			continue;
 		kblockd_schedule_work(&ns->head->requeue_work);
-		if (ctrl->state == NVME_CTRL_LIVE)
+		if (nvme_ctrl_state(ns->ctrl) == NVME_CTRL_LIVE)
 			disk_uevent(ns->head->disk, KOBJ_CHANGE);
 #else
-	if (ns->head->disk)
-		kblockd_schedule_work(&ns->head->requeue_work);
+		if (ns->head->disk)
+			kblockd_schedule_work(&ns->head->requeue_work);
 #endif
 	}
-	up_read(&ctrl->namespaces_rwsem);
+	srcu_read_unlock(&ctrl->srcu, srcu_idx);
 }
 
 static const char *nvme_ana_state_names[] = {
@@ -215,13 +224,14 @@ out:
 void nvme_mpath_clear_ctrl_paths(struct nvme_ctrl *ctrl)
 {
 	struct nvme_ns *ns;
+	int srcu_idx;
 
-	down_read(&ctrl->namespaces_rwsem);
-	list_for_each_entry(ns, &ctrl->namespaces, list) {
+	srcu_idx = srcu_read_lock(&ctrl->srcu);
+	list_for_each_entry_rcu(ns, &ctrl->namespaces, list) {
 		nvme_mpath_clear_current_path(ns);
 		kblockd_schedule_work(&ns->head->requeue_work);
 	}
-	up_read(&ctrl->namespaces_rwsem);
+	srcu_read_unlock(&ctrl->srcu, srcu_idx);
 }
 
 void nvme_mpath_revalidate_paths(struct nvme_ns *ns)
@@ -245,13 +255,14 @@ void nvme_mpath_revalidate_paths(struct nvme_ns *ns)
 
 static bool nvme_path_is_disabled(struct nvme_ns *ns)
 {
+	enum nvme_ctrl_state state = nvme_ctrl_state(ns->ctrl);
+
 	/*
 	 * We don't treat NVME_CTRL_DELETING as a disabled path as I/O should
 	 * still be able to complete assuming that the controller is connected.
 	 * Otherwise it will fail immediately and return to the requeue list.
 	 */
-	if (ns->ctrl->state != NVME_CTRL_LIVE &&
-	    ns->ctrl->state != NVME_CTRL_DELETING)
+	if (state != NVME_CTRL_LIVE && state != NVME_CTRL_DELETING)
 		return true;
 	if (test_bit(NVME_NS_ANA_PENDING, &ns->flags) ||
 	    !test_bit(NVME_NS_READY, &ns->flags))
@@ -268,7 +279,8 @@ static struct nvme_ns *__nvme_find_path(struct nvme_ns_head *head, int node)
 		if (nvme_path_is_disabled(ns))
 			continue;
 
-		if (READ_ONCE(head->subsys->iopolicy) == NVME_IOPOLICY_NUMA)
+		if (ns->ctrl->numa_node != NUMA_NO_NODE &&
+		    READ_ONCE(head->subsys->iopolicy) == NVME_IOPOLICY_NUMA)
 			distance = node_distance(node, ns->ctrl->numa_node);
 		else
 			distance = LOCAL_DISTANCE;
@@ -353,7 +365,7 @@ out:
 
 static inline bool nvme_path_is_optimized(struct nvme_ns *ns)
 {
-	return ns->ctrl->state == NVME_CTRL_LIVE &&
+	return nvme_ctrl_state(ns->ctrl) == NVME_CTRL_LIVE &&
 		ns->ana_state == NVME_ANA_OPTIMIZED;
 }
 
@@ -380,7 +392,7 @@ static bool nvme_available_path(struct nvme_ns_head *head)
 	list_for_each_entry_rcu(ns, &head->list, siblings) {
 		if (test_bit(NVME_CTRL_FAILFAST_EXPIRED, &ns->ctrl->flags))
 			continue;
-		switch (ns->ctrl->state) {
+		switch (nvme_ctrl_state(ns->ctrl)) {
 		case NVME_CTRL_LIVE:
 		case NVME_CTRL_RESETTING:
 		case NVME_CTRL_CONNECTING:
@@ -427,7 +439,7 @@ static blk_qc_t nvme_ns_head_make_request(struct request_queue *q,
 		return;
 #else
 #ifdef HAVE_BLK_QUEUE_SPLIT_1_PARAM
- 	blk_queue_split(&bio);
+	blk_queue_split(&bio);
 #else
 	blk_queue_split(q, &bio);
 #endif
@@ -445,10 +457,10 @@ static blk_qc_t nvme_ns_head_make_request(struct request_queue *q,
 #ifdef HAVE_TRACE_BLOCK_BIO_REMAP_4_PARAM
 		trace_block_bio_remap(bio->bi_disk->queue, bio,
 				      disk_devt(ns->head->disk),
-       			      bio->bi_iter.bi_sector);
+				      bio->bi_iter.bi_sector);
 #else
 		trace_block_bio_remap(bio, disk_devt(ns->head->disk),
- 				      bio->bi_iter.bi_sector);
+				      bio->bi_iter.bi_sector);
 #endif
 #ifdef HAVE_SUBMIT_BIO_NOACCT
 #ifdef HAVE_BIO_BI_COOKIE
@@ -562,11 +574,11 @@ static const struct file_operations nvme_ns_head_chr_fops = {
 	.release	= nvme_ns_head_chr_release,
 	.unlocked_ioctl	= nvme_ns_head_chr_ioctl,
 	.compat_ioctl	= compat_ptr_ioctl,
-#ifdef HAVE_FILE_OPERATIONS_URING_CMD
+#if defined(HAVE_FILE_OPERATIONS_URING_CMD) && defined(HAVE_IO_URING_CMD_H) && defined(HAVE_BIO_INTEGRITY_MAP_USER)
 	.uring_cmd	= nvme_ns_head_chr_uring_cmd,
 #endif
-#ifdef HAVE_FILE_OPERATIONS_URING_CMD_IOPOLL
-	.uring_cmd_iopoll = nvme_ns_head_chr_uring_cmd_iopoll,
+#if defined(HAVE_FILE_OPERATIONS_URING_CMD_IOPOLL) && defined(HAVE_IO_URING_CMD_H) && defined(HAVE_BIO_INTEGRITY_MAP_USER)
+	.uring_cmd_iopoll = nvme_ns_chr_uring_cmd_iopoll,
 #endif
 };
 
@@ -613,7 +625,12 @@ int nvme_mpath_alloc_disk(struct nvme_ctrl *ctrl, struct nvme_ns_head *head)
 #ifndef HAVE_BLK_ALLOC_DISK
 	struct request_queue *q;
 #endif
+#ifdef HAVE_QUEUE_LIMITS_COMMIT_UPDATE
+	struct queue_limits lim;
+#endif
+#ifndef HAVE_BLK_INTEGRITY_CSUM_CRC64
 	bool vwc = false;
+#endif
 
 	mutex_init(&head->lock);
 	bio_list_init(&head->requeue_list);
@@ -628,31 +645,48 @@ int nvme_mpath_alloc_disk(struct nvme_ctrl *ctrl, struct nvme_ns_head *head)
 	if (!(ctrl->subsys->cmic & NVME_CTRL_CMIC_MULTI_CTRL) || !multipath)
 		return 0;
 
-#ifdef HAVE_BLK_ALLOC_DISK
+#ifdef HAVE_QUEUE_LIMITS_COMMIT_UPDATE
+	blk_set_stacking_limits(&lim);
+	lim.dma_alignment = 3;
+#ifdef HAVE_BLK_INTEGRITY_CSUM_CRC64
+	lim.features |= BLK_FEAT_IO_STAT | BLK_FEAT_NOWAIT | BLK_FEAT_POLL;
+#endif
+	if (head->ids.csi != NVME_CSI_ZNS)
+		lim.max_zone_append_sectors = 0;
+
+	head->disk = blk_alloc_disk(&lim, ctrl->numa_node);
+	if (IS_ERR(head->disk))
+		return PTR_ERR(head->disk);
+#else
+#  ifdef HAVE_BLK_ALLOC_DISK
+#    ifdef HAVE_BLK_ALLOC_DISK_2_PARAMS
+	head->disk = blk_alloc_disk(NULL, ctrl->numa_node);
+#    else
 	head->disk = blk_alloc_disk(ctrl->numa_node);
-#else
-#ifdef HAVE_BLOCK_DEVICE_OPERATIONS_SUBMIT_BIO
+#    endif // ALLOC_DISK_2_PARAM
+#  else
+#    ifdef HAVE_BLOCK_DEVICE_OPERATIONS_SUBMIT_BIO
 	q = blk_alloc_queue(ctrl->numa_node);
-#else
-#ifdef HAVE_BLK_QUEUE_MAKE_REQUEST
-#ifdef HAVE_BLK_ALLOC_QUEUE_NODE_3_ARGS
+#    else
+#      ifdef HAVE_BLK_QUEUE_MAKE_REQUEST
+#        ifdef HAVE_BLK_ALLOC_QUEUE_NODE_3_ARGS
 	q = blk_alloc_queue_node(GFP_KERNEL, NUMA_NO_NODE, NULL);
-#else
-#ifdef HAVE_BLK_ALLOC_QUEUE_RH
+#        else
+#          ifdef HAVE_BLK_ALLOC_QUEUE_RH
 	q = blk_alloc_queue_rh(nvme_ns_head_make_request, ctrl->numa_node);
-#else
+#          else
 	q = blk_alloc_queue_node(GFP_KERNEL, ctrl->numa_node);
-#endif
-#endif
-#else
+#          endif // ALLOC_QUEUE_RH
+#        endif // ALLOC_QUEUE_NODE_3_ARGS
+#      else
 	q = blk_alloc_queue(nvme_ns_head_make_request, ctrl->numa_node);
-#endif
-#endif /* HAVE_BLOCK_DEVICE_OPERATIONS_SUBMIT_BIO */
+#      endif // QUEUE_MAKE_REQUEST
+#    endif // BLOCK_DEVICE_OPERATIONS_SUBMIT_BIO
 	if (!q)
 		goto out;
-#if defined(HAVE_BLK_QUEUE_MAKE_REQUEST) && !defined(HAVE_BLK_ALLOC_QUEUE_RH)
+#    if defined(HAVE_BLK_QUEUE_MAKE_REQUEST) && !defined(HAVE_BLK_ALLOC_QUEUE_RH)
 	blk_queue_make_request(q, nvme_ns_head_make_request);
-#endif
+#    endif
 	blk_queue_flag_set(QUEUE_FLAG_NONROT, q);
 	/* set to a default value for 512 until disk is validated */
 	blk_queue_logical_block_size(q, 512);
@@ -664,13 +698,14 @@ int nvme_mpath_alloc_disk(struct nvme_ctrl *ctrl, struct nvme_ns_head *head)
 	blk_queue_write_cache(q, vwc, vwc);
 
 	head->disk = alloc_disk(0);
-#endif /* HAVE_BLK_ALLOC_DISK */
+#  endif // BLK_ALLOC_DISK
 	if (!head->disk)
-#ifdef HAVE_BLK_ALLOC_DISK
+#  ifdef HAVE_BLK_ALLOC_DISK
 		return -ENOMEM;
-#else
+#  else
 		goto out_cleanup_queue;
-#endif
+#  endif
+#endif // QUEUE_LIMITS_COMMIT_UPDATE
 	head->disk->fops = &nvme_ns_head_ops;
 	head->disk->private_data = head;
 #ifndef HAVE_BLK_ALLOC_DISK
@@ -681,8 +716,8 @@ int nvme_mpath_alloc_disk(struct nvme_ctrl *ctrl, struct nvme_ns_head *head)
 #endif
 	sprintf(head->disk->disk_name, "nvme%dn%d",
 			ctrl->subsys->instance, head->instance);
-
 #ifdef HAVE_BLK_ALLOC_DISK
+#ifndef HAVE_BLK_INTEGRITY_CSUM_CRC64
 	blk_queue_flag_set(QUEUE_FLAG_NONROT, head->disk->queue);
 	blk_queue_flag_set(QUEUE_FLAG_NOWAIT, head->disk->queue);
 #if defined HAVE_BDEV_START_IO_ACCT || defined HAVE_BDEV_START_IO_ACCT_3_PARAM
@@ -694,21 +729,23 @@ int nvme_mpath_alloc_disk(struct nvme_ctrl *ctrl, struct nvme_ns_head *head)
 	 * but if the assumption is wrong the effect is only suboptimal
 	 * performance but not correctness problem.
 	 */
-#ifdef HAVE_BIO_BI_COOKIE
 	if (ctrl->tagset->nr_maps > HCTX_TYPE_POLL &&
 	    ctrl->tagset->map[HCTX_TYPE_POLL].nr_queues)
 		blk_queue_flag_set(QUEUE_FLAG_POLL, head->disk->queue);
 #endif
 
+#  ifndef HAVE_QUEUE_LIMITS_COMMIT_UPDATE
 	/* set to a default value of 512 until the disk is validated */
 	blk_queue_logical_block_size(head->disk->queue, 512);
 	blk_set_stacking_limits(&head->disk->queue->limits);
 	blk_queue_dma_alignment(head->disk->queue, 3);
-
+#  endif
+#ifndef HAVE_BLK_INTEGRITY_CSUM_CRC64
 	/* we need to propagate up the VMC settings */
 	if (ctrl->vwc & NVME_CTRL_VWC_PRESENT)
 		vwc = true;
 	blk_queue_write_cache(head->disk->queue, vwc, vwc);
+#endif
 	return 0;
 #else
 	return 0;
@@ -737,35 +774,34 @@ static void nvme_mpath_set_live(struct nvme_ns *ns)
 	 */
 #ifdef HAVE_DEVICE_ADD_DISK_3_ARGS
 	if (!test_and_set_bit(NVME_NSHEAD_DISK_LIVE, &head->flags)) {
-#ifdef HAVE_DEVICE_ADD_DISK_RETURN
+#  ifdef HAVE_DEVICE_ADD_DISK_RETURN
 		rc = device_add_disk(&head->subsys->dev, head->disk,
-				     nvme_ns_id_attr_groups);
+				     nvme_ns_attr_groups);
 		if (rc) {
 			clear_bit(NVME_NSHEAD_DISK_LIVE, &ns->flags);
 			return;
 		}
-#else
+#  else
 		device_add_disk(&head->subsys->dev, head->disk,
-				nvme_ns_id_attr_groups);
-#endif
+				nvme_ns_attr_groups);
+#  endif
 		nvme_add_ns_head_cdev(head);
 	}
 #else
 	if (!test_and_set_bit(NVME_NSHEAD_DISK_LIVE, &head->flags)) {
 		device_add_disk(&head->subsys->dev, head->disk);
 		if (sysfs_create_group(&disk_to_dev(head->disk)->kobj,
-				&nvme_ns_id_attr_group))
+				&nvme_ns_attr_group))
 			dev_warn(&head->subsys->dev,
 				 "failed to create id group.\n");
 	}
 #endif
-
 	mutex_lock(&head->lock);
 	if (nvme_path_is_optimized(ns)) {
 		int node, srcu_idx;
 
 		srcu_idx = srcu_read_lock(&head->srcu);
-		for_each_node(node)
+		for_each_online_node(node)
 			__nvme_find_path(head, node);
 		srcu_read_unlock(&head->srcu, srcu_idx);
 	}
@@ -844,7 +880,7 @@ static void nvme_update_ns_ana_state(struct nvme_ana_group_desc *desc,
 	 * controller is ready.
 	 */
 	if (nvme_state_is_live(ns->ana_state) &&
-	    ns->ctrl->state == NVME_CTRL_LIVE)
+	    nvme_ctrl_state(ns->ctrl) == NVME_CTRL_LIVE)
 		nvme_mpath_set_live(ns);
 }
 
@@ -854,6 +890,7 @@ static int nvme_update_ana_state(struct nvme_ctrl *ctrl,
 	u32 nr_nsids = le32_to_cpu(desc->nnsids), n = 0;
 	unsigned *nr_change_groups = data;
 	struct nvme_ns *ns;
+	int srcu_idx;
 
 	dev_dbg(ctrl->device, "ANA group %d: %s.\n",
 			le32_to_cpu(desc->grpid),
@@ -865,8 +902,8 @@ static int nvme_update_ana_state(struct nvme_ctrl *ctrl,
 	if (!nr_nsids)
 		return 0;
 
-	down_read(&ctrl->namespaces_rwsem);
-	list_for_each_entry(ns, &ctrl->namespaces, list) {
+	srcu_idx = srcu_read_lock(&ctrl->srcu);
+	list_for_each_entry_rcu(ns, &ctrl->namespaces, list) {
 		unsigned nsid;
 again:
 		nsid = le32_to_cpu(desc->nsids[n]);
@@ -879,7 +916,7 @@ again:
 		if (ns->head->ns_id > nsid)
 			goto again;
 	}
-	up_read(&ctrl->namespaces_rwsem);
+	srcu_read_unlock(&ctrl->srcu, srcu_idx);
 	return 0;
 }
 
@@ -925,7 +962,7 @@ static void nvme_ana_work(struct work_struct *work)
 {
 	struct nvme_ctrl *ctrl = container_of(work, struct nvme_ctrl, ana_work);
 
-	if (ctrl->state != NVME_CTRL_LIVE)
+	if (nvme_ctrl_state(ctrl) != NVME_CTRL_LIVE)
 		return;
 
 	nvme_read_ana_log(ctrl);
@@ -943,16 +980,9 @@ void nvme_mpath_update(struct nvme_ctrl *ctrl)
 	mutex_unlock(&ctrl->ana_lock);
 }
 
-#ifdef HAVE_TIMER_SETUP
 static void nvme_anatt_timeout(struct timer_list *t)
 {
 	struct nvme_ctrl *ctrl = from_timer(ctrl, t, anatt_timer);
-#else
-static void nvme_anatt_timeout(unsigned long data)
-{
-	struct nvme_ctrl *ctrl = (struct nvme_ctrl *)data;
-#endif
-
 	dev_info(ctrl->device, "ANATT timeout, resetting controller.\n");
 	nvme_reset_ctrl(ctrl);
 }
@@ -1050,6 +1080,7 @@ void nvme_mpath_add_disk(struct nvme_ns *ns, __le32 anagrpid)
 		ns->ana_state = NVME_ANA_OPTIMIZED;
 		nvme_mpath_set_live(ns);
 	}
+#ifndef HAVE_BLK_INTEGRITY_CSUM_CRC64
 #ifdef HAVE_QUEUE_FLAG_STABLE_WRITES
 	if (blk_queue_stable_writes(ns->queue) && ns->head->disk)
 		blk_queue_flag_set(QUEUE_FLAG_STABLE_WRITES,
@@ -1063,6 +1094,7 @@ void nvme_mpath_add_disk(struct nvme_ns *ns, __le32 anagrpid)
 					 BDI_CAP_STABLE_WRITES;
 	}
 #endif
+#endif
 
 #ifdef CONFIG_BLK_DEV_ZONED
 	if (blk_queue_is_zoned(ns->queue) && ns->head->disk)
@@ -1087,7 +1119,7 @@ void nvme_mpath_shutdown_disk(struct nvme_ns_head *head)
 #else
 	if (test_bit(NVME_NSHEAD_DISK_LIVE, &head->flags)) {
 		sysfs_remove_group(&disk_to_dev(head->disk)->kobj,
-				   &nvme_ns_id_attr_group);
+				   &nvme_ns_attr_group);
 		del_gendisk(head->disk);
 	}
 #endif
@@ -1117,13 +1149,7 @@ void nvme_mpath_remove_disk(struct nvme_ns_head *head)
 void nvme_mpath_init_ctrl(struct nvme_ctrl *ctrl)
 {
 	mutex_init(&ctrl->ana_lock);
-#ifdef HAVE_TIMER_SETUP
 	timer_setup(&ctrl->anatt_timer, nvme_anatt_timeout, 0);
-#else
-	init_timer(&ctrl->anatt_timer);
-	ctrl->anatt_timer.data = (unsigned long)ctrl;
-	ctrl->anatt_timer.function = nvme_anatt_timeout;
-#endif
 	INIT_WORK(&ctrl->ana_work, nvme_ana_work);
 }
 
@@ -1184,4 +1210,3 @@ void nvme_mpath_uninit(struct nvme_ctrl *ctrl)
 	ctrl->ana_log_buf = NULL;
 	ctrl->ana_log_size = 0;
 }
-#endif /* HAVE_BLK_TYPES_REQ_DRV */
