@@ -36,6 +36,7 @@
 #include "csum.h"
 #include "dp-packet.h"
 #include "netdev.h"
+#include "netdev-offload.h"
 #include "netdev-vport.h"
 #include "netdev-vport-private.h"
 #include "odp-netlink.h"
@@ -243,13 +244,24 @@ netdev_tnl_push_udp_header(const struct netdev *netdev OVS_UNUSED,
                            const struct ovs_action_push_tnl *data)
 {
     struct udp_header *udp;
+    uint16_t entropy;
+    int err = EINVAL;
     int ip_tot_size;
 
+    /* check netdev provider entropy on original packet before pushing tunnel header */
+    if (packet->orig_netdev) {
+        err = netdev_packet_hw_entropy(packet->orig_netdev, packet, &entropy);
+    }
+
     udp = netdev_tnl_push_ip_header(packet, data->header, data->header_len, &ip_tot_size);
 
     /* set udp src port */
-    udp->udp_src = netdev_tnl_get_src_port(packet);
     udp->udp_len = htons(ip_tot_size);
+    if (!err) {
+        udp->udp_src = entropy;
+    } else {
+        udp->udp_src = netdev_tnl_get_src_port(packet);
+    }
 
     if (udp->udp_csum) {
         netdev_tnl_calc_udp_csum(udp, packet, ip_tot_size);
