@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 2008 - 2014, 2016, 2017 Nicira, Inc.
  * Copyright (c) 2019 Samsung Electronics Co.,Ltd.
- * Copyright (c) 2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2023-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -305,6 +305,7 @@ netdev_hw_miss_packet_recover(struct netdev *netdev,
         atomic_store_relaxed(&netdev->hw_info.miss_api_supported, false);
     }
 
+    packet->orig_netdev = netdev;
     return rv;
 }
 
@@ -1248,12 +1249,26 @@ netdev_set_flow_api_enabled(const struct smap *ovs_other_config)
 int
 netdev_packet_hw_hash(struct netdev *netdev,
                       struct dp_packet *packet,
+                      uint32_t seed,
                       uint32_t *hash)
 {
     const struct netdev_flow_api *flow_api =
         ovsrcu_get(const struct netdev_flow_api *, &netdev->flow_api);
 
     return (flow_api && flow_api->packet_hw_hash)
-        ? flow_api->packet_hw_hash(netdev, packet, hash)
+        ? flow_api->packet_hw_hash(netdev, packet, seed, hash)
+        : EOPNOTSUPP;
+}
+
+int
+netdev_packet_hw_entropy(struct netdev *netdev,
+                         struct dp_packet *packet,
+                         uint16_t *entropy)
+{
+    const struct netdev_flow_api *flow_api =
+        ovsrcu_get(const struct netdev_flow_api *, &netdev->flow_api);
+
+    return (flow_api && flow_api->packet_hw_entropy)
+        ? flow_api->packet_hw_entropy(netdev, packet, entropy)
         : EOPNOTSUPP;
 }
