@@ -1585,29 +1585,106 @@ static int cmd_port_function_rate_show(struct mlxdevm *mlxdevm)
 	return err;
 }
 
+static void get_out_rate(struct mlxdevm *mlxdevm, struct nlattr **tb)
+{
+	if (!tb[MLXDEVM_ATTR_EXT_RATE_TYPE])
+		return;
+
+	if (tb[MLXDEVM_ATTR_EXT_RATE_TX_MAX])
+		mlxdevm->opts.port_function_rate_tx_max =
+			mnl_attr_get_u64(tb[MLXDEVM_ATTR_EXT_RATE_TX_MAX]);
+
+	if (tb[MLXDEVM_ATTR_EXT_RATE_TX_SHARE])
+		mlxdevm->opts.port_function_rate_tx_share =
+			mnl_attr_get_u64(tb[MLXDEVM_ATTR_EXT_RATE_TX_SHARE]);
+}
+
+static int cmd_port_function_rate_set_cb(const struct nlmsghdr *nlh, void *data)
+{
+	struct mlxdevm *mlxdevm = data;
+	struct nlattr *tb[MLXDEVM_ATTR_MAX + 1] = {};
+	struct genlmsghdr *genl = mnl_nlmsg_get_payload(nlh);
+
+	mnl_attr_parse(nlh, sizeof(*genl), attr_cb, tb);
+	if (!tb[MLXDEVM_ATTR_DEV_BUS_NAME] || !tb[MLXDEVM_ATTR_DEV_NAME])
+		return MNL_CB_ERROR;
+
+	get_out_rate(mlxdevm, tb);
+	return MNL_CB_OK;
+}
+
+static int port_fn_check_tx_rates(uint64_t min_rate, uint64_t max_rate)
+{
+	if (max_rate && min_rate > max_rate) {
+		pr_err("Invalid. Expected tx_share <= tx_max or tx_share == 0.\n");
+		return -EINVAL;
+	}
+	return 0;
+}
+
+static int port_fn_get_and_check_tx_rates(struct mlxdevm_opts *reply,
+					  struct mlxdevm_opts *request)
+{
+	uint64_t min = reply->port_function_rate_tx_share;
+	uint64_t max = reply->port_function_rate_tx_max;
+
+	if (request->present & MLXDEVM_OPT_PORT_FN_RATE_TX_SHARE)
+		return port_fn_check_tx_rates(request->port_function_rate_tx_share, max);
+	return port_fn_check_tx_rates(min, request->port_function_rate_tx_max);
+}
+
 static int cmd_port_function_rate_set(struct mlxdevm *mlxdevm)
 {
+	struct mlxdevm_opts *opts = &mlxdevm->opts;
+	struct mlxdevm_opts tmp_opts = {0};
 	struct nlmsghdr *nlh;
 	int err = 0;
 
 	mlxdevm_arg_inc(mlxdevm);
 
-	nlh = mnlu_gen_socket_cmd_prepare(&mlxdevm->nlg,
-					  MLXDEVM_CMD_EXT_RATE_SET,
-					  NLM_F_REQUEST | NLM_F_ACK);
-
-	err = mlxdevm_argv_parse_put(nlh, mlxdevm,
-				     MLXDEVM_OPT_HANDLEP | MLXDEVM_OPT_PORT_FN_RATE_NODE_NAME,
-				     MLXDEVM_OPT_PORT_FN_RATE_TX_SHARE |
-				     MLXDEVM_OPT_PORT_FN_RATE_TX_MAX |
-				     MLXDEVM_OPT_PORT_FN_RATE_TC_BW |
-				     MLXDEVM_OPT_PORT_FN_RATE_PARENT_NODE_NAME);
+	err = mlxdevm_argv_parse(mlxdevm,
+				 MLXDEVM_OPT_HANDLEP | MLXDEVM_OPT_PORT_FN_RATE_NODE_NAME,
+				 MLXDEVM_OPT_PORT_FN_RATE_TX_SHARE |
+				 MLXDEVM_OPT_PORT_FN_RATE_TX_MAX |
+				 MLXDEVM_OPT_PORT_FN_RATE_TC_BW |
+				 MLXDEVM_OPT_PORT_FN_RATE_PARENT_NODE_NAME);
 
 	if (err) {
 		cmd_port_function_rate_help();
 		return err;
 	}
 
+	if ((opts->present & MLXDEVM_OPT_PORT_FN_RATE_TX_SHARE) &&
+	    (opts->present & MLXDEVM_OPT_PORT_FN_RATE_TX_MAX)) {
+		err = port_fn_check_tx_rates(opts->port_function_rate_tx_share,
+					     opts->port_function_rate_tx_max);
+		if (err)
+			return err;
+	} else if (opts->present &
+		   (MLXDEVM_OPT_PORT_FN_RATE_TX_SHARE | MLXDEVM_OPT_PORT_FN_RATE_TX_MAX)) {
+		nlh = mnlu_gen_socket_cmd_prepare(&mlxdevm->nlg, MLXDEVM_CMD_EXT_RATE_GET,
+						  NLM_F_REQUEST | NLM_F_ACK);
+		tmp_opts = *opts;
+		opts->present &= ~(MLXDEVM_OPT_PORT_FN_RATE_TX_SHARE |
+				   MLXDEVM_OPT_PORT_FN_RATE_TX_MAX |
+				   MLXDEVM_OPT_PORT_FN_RATE_PARENT_NODE_NAME);
+		pr_out_section_start(mlxdevm, "rate");
+		mlxdevm_opts_put(nlh, mlxdevm);
+		err = mnlu_gen_socket_sndrcv(&mlxdevm->nlg, nlh,
+					     cmd_port_function_rate_set_cb,
+					     mlxdevm);
+		pr_out_section_end(mlxdevm);
+		err = port_fn_get_and_check_tx_rates(&mlxdevm->opts, &tmp_opts);
+		if (err)
+			return err;
+		*opts = tmp_opts;
+	}
+
+	nlh = mnlu_gen_socket_cmd_prepare(&mlxdevm->nlg,
+					  MLXDEVM_CMD_EXT_RATE_SET,
+					  NLM_F_REQUEST | NLM_F_ACK);
+
+	mlxdevm_opts_put(nlh, mlxdevm);
 	return mnlu_gen_socket_sndrcv(&mlxdevm->nlg, nlh, NULL, NULL);
 }
 
@@ -1663,17 +1740,13 @@ static int cmd_port_function_rate(struct mlxdevm *mlxdevm)
 	}
 
 	if (mlxdevm_argv_match(mlxdevm, "set")) {
-		cmd_port_function_rate_set(mlxdevm);
-		return 0;
+		return cmd_port_function_rate_set(mlxdevm);
 	} else if (mlxdevm_argv_match(mlxdevm, "show")) {
-		cmd_port_function_rate_show(mlxdevm);
-		return 0;
+		return cmd_port_function_rate_show(mlxdevm);
 	} else if (mlxdevm_argv_match(mlxdevm, "add")) {
-		cmd_port_function_rate_add(mlxdevm);
-		return 0;
+		return cmd_port_function_rate_add(mlxdevm);
 	} else if (mlxdevm_argv_match(mlxdevm, "del")) {
-		cmd_port_function_rate_del(mlxdevm);
-		return 0;
+		return cmd_port_function_rate_del(mlxdevm);
 	}
 
 	cmd_port_function_rate_help();
