@@ -1,5 +1,7 @@
 /*
- * Copyright (c) 2001-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
+ * Copyright (c) 2001-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: GPL-2.0-only or BSD-3-Clause
  *
  * This software is available to you under a choice of one of two
  * licenses.  You may choose to be licensed under the terms of the GNU
@@ -60,48 +62,27 @@ ib_ctx_handler::ib_ctx_handler(struct ib_ctx_handler_desc *desc)
     , m_on_device_memory(0)
     , m_removed(false)
     , m_lock_umr("spin_lock_umr")
-    , m_p_ctx_time_converter(NULL)
+    , m_p_ctx_time_converter(nullptr)
 {
-    if (NULL == desc) {
+    if (!desc) {
         ibch_logpanic("Invalid ib_ctx_handler");
     }
 
     m_p_ibv_device = desc->device;
 
-    if (m_p_ibv_device == NULL) {
+    if (!m_p_ibv_device) {
         ibch_logpanic("m_p_ibv_device is invalid");
     }
 
-    m_p_ibv_context = NULL;
-#ifdef DEFINED_DPCP
     m_p_adapter = set_dpcp_adapter();
-    if (NULL == m_p_adapter)
-#endif /* DEFINED_DPCP */
-    {
-#if defined(DEFINED_ROCE_LAG)
-        struct mlx5dv_context_attr dv_attr;
-
-        memset(&dv_attr, 0, sizeof(dv_attr));
-        dv_attr.flags |= MLX5DV_CONTEXT_FLAGS_DEVX;
-        m_p_ibv_context = mlx5dv_open_device(m_p_ibv_device, &dv_attr);
-#endif /* DEFINED_ROCE_LAG */
-        if (m_p_ibv_context == NULL) {
-            m_p_ibv_context = ibv_open_device(m_p_ibv_device);
-        }
-        if (m_p_ibv_context == NULL) {
-            ibch_logpanic("m_p_ibv_context is invalid");
-        }
-        // Create pd for this device
-        m_p_ibv_pd = ibv_alloc_pd(m_p_ibv_context);
-        if (m_p_ibv_pd == NULL) {
-            ibch_logpanic("ibv device %p pd allocation failure (ibv context %p) (errno=%d %m)",
-                          m_p_ibv_device, m_p_ibv_context, errno);
-        }
+    if (!m_p_adapter) {
+        ibch_logpanic("ibv device %p adapter allocation failure (errno=%d %m)", m_p_ibv_device,
+                      errno);
     }
     VALGRIND_MAKE_MEM_DEFINED(m_p_ibv_pd, sizeof(struct ibv_pd));
 
     m_p_ibv_device_attr = new xlio_ibv_device_attr_ex();
-    if (m_p_ibv_device_attr == NULL) {
+    if (!m_p_ibv_device_attr) {
         ibch_logpanic("ibv device %p attr allocation failure (ibv context %p) (errno=%d %m)",
                       m_p_ibv_device, m_p_ibv_context, errno);
     }
@@ -138,15 +119,9 @@ err:
         ibv_dealloc_pd(m_p_ibv_pd);
     }
 
-#ifdef DEFINED_DPCP
     if (m_p_adapter) {
         delete m_p_adapter;
-        m_p_ibv_context = NULL;
-    }
-#endif /* DEFINED_DPCP */
-    if (m_p_ibv_context) {
-        ibv_close_device(m_p_ibv_context);
-        m_p_ibv_context = NULL;
+        m_p_ibv_context = nullptr;
     }
 }
 
@@ -171,7 +146,7 @@ ib_ctx_handler::~ib_ctx_handler()
         }
         ENDIF_VERBS_FAILURE;
         VALGRIND_MAKE_MEM_UNDEFINED(m_p_ibv_pd, sizeof(struct ibv_pd));
-        m_p_ibv_pd = NULL;
+        m_p_ibv_pd = nullptr;
     }
 
     if (m_p_ctx_time_converter) {
@@ -179,15 +154,9 @@ ib_ctx_handler::~ib_ctx_handler()
     }
     delete m_p_ibv_device_attr;
 
-#ifdef DEFINED_DPCP
     if (m_p_adapter) {
         delete m_p_adapter;
-        m_p_ibv_context = NULL;
-    }
-#endif /* DEFINED_DPCP */
-    if (m_p_ibv_context) {
-        ibv_close_device(m_p_ibv_context);
-        m_p_ibv_context = NULL;
+        m_p_ibv_context = nullptr;
     }
 
     BULLSEYE_EXCLUDE_BLOCK_END
@@ -235,8 +204,6 @@ void ib_ctx_handler::print_val()
     ibch_logdbg("%s", m_str);
 }
 
-#ifdef DEFINED_DPCP
-
 int parse_dpcp_version(const char *dpcp_ver)
 {
     static const std::string s_delimiter(".");
@@ -257,15 +224,15 @@ int parse_dpcp_version(const char *dpcp_ver)
 dpcp::adapter *ib_ctx_handler::set_dpcp_adapter()
 {
     dpcp::status status = dpcp::DPCP_ERR_NO_SUPPORT;
-    dpcp::provider *p_provider = NULL;
-    dpcp::adapter_info *dpcp_lst = NULL;
+    dpcp::provider *p_provider = nullptr;
+    dpcp::adapter_info *dpcp_lst = nullptr;
     size_t adapters_num = 0;
     size_t i = 0;
     int dpcp_ver = 0;
 
-    m_p_adapter = NULL;
+    m_p_adapter = nullptr;
     if (!m_p_ibv_device) {
-        return NULL;
+        return nullptr;
     }
 
     status = dpcp::provider::get_instance(p_provider);
@@ -281,13 +248,18 @@ dpcp::adapter *ib_ctx_handler::set_dpcp_adapter()
         goto err;
     }
 
-    status = p_provider->get_adapter_info_lst(NULL, adapters_num);
-    if (0 == adapters_num) {
+    /*
+     * get_adapter_info_lst() returns number of adapters in response to NULL or
+     * 0 arguments along with DPCP_ERR_OUT_OF_RANGE error. On success, the
+     * number of actual adapters is not set, so we need a separate call here.
+     */
+    status = p_provider->get_adapter_info_lst(nullptr, adapters_num);
+    if (dpcp::DPCP_ERR_OUT_OF_RANGE != status || 0 == adapters_num) {
         ibch_logdbg("found no adapters status = %d", status);
         goto err;
     }
 
-    dpcp_lst = new (std::nothrow) dpcp::adapter_info[adapters_num];
+    dpcp_lst = new (std::nothrow) dpcp::adapter_info[static_cast<unsigned>(adapters_num)];
     if (!dpcp_lst) {
         ibch_logerr("failed allocating memory for devices");
         goto err;
@@ -301,13 +273,13 @@ dpcp::adapter *ib_ctx_handler::set_dpcp_adapter()
 
     for (i = 0; i < adapters_num; i++) {
         if (dpcp_lst[i].name == m_p_ibv_device->name) {
-            dpcp::adapter *adapter = NULL;
+            dpcp::adapter *adapter = nullptr;
 
             status = p_provider->open_adapter(dpcp_lst[i].name, adapter);
             if ((dpcp::DPCP_OK == status) && (adapter)) {
                 int ret = 0;
-                struct ibv_context *ctx = NULL;
-                struct ibv_pd *pd = NULL;
+                struct ibv_context *ctx = nullptr;
+                struct ibv_pd *pd = nullptr;
                 mlx5dv_obj mlx5_obj;
 
                 ctx = (ibv_context *)adapter->get_ibv_context();
@@ -350,6 +322,7 @@ dpcp::adapter *ib_ctx_handler::set_dpcp_adapter()
                 m_p_adapter = adapter;
                 m_p_ibv_context = ctx;
                 m_p_ibv_pd = pd;
+                check_capabilities();
                 ibch_logdbg("dpcp adapter: %s is up", adapter->get_name().c_str());
             }
 
@@ -364,11 +337,20 @@ err:
 
     return m_p_adapter;
 }
-#endif /* DEFINED_DPCP */
+
+void ib_ctx_handler::check_capabilities()
+{
+    dpcp::adapter_hca_capabilities caps;
+    dpcp::status rc = m_p_adapter->get_hca_capabilities(caps);
+    if (rc == dpcp::DPCP_OK) {
+        set_flow_tag_capability(caps.flow_table_caps.receive.is_flow_action_tag_supported);
+        ibch_logdbg("Flow Tag Support: %s", get_flow_tag_capability() ? "Yes" : "No");
+    }
+}
 
 void ib_ctx_handler::set_ctx_time_converter_status(ts_conversion_mode_t conversion_mode)
 {
-    if (m_p_ctx_time_converter != NULL) {
+    if (m_p_ctx_time_converter) {
         /*
          * Don't override time_converter object. Current method may be
          * called more than once if multiple slaves point to the same
@@ -434,20 +416,14 @@ void ib_ctx_handler::set_ctx_time_converter_status(ts_conversion_mode_t conversi
 #endif // DEFINED_IBV_CQ_TIMESTAMP
 }
 
-ts_conversion_mode_t ib_ctx_handler::get_ctx_time_converter_status()
-{
-    return m_p_ctx_time_converter ? m_p_ctx_time_converter->get_converter_status()
-                                  : TS_CONVERSION_MODE_DISABLE;
-}
-
 uint32_t ib_ctx_handler::mem_reg(void *addr, size_t length, uint64_t access)
 {
-    struct ibv_mr *mr = NULL;
+    struct ibv_mr *mr = nullptr;
     uint32_t lkey = LKEY_ERROR;
 
     mr = ibv_reg_mr(m_p_ibv_pd, addr, length, access);
     VALGRIND_MAKE_MEM_DEFINED(mr, sizeof(ibv_mr));
-    if (NULL == mr) {
+    if (!mr) {
         print_warning_rlimit_memlock(length, errno);
     } else {
         m_mr_map_lkey[mr->lkey] = mr;
@@ -486,7 +462,7 @@ struct ibv_mr *ib_ctx_handler::get_mem_reg(uint32_t lkey)
         return iter->second;
     }
 
-    return NULL;
+    return nullptr;
 }
 
 uint32_t ib_ctx_handler::user_mem_reg(void *addr, size_t length, uint64_t access)
@@ -573,6 +549,6 @@ void ib_ctx_handler::handle_event_device_fatal()
     g_p_event_handler_manager->unregister_ibverbs_event(m_p_ibv_context->async_fd, this);
     if (m_p_ctx_time_converter) {
         m_p_ctx_time_converter->clean_obj();
-        m_p_ctx_time_converter = NULL;
+        m_p_ctx_time_converter = nullptr;
     }
 }
