@@ -1,5 +1,7 @@
 /*
- * Copyright (c) 2001-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
+ * Copyright (c) 2001-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: GPL-2.0-only or BSD-3-Clause
  *
  * This software is available to you under a choice of one of two
  * licenses.  You may choose to be licensed under the terms of the GNU
@@ -61,7 +63,7 @@
 #define ndtm_logfunc    __log_info_func
 #define ndtm_logfuncall __log_info_funcall
 
-net_device_table_mgr *g_p_net_device_table_mgr = NULL;
+net_device_table_mgr *g_p_net_device_table_mgr = nullptr;
 
 enum net_device_table_mgr_timers { RING_PROGRESS_ENGINE_TIMER, RING_ADAPT_CQ_MODERATION_TIMER };
 
@@ -76,7 +78,7 @@ net_device_table_mgr::net_device_table_mgr()
 
     ndtm_logdbg("");
 
-    m_global_ring_epfd = orig_os_api.epoll_create(48);
+    m_global_ring_epfd = SYSCALL(epoll_create, 48);
 
     BULLSEYE_EXCLUDE_BLOCK_START
     if (m_global_ring_epfd == -1) {
@@ -85,12 +87,12 @@ net_device_table_mgr::net_device_table_mgr()
         throw_xlio_exception("epoll_create failed");
     }
 
-    if (orig_os_api.pipe(m_global_ring_pipe_fds)) {
+    if (SYSCALL(pipe, m_global_ring_pipe_fds)) {
         ndtm_logerr("pipe create failed. (errno=%d %m)", errno);
         free_ndtm_resources();
         throw_xlio_exception("pipe create failed");
     }
-    if (orig_os_api.write(m_global_ring_pipe_fds[1], "#", 1) != 1) {
+    if (SYSCALL(write, m_global_ring_pipe_fds[1], "#", 1) != 1) {
         ndtm_logerr("pipe write failed. (errno=%d %m)", errno);
         free_ndtm_resources();
         throw_xlio_exception("pipe write failed");
@@ -103,7 +105,7 @@ net_device_table_mgr::net_device_table_mgr()
     /* throw exception if there are no supported devices. */
     if (m_net_device_map_index.empty()) {
         int num_devices = 0;
-        struct ibv_device **dev_list = NULL;
+        struct ibv_device **dev_list = nullptr;
         dev_list = xlio_ibv_get_device_list(&num_devices);
         if (dev_list && num_devices == 0) {
             ibv_free_device_list(dev_list);
@@ -151,12 +153,12 @@ void net_device_table_mgr::free_ndtm_resources()
     m_lock.lock();
 
     if (m_global_ring_epfd > 0) {
-        orig_os_api.close(m_global_ring_epfd);
+        SYSCALL(close, m_global_ring_epfd);
         m_global_ring_epfd = 0;
     }
 
-    orig_os_api.close(m_global_ring_pipe_fds[1]);
-    orig_os_api.close(m_global_ring_pipe_fds[0]);
+    SYSCALL(close, m_global_ring_pipe_fds[1]);
+    SYSCALL(close, m_global_ring_pipe_fds[0]);
 
     net_device_map_index_t::iterator itr;
     while ((itr = m_net_device_map_index.begin()) != m_net_device_map_index.end()) {
@@ -191,7 +193,7 @@ void net_device_table_mgr::update_tbl()
     net_device_val *p_net_device_val;
 
     /* Set up the netlink socket */
-    fd = orig_os_api.socket(AF_NETLINK, SOCK_RAW, NETLINK_ROUTE);
+    fd = SYSCALL(socket, AF_NETLINK, SOCK_RAW, NETLINK_ROUTE);
     if (fd < 0) {
         ndtm_logerr("netlink socket() creation");
         return;
@@ -210,7 +212,7 @@ void net_device_table_mgr::update_tbl()
     nl_req.infomsg.ifi_change = 0xffffffff;
 
     /* Send the netlink request */
-    rc = orig_os_api.send(fd, &nl_req, nl_req.hdr.nlmsg_len, 0);
+    rc = SYSCALL(send, fd, &nl_req, nl_req.hdr.nlmsg_len, 0);
     if (rc < 0) {
         ndtm_logerr("netlink send() operation");
         goto ret;
@@ -220,7 +222,7 @@ void net_device_table_mgr::update_tbl()
 
     do {
         /* Receive the netlink reply */
-        rc = orig_os_api.recv(fd, nl_res, sizeof(nl_res), 0);
+        rc = SYSCALL(recv, fd, nl_res, sizeof(nl_res), 0);
         if (rc < 0) {
             ndtm_logerr("netlink recv() operation");
             goto ret;
@@ -296,7 +298,7 @@ ret:
     ndtm_logdbg("Check completed. Found %ld offload capable network interfaces",
                 m_net_device_map_index.size());
 
-    orig_os_api.close(fd);
+    SYSCALL(close, fd);
 }
 
 void net_device_table_mgr::print_val_tbl()
@@ -323,18 +325,18 @@ net_device_val *net_device_table_mgr::get_net_device_val(const ip_addr &if_addr)
         ndtm_logdbg("Found %s for addr: %s", net_dev->to_str().c_str(), if_addr.to_str().c_str());
         if (net_dev->get_state() == net_device_val::INVALID) {
             ndtm_logdbg("invalid net_device %s", net_dev->to_str().c_str());
-            return NULL;
+            return nullptr;
         }
         return iter->second;
     }
     ndtm_logdbg("Can't find net_device for addr: %s", if_addr.to_str().c_str());
-    return NULL;
+    return nullptr;
 }
 
 net_device_val *net_device_table_mgr::get_net_device_val(int if_index)
 {
     net_device_map_index_t::iterator iter;
-    net_device_val *net_dev = NULL;
+    net_device_val *net_dev = nullptr;
 
     std::lock_guard<decltype(m_lock)> lock(m_lock);
 
@@ -362,9 +364,9 @@ net_device_val *net_device_table_mgr::get_net_device_val(int if_index)
                                net_dev->get_ifname());
                 if (ret > 0 && (size_t)ret < sizeof(sys_path)) {
                     ret = errno; /* to suppress errno */
-                    int fd = open(sys_path, O_RDONLY);
+                    int fd = SYSCALL(open, sys_path, O_RDONLY);
                     if (fd >= 0) {
-                        close(fd);
+                        SYSCALL(close, fd);
                         goto out;
                     }
                     errno = ret;
@@ -374,14 +376,14 @@ net_device_val *net_device_table_mgr::get_net_device_val(int if_index)
     }
 
     ndtm_logdbg("Can't find net_device for index: %d", if_index);
-    return NULL;
+    return nullptr;
 
 out:
 
     ndtm_logdbg("Found %s for index: %d", net_dev->to_str().c_str(), if_index);
     if (net_dev->get_state() == net_device_val::INVALID) {
         ndtm_logdbg("invalid net_device %s", net_dev->to_str().c_str());
-        return NULL;
+        return nullptr;
     }
     return net_dev;
 }
@@ -396,7 +398,7 @@ net_device_entry *net_device_table_mgr::create_new_entry(int if_index, const obs
     if (p_ndv) {
         return new net_device_entry(if_index, p_ndv);
     }
-    return NULL;
+    return nullptr;
 }
 
 void net_device_table_mgr::get_ip_list(local_ip_list_t &ip_list, sa_family_t family, int if_index)
@@ -420,40 +422,31 @@ void net_device_table_mgr::get_ip_list(local_ip_list_t &ip_list, sa_family_t fam
     m_lock.unlock();
 }
 
-int net_device_table_mgr::global_ring_poll_and_process_element(uint64_t *p_poll_sn,
-                                                               void *pv_fd_ready_array /*= NULL*/)
+bool net_device_table_mgr::global_ring_poll_and_process_element(uint64_t *p_poll_sn_rx,
+                                                                uint64_t *p_poll_sn_tx,
+                                                                void *pv_fd_ready_array /*= NULL*/)
 {
     ndtm_logfunc("");
-    int ret_total = 0;
+    bool all_drained = true;
 
     net_device_map_index_t::iterator net_dev_iter;
     for (net_dev_iter = m_net_device_map_index.begin();
          net_dev_iter != m_net_device_map_index.end(); net_dev_iter++) {
-        int ret = net_dev_iter->second->global_ring_poll_and_process_element(p_poll_sn,
-                                                                             pv_fd_ready_array);
-        if (ret < 0) {
-            ndtm_logdbg("Error in net_device_val[%p]->poll_and_process_element() (errno=%d %m)",
-                        net_dev_iter->second, errno);
-            return ret;
-        }
-        ret_total += ret;
+        all_drained &= net_dev_iter->second->global_ring_poll_and_process_element(
+            p_poll_sn_rx, p_poll_sn_tx, pv_fd_ready_array);
     }
-    if (ret_total) {
-        ndtm_logfunc("ret_total=%d", ret_total);
-    } else {
-        ndtm_logfuncall("ret_total=%d", ret_total);
-    }
-    return ret_total;
+
+    return all_drained;
 }
 
-int net_device_table_mgr::global_ring_request_notification(uint64_t poll_sn)
+int net_device_table_mgr::global_ring_request_notification(uint64_t poll_sn_rx, uint64_t poll_sn_tx)
 {
     ndtm_logfunc("");
     int ret_total = 0;
     net_device_map_index_t::iterator net_dev_iter;
     for (net_dev_iter = m_net_device_map_index.begin();
          m_net_device_map_index.end() != net_dev_iter; net_dev_iter++) {
-        int ret = net_dev_iter->second->global_ring_request_notification(poll_sn);
+        int ret = net_dev_iter->second->global_ring_request_notification(poll_sn_rx, poll_sn_tx);
         BULLSEYE_EXCLUDE_BLOCK_START
         if (ret < 0) {
             ndtm_logerr("Error in net_device_val[%p]->request_notification() (errno=%d %m)",
@@ -471,15 +464,14 @@ int net_device_table_mgr::global_ring_epfd_get()
     return m_global_ring_epfd;
 }
 
-int net_device_table_mgr::global_ring_wait_for_notification_and_process_element(
+void net_device_table_mgr::global_ring_wait_for_notification_and_process_element(
     uint64_t *p_poll_sn, void *pv_fd_ready_array /*=NULL*/)
 {
     ndtm_logfunc("");
-    int ret_total = 0;
     int max_fd = 16;
     struct epoll_event events[max_fd];
 
-    int res = orig_os_api.epoll_wait(global_ring_epfd_get(), events, max_fd, 0);
+    int res = SYSCALL(epoll_wait, global_ring_epfd_get(), events, max_fd, 0);
     if (res > 0) {
         for (int event_idx = 0; event_idx < res; ++event_idx) {
             int fd = events[event_idx].data.fd; // This is the Rx cq channel fd
@@ -488,32 +480,13 @@ int net_device_table_mgr::global_ring_wait_for_notification_and_process_element(
             if (p_cq_ch_info) {
                 ring *p_ready_ring = p_cq_ch_info->get_ring();
                 // Handle the CQ notification channel
-                int ret = p_ready_ring->wait_for_notification_and_process_element(
-                    fd, p_poll_sn, pv_fd_ready_array);
-                if (ret < 0) {
-                    if (errno == EAGAIN) {
-                        ndtm_logdbg(
-                            "Error in ring[%d]->wait_for_notification_and_process_element() of %p "
-                            "(errno=%d %m)",
-                            event_idx, p_ready_ring, errno);
-                    } else {
-                        ndtm_logerr(
-                            "Error in ring[%d]->wait_for_notification_and_process_element() of %p "
-                            "(errno=%d %m)",
-                            event_idx, p_ready_ring, errno);
-                    }
-                    continue;
-                }
-                if (ret > 0) {
-                    ndtm_logfunc("ring[%p] Returned with: %d (sn=%d)", p_ready_ring, ret,
-                                 *p_poll_sn);
-                }
-                ret_total += ret;
+                p_ready_ring->wait_for_notification_and_process_element(p_poll_sn,
+                                                                        pv_fd_ready_array);
             } else {
                 ndtm_logdbg("removing wakeup fd from epfd");
                 BULLSEYE_EXCLUDE_BLOCK_START
-                if ((orig_os_api.epoll_ctl(m_global_ring_epfd, EPOLL_CTL_DEL,
-                                           m_global_ring_pipe_fds[0], NULL)) &&
+                if ((SYSCALL(epoll_ctl, m_global_ring_epfd, EPOLL_CTL_DEL,
+                             m_global_ring_pipe_fds[0], nullptr)) &&
                     (!(errno == ENOENT || errno == EBADF))) {
                     ndtm_logerr("failed to del pipe channel fd from internal epfd (errno=%d %m)",
                                 errno);
@@ -522,12 +495,6 @@ int net_device_table_mgr::global_ring_wait_for_notification_and_process_element(
             }
         }
     }
-    if (ret_total) {
-        ndtm_logfunc("ret_total=%d", ret_total);
-    } else {
-        ndtm_logfuncall("ret_total=%d", ret_total);
-    }
-    return ret_total;
 }
 
 int net_device_table_mgr::global_ring_drain_and_procces()
@@ -582,14 +549,13 @@ void net_device_table_mgr::handle_timer_expired(void *user_data)
 void net_device_table_mgr::global_ring_wakeup()
 {
     ndtm_logdbg("");
-    epoll_event ev = {0, {0}};
+    epoll_event ev = {0, {nullptr}};
 
     ev.events = EPOLLIN;
-    ev.data.ptr = NULL;
+    ev.data.ptr = nullptr;
     int errno_tmp = errno; // don't let wakeup affect errno, as this can fail with EEXIST
     BULLSEYE_EXCLUDE_BLOCK_START
-    if ((orig_os_api.epoll_ctl(m_global_ring_epfd, EPOLL_CTL_ADD, m_global_ring_pipe_fds[0],
-                               &ev)) &&
+    if ((SYSCALL(epoll_ctl, m_global_ring_epfd, EPOLL_CTL_ADD, m_global_ring_pipe_fds[0], &ev)) &&
         (errno != EEXIST)) {
         ndtm_logerr("failed to add pipe channel fd to internal epfd (errno=%d %m)", errno);
     }
@@ -617,7 +583,7 @@ void net_device_table_mgr::del_link_event(const netlink_link_info *info)
      * resources correctly.
      */
     if (info->flags & IFF_SLAVE) {
-        net_device_val *net_dev = NULL;
+        net_device_val *net_dev = nullptr;
         int if_index = info->ifindex;
 
         ndtm_logdbg("netlink event: if_index: %d state: %s", info->ifindex,
@@ -642,7 +608,7 @@ void net_device_table_mgr::new_link_event(const netlink_link_info *info)
      * DOWN state (see RTM_DELLINK).
      */
     if (info->flags & IFF_SLAVE) {
-        net_device_val *net_dev = NULL;
+        net_device_val *net_dev = nullptr;
         int if_index = info->ifindex;
 
         ndtm_logdbg("netlink event: if_index: %d state: %s", info->ifindex,
