@@ -1,5 +1,7 @@
 /*
- * Copyright (c) 2001-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
+ * Copyright (c) 2001-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: GPL-2.0-only or BSD-3-Clause
  *
  * This software is available to you under a choice of one of two
  * licenses.  You may choose to be licensed under the terms of the GNU
@@ -47,11 +49,13 @@
 // AF_INET address 0.0.0.0:0, used for 3T flow spec keys.
 static const sock_addr s_sock_addrany;
 
+static thread_local lock_dummy t_lock_dummy_ring;
+
 static lock_base *get_new_lock(const char *name, bool real_lock)
 {
     return (real_lock
                 ? static_cast<lock_base *>(multilock::create_new_lock(MULTILOCK_RECURSIVE, name))
-                : static_cast<lock_base *>(new lock_dummy()));
+                : static_cast<lock_base *>(&t_lock_dummy_ring));
 }
 
 ring_slave::ring_slave(int if_index, ring *parent, ring_type_t type, bool use_locks)
@@ -61,14 +65,14 @@ ring_slave::ring_slave(int if_index, ring *parent, ring_type_t type, bool use_lo
     , m_lock_ring_rx(get_new_lock("ring_slave:lock_rx", use_locks))
     , m_lock_ring_tx(get_new_lock("ring_slave:lock_tx", use_locks))
     , m_p_ring_stat(new ring_stats_t)
-    , m_partition(0)
+    , m_vlan(0)
     , m_flow_tag_enabled(false)
     , m_b_sysvar_eth_mc_l2_only_rules(safe_mce_sys().eth_mc_l2_only_rules)
     , m_b_sysvar_mc_force_flowtag(safe_mce_sys().mc_force_flowtag)
     , m_type(type)
 {
-    net_device_val *p_ndev = NULL;
-    const slave_data_t *p_slave = NULL;
+    net_device_val *p_ndev = nullptr;
+    const slave_data_t *p_slave = nullptr;
 
     /* Configure ring() fields */
     set_parent(parent);
@@ -76,7 +80,7 @@ ring_slave::ring_slave(int if_index, ring *parent, ring_type_t type, bool use_lo
 
     /* Sanity check */
     p_ndev = g_p_net_device_table_mgr->get_net_device_val(m_parent->get_if_index());
-    if (NULL == p_ndev) {
+    if (!p_ndev) {
         ring_logpanic("Invalid if_index = %d", if_index);
     }
 
@@ -122,7 +126,8 @@ ring_slave::~ring_slave()
 void ring_slave::print_val()
 {
     ring_logdbg("%d: %p: parent %p type %s", m_if_index, this,
-                ((uintptr_t)this == (uintptr_t)m_parent ? 0 : m_parent), ring_type_str[m_type]);
+                ((uintptr_t)this == (uintptr_t)m_parent ? nullptr : m_parent),
+                ring_type_str[m_type]);
 }
 
 void ring_slave::restart()
@@ -157,14 +162,14 @@ void ring_slave::inc_tx_retransmissions_stats(ring_user_id_t)
 }
 
 template <typename KEY4T, typename KEY2T, typename HDR>
-bool steering_handler<KEY4T, KEY2T, HDR>::attach_flow(flow_tuple &flow_spec_5t, pkt_rcvr_sink *sink,
+bool steering_handler<KEY4T, KEY2T, HDR>::attach_flow(flow_tuple &flow_spec_5t, sockinfo *sink,
                                                       bool force_5t)
 {
     rfs *p_rfs;
-    rfs *p_tmp_rfs = NULL;
+    rfs *p_tmp_rfs = nullptr;
     sockinfo *si = static_cast<sockinfo *>(sink);
 
-    if (si == NULL) {
+    if (!si) {
         return false;
     }
 
@@ -184,7 +189,7 @@ bool steering_handler<KEY4T, KEY2T, HDR>::attach_flow(flow_tuple &flow_spec_5t,
                       flow_spec_5t.get_dst_port(), flow_spec_5t.get_src_port());
         sock_addr rule_key(flow_spec_5t.get_family(), &flow_spec_5t.get_dst_ip(),
                            flow_spec_5t.get_dst_port());
-        rfs_rule_filter *dst_port_filter = NULL;
+        rfs_rule_filter *dst_port_filter = nullptr;
         if (safe_mce_sys().udp_3t_rules) {
             auto dst_port_iter = m_ring.m_udp_uc_dst_port_attach_map.find(rule_key);
             if (dst_port_iter == m_ring.m_udp_uc_dst_port_attach_map.end()) {
@@ -220,7 +225,7 @@ bool steering_handler<KEY4T, KEY2T, HDR>::attach_flow(flow_tuple &flow_spec_5t,
                 return false;
             }
             BULLSEYE_EXCLUDE_BLOCK_START
-            if (p_tmp_rfs == NULL) {
+            if (!p_tmp_rfs) {
                 ring_logerr("Failed to allocate rfs!");
                 return false;
             }
@@ -257,7 +262,7 @@ bool steering_handler<KEY4T, KEY2T, HDR>::attach_flow(flow_tuple &flow_spec_5t,
         // It means that for every MC group, even if we have sockets with different ports - only one
         // rule in the HW. So the hash map below keeps track of the number of sockets per rule so we
         // know when to call ibv_attach and ibv_detach
-        rfs_rule_filter *l2_mc_ip_filter = NULL;
+        rfs_rule_filter *l2_mc_ip_filter = nullptr;
         if (m_ring.m_b_sysvar_eth_mc_l2_only_rules) {
             auto l2_mc_iter = m_ring.m_l2_mc_ip_attach_map.find(rule_key);
             // It means that this is the first time attach called with this MC ip
@@ -297,8 +302,11 @@ bool steering_handler<KEY4T, KEY2T, HDR>::attach_flow(flow_tuple &flow_spec_5t,
                       flow_spec_5t.get_dst_port(), flow_spec_5t.get_src_port());
         sock_addr rule_key(flow_spec_5t.get_family(), &flow_spec_5t.get_dst_ip(),
                            flow_spec_5t.get_dst_port());
-        rfs_rule_filter *dst_port_filter = NULL;
-        if (safe_mce_sys().tcp_3t_rules) {
+        rfs_rule_filter *dst_port_filter = nullptr;
+        if (safe_mce_sys().tcp_3t_rules || safe_mce_sys().tcp_2t_rules) {
+            if (safe_mce_sys().tcp_2t_rules) {
+                rule_key.set_in_port(0);
+            }
             auto dst_port_iter = m_ring.m_tcp_dst_port_attach_map.find(rule_key);
             if (dst_port_iter == m_ring.m_tcp_dst_port_attach_map.end()) {
                 m_ring.m_tcp_dst_port_attach_map[rule_key].counter = 1;
@@ -309,7 +317,8 @@ bool steering_handler<KEY4T, KEY2T, HDR>::attach_flow(flow_tuple &flow_spec_5t,
         }
 
         if (flow_tag_id &&
-            (flow_spec_5t.is_3_tuple() || (!force_5t && safe_mce_sys().tcp_3t_rules))) {
+            (flow_spec_5t.is_3_tuple() || (!force_5t && safe_mce_sys().tcp_3t_rules) ||
+             safe_mce_sys().tcp_2t_rules)) {
             ring_logdbg("flow tag id = %d is disabled for socket fd = %d to be processed on RFS!",
                         flow_tag_id, si->get_fd());
             flow_tag_id = FLOW_TAG_MASK;
@@ -319,7 +328,12 @@ bool steering_handler<KEY4T, KEY2T, HDR>::attach_flow(flow_tuple &flow_spec_5t,
         if (itr == end(m_flow_tcp_map)) {
             // It means that no rfs object exists so I need to create a new one and insert it to
             // the flow map
-            if (!force_5t && safe_mce_sys().tcp_3t_rules) {
+            if (safe_mce_sys().tcp_2t_rules) {
+                flow_tuple tcp_2t_only(flow_spec_5t.get_dst_ip(), 0, ip_address::any_addr(), 0,
+                                       flow_spec_5t.get_protocol(), flow_spec_5t.get_family());
+                dst_port_filter =
+                    new rfs_rule_filter(m_ring.m_tcp_dst_port_attach_map, rule_key, tcp_2t_only);
+            } else if (!force_5t && safe_mce_sys().tcp_3t_rules) {
                 flow_tuple tcp_3t_only(flow_spec_5t.get_dst_ip(), flow_spec_5t.get_dst_port(),
                                        ip_address::any_addr(), 0, flow_spec_5t.get_protocol(),
                                        flow_spec_5t.get_family());
@@ -339,14 +353,14 @@ bool steering_handler<KEY4T, KEY2T, HDR>::attach_flow(flow_tuple &flow_spec_5t,
                 return false;
             }
             BULLSEYE_EXCLUDE_BLOCK_START
-            if (p_tmp_rfs == NULL) {
+            if (!p_tmp_rfs) {
                 ring_logerr("Failed to allocate rfs!");
                 return false;
             }
             BULLSEYE_EXCLUDE_BLOCK_END
 
             p_rfs = p_tmp_rfs;
-            si->rfs_ptr = p_rfs;
+            si->set_rfs_ptr(p_rfs);
 #if defined(DEFINED_NGINX) || defined(DEFINED_ENVOY)
             if (g_p_app->type == APP_NONE || !g_p_app->add_second_4t_rule)
 #endif
@@ -386,7 +400,7 @@ bool steering_handler<KEY4T, KEY2T, HDR>::attach_flow(flow_tuple &flow_spec_5t,
     return ret;
 }
 
-bool ring_slave::attach_flow(flow_tuple &flow_spec_5t, pkt_rcvr_sink *sink, bool force_5t)
+bool ring_slave::attach_flow(flow_tuple &flow_spec_5t, sockinfo *sink, bool force_5t)
 {
     std::lock_guard<decltype(m_lock_ring_rx)> lock(m_lock_ring_rx);
 
@@ -396,9 +410,9 @@ bool ring_slave::attach_flow(flow_tuple &flow_spec_5t, pkt_rcvr_sink *sink, bool
 }
 
 template <typename KEY4T, typename KEY2T, typename HDR>
-bool steering_handler<KEY4T, KEY2T, HDR>::detach_flow(flow_tuple &flow_spec_5t, pkt_rcvr_sink *sink)
+bool steering_handler<KEY4T, KEY2T, HDR>::detach_flow(flow_tuple &flow_spec_5t, sockinfo *sink)
 {
-    rfs *p_rfs = NULL;
+    rfs *p_rfs = nullptr;
 
     ring_logdbg("flow: %s, with sink (%p)", flow_spec_5t.to_str().c_str(), sink);
 
@@ -480,7 +494,7 @@ bool steering_handler<KEY4T, KEY2T, HDR>::detach_flow(flow_tuple &flow_spec_5t,
                       flow_spec_5t.get_dst_port(), flow_spec_5t.get_src_port());
         sock_addr rule_key(flow_spec_5t.get_family(), &flow_spec_5t.get_dst_ip(),
                            flow_spec_5t.get_dst_port());
-        if (safe_mce_sys().tcp_3t_rules) {
+        if (safe_mce_sys().tcp_3t_rules || safe_mce_sys().tcp_2t_rules) {
             auto dst_port_iter = m_ring.m_tcp_dst_port_attach_map.find(rule_key);
             BULLSEYE_EXCLUDE_BLOCK_START
             if (dst_port_iter == m_ring.m_tcp_dst_port_attach_map.end()) {
@@ -520,7 +534,7 @@ bool steering_handler<KEY4T, KEY2T, HDR>::detach_flow(flow_tuple &flow_spec_5t,
     return true;
 }
 
-bool ring_slave::detach_flow(flow_tuple &flow_spec_5t, pkt_rcvr_sink *sink)
+bool ring_slave::detach_flow(flow_tuple &flow_spec_5t, sockinfo *sink)
 {
     std::lock_guard<decltype(m_lock_ring_rx)> lock(m_lock_ring_rx);
 
@@ -583,7 +597,7 @@ bool ring_slave::rx_process_buffer(mem_buf_desc_t *p_rx_wc_buf_desc, void *pv_fd
         return false;
     }
 
-    inc_cq_moderation_stats(sz_data);
+    inc_cq_moderation_stats();
 
     m_p_ring_stat->n_rx_byte_count += sz_data;
     ++m_p_ring_stat->n_rx_pkt_count;
@@ -592,14 +606,14 @@ bool ring_slave::rx_process_buffer(mem_buf_desc_t *p_rx_wc_buf_desc, void *pv_fd
     if (likely(m_flow_tag_enabled && p_rx_wc_buf_desc->rx.flow_tag_id &&
                p_rx_wc_buf_desc->rx.flow_tag_id != FLOW_TAG_MASK &&
                !p_rx_wc_buf_desc->rx.is_sw_csum_need)) {
-        sockinfo *si = NULL;
+        sockinfo *si = nullptr;
         // trying to get sockinfo per flow_tag_id-1 as it was incremented at attach
         // to allow mapping sockfd=0
         assert(g_p_fd_collection);
         si = static_cast<sockinfo *>(
             g_p_fd_collection->get_sockfd(p_rx_wc_buf_desc->rx.flow_tag_id - 1));
 
-        if (likely((si != NULL) && si->flow_tag_enabled())) {
+        if (likely(si)) {
             // will process packets with set flow_tag_id and enabled for the socket
             if (p_eth_h->h_proto == NET_ETH_P_8021Q) {
                 // Handle VLAN header as next protocol
@@ -643,10 +657,6 @@ bool ring_slave::rx_process_buffer(mem_buf_desc_t *p_rx_wc_buf_desc, void *pv_fd
             if (likely(protocol == IPPROTO_TCP)) {
                 struct tcphdr *p_tcp_h = (struct tcphdr *)((uint8_t *)p_ip_h + ip_hdr_len);
 
-                // Update the L3 and L4 info
-                p_rx_wc_buf_desc->rx.src.set_ip_port(family, saddr, p_tcp_h->source);
-                p_rx_wc_buf_desc->rx.dst.set_ip_port(family, daddr, p_tcp_h->dest);
-
                 // Update packet descriptor with datagram base address and length
                 p_rx_wc_buf_desc->rx.frag.iov_base = (uint8_t *)p_tcp_h + sizeof(struct tcphdr);
                 p_rx_wc_buf_desc->rx.frag.iov_len = ip_payload_len - sizeof(struct tcphdr);
@@ -665,7 +675,7 @@ bool ring_slave::rx_process_buffer(mem_buf_desc_t *p_rx_wc_buf_desc, void *pv_fd
                              p_tcp_h->fin ? "F" : "", ntohl(p_tcp_h->seq), ntohl(p_tcp_h->ack_seq),
                              ntohs(p_tcp_h->window), p_rx_wc_buf_desc->rx.sz_payload);
 
-                return si->rfs_ptr->rx_dispatch_packet(p_rx_wc_buf_desc, pv_fd_ready_array);
+                return si->get_rfs_ptr()->rx_dispatch_packet(p_rx_wc_buf_desc, pv_fd_ready_array);
             }
 
             if (likely(protocol == IPPROTO_UDP)) {
@@ -706,7 +716,7 @@ bool ring_slave::rx_process_buffer(mem_buf_desc_t *p_rx_wc_buf_desc, void *pv_fd
                      ETH_HW_ADDR_PRINT_ADDR(p_eth_h->h_source), htons(h_proto));
 
         // Handle VLAN header as next protocol
-        struct vlanhdr *p_vlan_hdr = NULL;
+        struct vlanhdr *p_vlan_hdr = nullptr;
         uint16_t packet_vlan = 0;
         if (h_proto == NET_ETH_P_8021Q) {
             p_vlan_hdr = (struct vlanhdr *)((uint8_t *)p_eth_h + ETH_HDR_LEN);
@@ -721,9 +731,9 @@ bool ring_slave::rx_process_buffer(mem_buf_desc_t *p_rx_wc_buf_desc, void *pv_fd
 
         // TODO: Remove this code when handling vlan in flow steering will be available. Change this
         // code if vlan stripping is performed.
-        if ((m_partition & VLAN_VID_MASK) != packet_vlan) {
+        if ((m_vlan & VLAN_VID_MASK) != packet_vlan) {
             ring_logfunc("Rx buffer dropped- Mismatched vlan. Packet vlan = %d, Local vlan = %d",
-                         packet_vlan, m_partition & VLAN_VID_MASK);
+                         packet_vlan, m_vlan & VLAN_VID_MASK);
             return false;
         }
 
@@ -965,7 +975,7 @@ bool steering_handler<KEY4T, KEY2T, HDR>::rx_process_buffer_no_flow_id(
         p_rx_wc_buf_desc->rx.frag.iov_len = ip_tot_len - hdr_data.ip_hdr_len;
 
         // Add ip fragment packet to out fragment manager
-        mem_buf_desc_t *new_buf = NULL;
+        mem_buf_desc_t *new_buf = nullptr;
         int ret = -1;
         if (g_p_ip_frag_manager) {
             ret = g_p_ip_frag_manager->add_frag(p_ip_h, p_rx_wc_buf_desc, &new_buf);
