@@ -1,5 +1,7 @@
 /*
- * Copyright (c) 2001-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
+ * Copyright (c) 2001-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: GPL-2.0-only or BSD-3-Clause
  *
  * This software is available to you under a choice of one of two
  * licenses.  You may choose to be licensed under the terms of the GNU
@@ -45,7 +47,7 @@
 
 int epfd_info::remove_fd_from_epoll_os(int fd)
 {
-    int ret = orig_os_api.epoll_ctl(m_epfd, EPOLL_CTL_DEL, fd, NULL);
+    int ret = SYSCALL(epoll_ctl, m_epfd, EPOLL_CTL_DEL, fd, nullptr);
     BULLSEYE_EXCLUDE_BLOCK_START
     if (ret < 0) {
         __log_dbg("failed to remove fd=%d from os epoll epfd=%d (errno=%d %m)", fd, m_epfd, errno);
@@ -59,9 +61,6 @@ epfd_info::epfd_info(int epfd, int size)
     , m_epfd(epfd)
     , m_size(size)
     , m_ring_map_lock("epfd_ring_map_lock")
-    , m_lock_poll_os(MULTILOCK_NON_RECURSIVE, "epfd_lock_poll_os")
-    , m_sysvar_thread_mode(safe_mce_sys().thread_mode)
-    , m_b_os_data_available(false)
 {
     __log_funcall("");
     int max_sys_fd = get_sys_max_fd_num();
@@ -89,17 +88,13 @@ epfd_info::epfd_info(int epfd, int size)
 
     xlio_stats_instance_create_epoll_block(m_epfd, &(m_stats->stats));
 
-    // Register this socket to read nonoffloaded data
-    g_p_event_handler_manager->update_epfd(m_epfd, EPOLL_CTL_ADD,
-                                           EPOLLIN | EPOLLPRI | EPOLLONESHOT);
-
     wakeup_set_epoll_fd(m_epfd);
 }
 
 epfd_info::~epfd_info()
 {
     __log_funcall("");
-    socket_fd_api *sock_fd;
+    sockinfo *sock_fd;
 
     // Meny: going over all handled fds and removing epoll context.
 
@@ -133,9 +128,6 @@ epfd_info::~epfd_info()
         BULLSEYE_EXCLUDE_BLOCK_END
     }
 
-    g_p_event_handler_manager->update_epfd(m_epfd, EPOLL_CTL_DEL,
-                                           EPOLLIN | EPOLLPRI | EPOLLONESHOT);
-
     unlock();
 
     xlio_stats_instance_remove_epoll_block(&m_stats->stats);
@@ -146,13 +138,13 @@ int epfd_info::ctl(int op, int fd, epoll_event *event)
 {
     int ret;
     epoll_event event_dummy;
-    if (event == NULL) {
+    if (!event) {
         memset(&event_dummy, 0, sizeof(event_dummy));
         event = &event_dummy;
     }
 
     // YossiE TODO make "event table" - and add index in that table instead
-    // of real event (in orig_os_api.epoll_ctl). must have this because fd's can
+    // of real event (in SYSCALL(epoll_ctl)). must have this because fd's can
     // be added after the cq.
     lock();
 
@@ -201,13 +193,13 @@ int epfd_info::add_fd(int fd, epoll_event *event)
 {
     int ret;
     epoll_fd_rec fd_rec;
-    epoll_event evt = {0, {0}};
+    epoll_event evt = {0, {nullptr}};
 
     bool is_offloaded = false;
 
     __log_funcall("fd=%d", fd);
 
-    socket_fd_api *temp_sock_fd_api = fd_collection_get_sockfd(fd);
+    sockinfo *temp_sock_fd_api = fd_collection_get_sockfd(fd);
     if (temp_sock_fd_api && temp_sock_fd_api->get_type() == FD_TYPE_SOCKET) {
         is_offloaded = true;
     }
@@ -237,7 +229,7 @@ int epfd_info::add_fd(int fd, epoll_event *event)
         evt.events = event->events;
         evt.data.u64 = 0; // zero all data
         evt.data.fd = fd;
-        ret = orig_os_api.epoll_ctl(m_epfd, EPOLL_CTL_ADD, fd, &evt);
+        ret = SYSCALL(epoll_ctl, m_epfd, EPOLL_CTL_ADD, fd, &evt);
         BULLSEYE_EXCLUDE_BLOCK_START
         if (ret < 0) {
             __log_dbg("failed to add fd=%d to epoll epfd=%d (errno=%d %m)", fd, m_epfd, errno);
@@ -294,7 +286,7 @@ int epfd_info::add_fd(int fd, epoll_event *event)
         // if the socket is ready, add it to ready events
         uint32_t events = 0;
         int errors;
-        if ((event->events & EPOLLIN) && temp_sock_fd_api->is_readable(NULL, NULL)) {
+        if ((event->events & EPOLLIN) && temp_sock_fd_api->is_readable(nullptr, nullptr)) {
             events |= EPOLLIN;
         }
         if ((event->events & EPOLLOUT) && temp_sock_fd_api->is_writeable()) {
@@ -337,11 +329,11 @@ void epfd_info::increase_ring_ref_count(ring *ring)
         size_t num_ring_rx_fds;
         int *ring_rx_fds_array = ring->get_rx_channel_fds(num_ring_rx_fds);
         for (size_t i = 0; i < num_ring_rx_fds; i++) {
-            epoll_event evt = {0, {0}};
+            epoll_event evt = {0, {nullptr}};
             evt.events = EPOLLIN | EPOLLPRI;
             int fd = ring_rx_fds_array[i];
             evt.data.u64 = (((uint64_t)CQ_FD_MARK << 32) | fd);
-            int ret = orig_os_api.epoll_ctl(m_epfd, EPOLL_CTL_ADD, fd, &evt);
+            int ret = SYSCALL(epoll_ctl, m_epfd, EPOLL_CTL_ADD, fd, &evt);
             BULLSEYE_EXCLUDE_BLOCK_START
             if (ret < 0) {
                 __log_dbg("failed to add cq fd=%d to epoll epfd=%d (errno=%d %m)", fd, m_epfd,
@@ -378,7 +370,7 @@ void epfd_info::decrease_ring_ref_count(ring *ring)
         int *ring_rx_fds_array = ring->get_rx_channel_fds(num_ring_rx_fds);
         for (size_t i = 0; i < num_ring_rx_fds; i++) {
             // delete cq fd from epfd
-            int ret = orig_os_api.epoll_ctl(m_epfd, EPOLL_CTL_DEL, ring_rx_fds_array[i], NULL);
+            int ret = SYSCALL(epoll_ctl, m_epfd, EPOLL_CTL_DEL, ring_rx_fds_array[i], nullptr);
             BULLSEYE_EXCLUDE_BLOCK_START
             if (ret < 0) {
                 __log_dbg("failed to remove cq fd=%d from epfd=%d (errno=%d %m)",
@@ -403,7 +395,7 @@ int epfd_info::del_fd(int fd, bool passthrough)
     __log_funcall("fd=%d", fd);
 
     epoll_fd_rec *fi;
-    socket_fd_api *temp_sock_fd_api = fd_collection_get_sockfd(fd);
+    sockinfo *temp_sock_fd_api = fd_collection_get_sockfd(fd);
     if (temp_sock_fd_api && temp_sock_fd_api->skip_os_select()) {
         __log_dbg("fd=%d must be skipped from os epoll()", fd);
     } else if (!passthrough) {
@@ -448,7 +440,7 @@ int epfd_info::del_fd(int fd, bool passthrough)
             // remove fd and replace by last fd
             m_p_offloaded_fds[fi->offloaded_index - 1] = m_p_offloaded_fds[m_n_offloaded_fds - 1];
 
-            socket_fd_api *last_socket =
+            sockinfo *last_socket =
                 fd_collection_get_sockfd(m_p_offloaded_fds[m_n_offloaded_fds - 1]);
             if (last_socket && last_socket->get_epoll_context_fd() == m_epfd) {
                 last_socket->m_fd_rec.offloaded_index = fi->offloaded_index;
@@ -485,7 +477,7 @@ int epfd_info::mod_fd(int fd, epoll_event *event)
         return -1;
     }
 
-    socket_fd_api *temp_sock_fd_api = fd_collection_get_sockfd(fd);
+    sockinfo *temp_sock_fd_api = fd_collection_get_sockfd(fd);
     // check if fd is offloaded that new event mask is OK
     if (temp_sock_fd_api && temp_sock_fd_api->m_fd_rec.offloaded_index > 0) {
         if (m_log_invalid_events && (event->events & ~SUPPORTED_EPOLL_EVENTS)) {
@@ -503,7 +495,7 @@ int epfd_info::mod_fd(int fd, epoll_event *event)
         evt.events = event->events;
         evt.data.u64 = 0; // zero all data
         evt.data.fd = fd;
-        ret = orig_os_api.epoll_ctl(m_epfd, EPOLL_CTL_MOD, fd, &evt);
+        ret = SYSCALL(epoll_ctl, m_epfd, EPOLL_CTL_MOD, fd, &evt);
         BULLSEYE_EXCLUDE_BLOCK_START
         if (ret < 0) {
             __log_err("failed to modify fd=%d in epoll epfd=%d (errno=%d %m)", fd, m_epfd, errno);
@@ -521,7 +513,7 @@ int epfd_info::mod_fd(int fd, epoll_event *event)
     uint32_t events = 0;
     if (is_offloaded) {
         // if the socket is ready, add it to ready events
-        if ((event->events & EPOLLIN) && temp_sock_fd_api->is_readable(NULL, NULL)) {
+        if ((event->events & EPOLLIN) && temp_sock_fd_api->is_readable(nullptr, nullptr)) {
             events |= EPOLLIN;
         }
         if ((event->events & EPOLLOUT) && temp_sock_fd_api->is_writeable()) {
@@ -549,8 +541,8 @@ int epfd_info::mod_fd(int fd, epoll_event *event)
 
 epoll_fd_rec *epfd_info::get_fd_rec(int fd)
 {
-    epoll_fd_rec *fd_rec = NULL;
-    socket_fd_api *temp_sock_fd_api = fd_collection_get_sockfd(fd);
+    epoll_fd_rec *fd_rec = nullptr;
+    sockinfo *temp_sock_fd_api = fd_collection_get_sockfd(fd);
     lock();
 
     if (temp_sock_fd_api && temp_sock_fd_api->get_epoll_context_fd() == m_epfd) {
@@ -575,7 +567,7 @@ void epfd_info::fd_closed(int fd, bool passthrough)
     unlock();
 }
 
-void epfd_info::insert_epoll_event_cb(socket_fd_api *sock_fd, uint32_t event_flags)
+void epfd_info::insert_epoll_event_cb(sockinfo *sock_fd, uint32_t event_flags)
 {
     lock();
     // EPOLLHUP | EPOLLERR are reported without user request
@@ -585,7 +577,7 @@ void epfd_info::insert_epoll_event_cb(socket_fd_api *sock_fd, uint32_t event_fla
     unlock();
 }
 
-void epfd_info::insert_epoll_event(socket_fd_api *sock_fd, uint32_t event_flags)
+void epfd_info::insert_epoll_event(sockinfo *sock_fd, uint32_t event_flags)
 {
     // assumed lock
     if (sock_fd->ep_ready_fd_node.is_list_member()) {
@@ -598,7 +590,7 @@ void epfd_info::insert_epoll_event(socket_fd_api *sock_fd, uint32_t event_flags)
     do_wakeup();
 }
 
-void epfd_info::remove_epoll_event(socket_fd_api *sock_fd, uint32_t event_flags)
+void epfd_info::remove_epoll_event(sockinfo *sock_fd, uint32_t event_flags)
 {
     sock_fd->m_epoll_event_flags &= ~event_flags;
     if (sock_fd->m_epoll_event_flags == 0) {
@@ -611,65 +603,29 @@ epoll_stats_t *epfd_info::stats()
     return m_stats;
 }
 
-int epfd_info::ring_poll_and_process_element(uint64_t *p_poll_sn,
-                                             void *pv_fd_ready_array /* = NULL*/)
+bool epfd_info::ring_poll_and_process_element(uint64_t *p_poll_sn_rx, uint64_t *p_poll_sn_tx,
+                                              void *pv_fd_ready_array /* = NULL*/)
 {
     __log_func("");
 
-    int ret_total = 0;
-
     if (m_ring_map.empty()) {
-        return ret_total;
+        return true;
     }
 
     m_ring_map_lock.lock();
 
+    bool all_drained = true;
     for (ring_map_t::iterator iter = m_ring_map.begin(); iter != m_ring_map.end(); iter++) {
-        int ret = iter->first->poll_and_process_element_rx(p_poll_sn, pv_fd_ready_array);
-        BULLSEYE_EXCLUDE_BLOCK_START
-        if (ret < 0 && errno != EAGAIN) {
-            __log_err("Error in RX ring->poll_and_process_element() of %p (errno=%d %m)",
-                      iter->first, errno);
-            m_ring_map_lock.unlock();
-            return ret;
-        }
-        BULLSEYE_EXCLUDE_BLOCK_END
-        if (ret > 0) {
-            __log_func("ring[%p] RX Returned with: %d (sn=%d)", iter->first, ret, *p_poll_sn);
-            ret_total += ret;
-        }
-#if defined(DEFINED_FORCE_TX_POLLING)
-        ret = iter->first->poll_and_process_element_tx(p_poll_sn);
-        BULLSEYE_EXCLUDE_BLOCK_START
-        if (ret < 0 && errno != EAGAIN) {
-            __log_err("Error in TX ring->poll_and_process_element() of %p (errno=%d %m)",
-                      iter->first, errno);
-            m_ring_map_lock.unlock();
-            return ret;
-        }
-        BULLSEYE_EXCLUDE_BLOCK_END
-        if (ret > 0) {
-            __log_func("ring[%p] TX Returned with: %d (sn=%d)", iter->first, ret, *p_poll_sn);
-            ret_total += ret;
-        }
-#endif /* DEFINED_FORCE_TX_POLLING */
+        all_drained &= iter->first->poll_and_process_element_rx(p_poll_sn_rx, pv_fd_ready_array);
+        all_drained &= (iter->first->poll_and_process_element_tx(p_poll_sn_tx) >= 0);
     }
 
     m_ring_map_lock.unlock();
 
-    if (m_sysvar_thread_mode == THREAD_MODE_PLENTY && ret_total == 0 && errno == EAGAIN) {
-        sched_yield();
-    }
-
-    if (ret_total) {
-        __log_func("ret_total=%d", ret_total);
-    } else {
-        __log_funcall("ret_total=%d", ret_total);
-    }
-    return ret_total;
+    return all_drained;
 }
 
-int epfd_info::ring_request_notification(uint64_t poll_sn)
+int epfd_info::ring_request_notification(uint64_t poll_sn_rx, uint64_t poll_sn_tx)
 {
     __log_func("");
     int ret_total = 0;
@@ -681,7 +637,7 @@ int epfd_info::ring_request_notification(uint64_t poll_sn)
     m_ring_map_lock.lock();
 
     for (ring_map_t::iterator iter = m_ring_map.begin(); iter != m_ring_map.end(); iter++) {
-        int ret = iter->first->request_notification(CQT_RX, poll_sn);
+        int ret = iter->first->request_notification(CQT_RX, poll_sn_rx);
         BULLSEYE_EXCLUDE_BLOCK_START
         if (ret < 0) {
             __log_err("Error RX ring[%p]->request_notification() (errno=%d %m)", iter->first,
@@ -690,10 +646,10 @@ int epfd_info::ring_request_notification(uint64_t poll_sn)
             return ret;
         }
         BULLSEYE_EXCLUDE_BLOCK_END
-        __log_func("ring[%p] RX Returned with: %d (sn=%d)", iter->first, ret, poll_sn);
+        __log_func("ring[%p] RX Returned with: %d (sn=%d)", iter->first, ret, poll_sn_rx);
         ret_total += ret;
-#if defined(DEFINED_FORCE_TX_POLLING)
-        ret = iter->first->request_notification(CQT_TX, poll_sn);
+
+        ret = iter->first->request_notification(CQT_TX, poll_sn_tx);
         BULLSEYE_EXCLUDE_BLOCK_START
         if (ret < 0) {
             __log_err("Error TX ring[%p]->request_notification() (errno=%d %m)", iter->first,
@@ -702,9 +658,8 @@ int epfd_info::ring_request_notification(uint64_t poll_sn)
             return ret;
         }
         BULLSEYE_EXCLUDE_BLOCK_END
-        __log_func("ring[%p] TX Returned with: %d (sn=%d)", iter->first, ret, poll_sn);
+        __log_func("ring[%p] TX Returned with: %d (sn=%d)", iter->first, ret, poll_sn_tx);
         ret_total += ret;
-#endif /* DEFINED_FORCE_TX_POLLING */
     }
 
     m_ring_map_lock.unlock();
@@ -712,12 +667,10 @@ int epfd_info::ring_request_notification(uint64_t poll_sn)
     return ret_total;
 }
 
-int epfd_info::ring_wait_for_notification_and_process_element(uint64_t *p_poll_sn,
-                                                              void *pv_fd_ready_array /* = NULL*/)
+void epfd_info::ring_wait_for_notification_and_process_element(uint64_t *p_poll_sn,
+                                                               void *pv_fd_ready_array /* = NULL*/)
 {
     __log_func("");
-    int ret_total = 0;
-
     while (!m_ready_cq_fd_q.empty()) {
 
         lock();
@@ -733,28 +686,11 @@ int epfd_info::ring_wait_for_notification_and_process_element(uint64_t *p_poll_s
         if (p_cq_ch_info) {
             ring *p_ready_ring = p_cq_ch_info->get_ring();
             // Handle the CQ notification channel
-            int ret = p_ready_ring->wait_for_notification_and_process_element(fd, p_poll_sn,
-                                                                              pv_fd_ready_array);
-            if (ret < 0) {
-                if (errno == EAGAIN) {
-                    __log_dbg("Error in ring->wait_for_notification_and_process_element() of %p "
-                              "(errno=%d %m)",
-                              p_ready_ring, errno);
-                } else {
-                    __log_err("Error in ring->wait_for_notification_and_process_element() of %p "
-                              "(errno=%d %m)",
-                              p_ready_ring, errno);
-                }
-                continue;
-            }
-            if (ret > 0) {
-                __log_func("ring[%p] Returned with: %d (sn=%d)", p_ready_ring, ret, *p_poll_sn);
-            }
-            ret_total += ret;
+            p_ready_ring->wait_for_notification_and_process_element(p_poll_sn, pv_fd_ready_array);
         } else {
             __log_dbg("failed to find channel fd. removing cq fd=%d from epfd=%d", fd, m_epfd);
             BULLSEYE_EXCLUDE_BLOCK_START
-            if ((orig_os_api.epoll_ctl(m_epfd, EPOLL_CTL_DEL, fd, NULL)) &&
+            if ((SYSCALL(epoll_ctl, m_epfd, EPOLL_CTL_DEL, fd, nullptr)) &&
                 (!(errno == ENOENT || errno == EBADF))) {
                 __log_err("failed to del cq channel fd=%d from os epfd=%d (errno=%d %m)", fd,
                           m_epfd, errno);
@@ -762,13 +698,6 @@ int epfd_info::ring_wait_for_notification_and_process_element(uint64_t *p_poll_s
             BULLSEYE_EXCLUDE_BLOCK_END
         }
     }
-
-    if (ret_total) {
-        __log_func("ret_total=%d", ret_total);
-    } else {
-        __log_funcall("ret_total=%d", ret_total);
-    }
-    return ret_total;
 }
 
 void epfd_info::clean_obj()
@@ -852,27 +781,3 @@ void epfd_info::statistics_print(vlog_levels_t log_level /* = VLOG_DEBUG */)
         }
     }
 }
-
-void epfd_info::set_os_data_available()
-{
-    std::lock_guard<decltype(m_lock_poll_os)> locker(m_lock_poll_os);
-    m_b_os_data_available = true;
-}
-
-void epfd_info::register_to_internal_thread()
-{
-    std::lock_guard<decltype(m_lock_poll_os)> locker(m_lock_poll_os);
-    m_b_os_data_available = false;
-
-    // Reassign EPOLLIN event
-    g_p_event_handler_manager->update_epfd(m_epfd, EPOLL_CTL_MOD,
-                                           EPOLLIN | EPOLLPRI | EPOLLONESHOT);
-}
-
-bool epfd_info::get_and_unset_os_data_available()
-{
-    std::lock_guard<decltype(m_lock_poll_os)> locker(m_lock_poll_os);
-    bool ret = m_b_os_data_available;
-    m_b_os_data_available = false;
-    return ret;
-}
