@@ -1,5 +1,7 @@
 /*
- * Copyright (c) 2001-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
+ * Copyright (c) 2001-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: GPL-2.0-only or BSD-3-Clause
  *
  * This software is available to you under a choice of one of two
  * licenses.  You may choose to be licensed under the terms of the GNU
@@ -52,6 +54,7 @@
 #include "util/xlio_stats.h"
 #include "util/utils.h"
 #include "event/event_handler_manager.h"
+#include "event/poll_group.h"
 #include "event/vlogger_timer_handler.h"
 #include "dev/buffer_pool.h"
 #include "dev/ib_ctx_handler_collection.h"
@@ -65,18 +68,18 @@
 #include "proto/neighbour_table_mgr.h"
 #include "netlink/netlink_wrapper.h"
 #include "event/command.h"
-
+#include "sock/sock_stats.h"
 #include "sock/sock-redirect.h"
 #include "sock/sock-app.h"
 #include "sock/fd_collection.h"
 #include "sock/sockinfo_tcp.h"
 #include "sock/sockinfo_udp.h"
-#include "sock/tcp_seg_pool.h"
 #include "sock/bind_no_port.h"
 #include "iomux/io_mux_call.h"
 
 #include "util/instrumentation.h"
 #include "util/agent.h"
+#include "xlio.h"
 
 void check_netperf_flags();
 
@@ -105,10 +108,12 @@ bool g_b_exit = false;
 bool g_init_ibv_fork_done = false;
 bool g_is_forked_child = false;
 bool g_init_global_ctors_done = true;
-static command_netlink *s_cmd_nl = NULL;
+static command_netlink *s_cmd_nl = nullptr;
 #define MAX_VERSION_STR_LEN 128
 
 global_stats_t g_global_stat_static;
+static uint32_t g_ec_pool_size = 0U;
+static uint32_t g_ec_pool_no_objs = 0U;
 
 static int free_libxlio_resources()
 {
@@ -116,6 +121,9 @@ static int free_libxlio_resources()
 
     g_b_exit = true;
 
+    // Destroy polling groups before fd_collection to clear XLIO sockets from the fd_collection
+    poll_group::destroy_all_groups();
+
     // Triggers connection close, relevant for TCP which may need some time to terminate the
     // connection. and for any socket that may wait from another thread
     if (g_p_fd_collection) {
@@ -134,11 +142,11 @@ static int free_libxlio_resources()
     if (g_tcp_timers_collection) {
         g_tcp_timers_collection->clean_obj();
     }
-    g_tcp_timers_collection = NULL;
+    g_tcp_timers_collection = nullptr;
 
     // Block all sock-redicrt API calls into our offloading core
     fd_collection *g_p_fd_collection_temp = g_p_fd_collection;
-    g_p_fd_collection = NULL;
+    g_p_fd_collection = nullptr;
     if (g_p_fd_collection_temp) {
         delete g_p_fd_collection_temp;
     }
@@ -146,30 +154,30 @@ static int free_libxlio_resources()
     if (g_p_lwip) {
         delete g_p_lwip;
     }
-    g_p_lwip = NULL;
+    g_p_lwip = nullptr;
 
     if (g_p_route_table_mgr) {
         delete g_p_route_table_mgr;
     }
-    g_p_route_table_mgr = NULL;
+    g_p_route_table_mgr = nullptr;
 
     if (g_bind_no_port) {
         delete g_bind_no_port;
     }
-    g_bind_no_port = NULL;
+    g_bind_no_port = nullptr;
 
     if (g_p_rule_table_mgr) {
         delete g_p_rule_table_mgr;
     }
-    g_p_rule_table_mgr = NULL;
+    g_p_rule_table_mgr = nullptr;
 
     if (g_p_net_device_table_mgr) {
         delete g_p_net_device_table_mgr;
     }
-    g_p_net_device_table_mgr = NULL;
+    g_p_net_device_table_mgr = nullptr;
 
     ip_frag_manager *g_p_ip_frag_manager_temp = g_p_ip_frag_manager;
-    g_p_ip_frag_manager = NULL;
+    g_p_ip_frag_manager = nullptr;
     if (g_p_ip_frag_manager_temp) {
         delete g_p_ip_frag_manager_temp;
     }
@@ -177,81 +185,88 @@ static int free_libxlio_resources()
     if (g_p_neigh_table_mgr) {
         delete g_p_neigh_table_mgr;
     }
-    g_p_neigh_table_mgr = NULL;
+    g_p_neigh_table_mgr = nullptr;
 
     if (g_tcp_seg_pool) {
         delete g_tcp_seg_pool;
     }
-    g_tcp_seg_pool = NULL;
+    g_tcp_seg_pool = nullptr;
+
+    if (g_socketxtreme_ec_pool) {
+        delete g_socketxtreme_ec_pool;
+    }
+    g_socketxtreme_ec_pool = NULL;
 
     if (safe_mce_sys().print_report) {
-        buffer_pool::print_report_on_errors(VLOG_INFO);
+        buffer_pool::print_full_report(VLOG_INFO);
     }
 
     if (g_buffer_pool_zc) {
         delete g_buffer_pool_zc;
     }
-    g_buffer_pool_zc = NULL;
+    g_buffer_pool_zc = nullptr;
 
     if (g_buffer_pool_tx) {
         delete g_buffer_pool_tx;
     }
-    g_buffer_pool_tx = NULL;
+    g_buffer_pool_tx = nullptr;
 
     if (g_buffer_pool_rx_stride) {
         delete g_buffer_pool_rx_stride;
     }
-    g_buffer_pool_rx_stride = NULL;
+    g_buffer_pool_rx_stride = nullptr;
 
     if (g_buffer_pool_rx_rwqe) {
         delete g_buffer_pool_rx_rwqe;
     }
-    g_buffer_pool_rx_rwqe = NULL;
+    g_buffer_pool_rx_rwqe = nullptr;
 
     if (g_zc_cache) {
         delete g_zc_cache;
     }
-    g_zc_cache = NULL;
+    g_zc_cache = nullptr;
 
     xlio_heap::finalize();
 
     if (s_cmd_nl) {
         delete s_cmd_nl;
     }
-    s_cmd_nl = NULL;
+    s_cmd_nl = nullptr;
 
     if (g_p_netlink_handler) {
         delete g_p_netlink_handler;
     }
-    g_p_netlink_handler = NULL;
+    g_p_netlink_handler = nullptr;
 
     if (g_p_ib_ctx_handler_collection) {
         delete g_p_ib_ctx_handler_collection;
     }
-    g_p_ib_ctx_handler_collection = NULL;
+    g_p_ib_ctx_handler_collection = nullptr;
 
     if (g_p_vlogger_timer_handler) {
         delete g_p_vlogger_timer_handler;
     }
-    g_p_vlogger_timer_handler = NULL;
+    g_p_vlogger_timer_handler = nullptr;
 
     if (g_p_event_handler_manager) {
         delete g_p_event_handler_manager;
     }
-    g_p_event_handler_manager = NULL;
+    g_p_event_handler_manager = nullptr;
 
     if (g_p_agent) {
         delete g_p_agent;
     }
-    g_p_agent = NULL;
+    g_p_agent = nullptr;
 
     if (safe_mce_sys().app_name) {
         free(safe_mce_sys().app_name);
     }
-    safe_mce_sys().app_name = NULL;
+    safe_mce_sys().app_name = nullptr;
 
     vlog_printf(VLOG_DEBUG, "Stopping logger module\n");
 
+    sock_stats::destroy_instance();
+
     sock_redirect_exit();
 
     vlog_stop();
@@ -260,14 +275,14 @@ static int free_libxlio_resources()
         // cosmetics - remove when adding iomux block
         fprintf(g_stats_file, "======================================================\n");
         fclose(g_stats_file);
-        g_stats_file = NULL;
+        g_stats_file = nullptr;
     }
 
 #if defined(DEFINED_NGINX) || defined(DEFINED_ENVOY)
     if (g_p_app) {
         delete g_p_app;
     }
-    g_p_app = NULL;
+    g_p_app = nullptr;
 #endif
 
     return 0;
@@ -351,23 +366,6 @@ void check_locked_mem()
     }
 }
 
-const char *thread_mode_str(thread_mode_t thread_mode)
-{
-    switch (thread_mode) {
-    case THREAD_MODE_SINGLE:
-        return "Single";
-    case THREAD_MODE_MULTI:
-        return "Multi spin lock";
-    case THREAD_MODE_MUTEX:
-        return "Multi mutex lock";
-    case THREAD_MODE_PLENTY:
-        return "Multi more threads than cores";
-    default:
-        break;
-    }
-    return "";
-}
-
 const char *buffer_batching_mode_str(buffer_batching_mode_t buffer_batching_mode)
 {
     switch (buffer_batching_mode) {
@@ -430,7 +428,7 @@ int get_ofed_version_info(char *ofed_version_str, int len)
 void print_xlio_global_settings()
 {
     struct utsname sys_info;
-    time_t clock = time(NULL);
+    time_t clock = time(nullptr);
     char ofed_version_info[MAX_VERSION_STR_LEN];
 
     vlog_printf(VLOG_INFO,
@@ -553,12 +551,8 @@ void print_xlio_global_settings()
     VLOG_PARAM_STRING("Zerocopy Cache Threshold", safe_mce_sys().zc_cache_threshold,
                       MCE_DEFAULT_ZC_CACHE_THRESHOLD, SYS_VAR_ZC_CACHE_THRESHOLD,
                       option_size::to_str(safe_mce_sys().zc_cache_threshold));
-    VLOG_PARAM_NUMBER("Tx Mem Bufs", safe_mce_sys().tx_num_bufs, MCE_DEFAULT_TX_NUM_BUFS,
-                      SYS_VAR_TX_NUM_BUFS);
     VLOG_PARAM_STRING("Tx Mem Buf size", safe_mce_sys().tx_buf_size, MCE_DEFAULT_TX_BUF_SIZE,
                       SYS_VAR_TX_BUF_SIZE, option_size::to_str(safe_mce_sys().tx_buf_size));
-    VLOG_PARAM_STRING("ZC TX size", safe_mce_sys().zc_tx_size, MCE_DEFAULT_ZC_TX_SIZE,
-                      SYS_VAR_ZC_TX_SIZE, option_size::to_str(safe_mce_sys().zc_tx_size));
     VLOG_PARAM_NUMBER("Tx QP WRE", safe_mce_sys().tx_num_wr, MCE_DEFAULT_TX_NUM_WRE,
                       SYS_VAR_TX_NUM_WRE);
     VLOG_PARAM_NUMBER("Tx QP WRE Batching", safe_mce_sys().tx_num_wr_to_signal,
@@ -582,10 +576,6 @@ void print_xlio_global_settings()
     VLOG_PARAM_STRING("TCP Send Buffer size", safe_mce_sys().tcp_send_buffer_size,
                       MCE_DEFAULT_TCP_SEND_BUFFER_SIZE, SYS_VAR_TCP_SEND_BUFFER_SIZE,
                       option_size::to_str(safe_mce_sys().tcp_send_buffer_size));
-    VLOG_PARAM_NUMBER(
-        "Rx Mem Bufs", safe_mce_sys().rx_num_bufs,
-        (safe_mce_sys().enable_striding_rq ? MCE_DEFAULT_STRQ_NUM_BUFS : MCE_DEFAULT_RX_NUM_BUFS),
-        SYS_VAR_RX_NUM_BUFS);
     VLOG_PARAM_NUMBER(
         "Rx QP WRE", safe_mce_sys().rx_num_wr,
         (safe_mce_sys().enable_striding_rq ? MCE_DEFAULT_STRQ_NUM_WRE : MCE_DEFAULT_RX_NUM_WRE),
@@ -639,6 +629,8 @@ void print_xlio_global_settings()
     VLOG_PARAM_NUMBER("Disable flow tag", safe_mce_sys().disable_flow_tag,
                       MCE_DEFAULT_DISABLE_FLOW_TAG, SYS_VAR_DISABLE_FLOW_TAG);
 
+    VLOG_PARAM_STRING("TCP 2T rules", safe_mce_sys().tcp_2t_rules, MCE_DEFAULT_TCP_2T_RULES,
+                      SYS_VAR_TCP_2T_RULES, safe_mce_sys().tcp_2t_rules ? "Enabled " : "Disabled");
     VLOG_PARAM_STRING("TCP 3T rules", safe_mce_sys().tcp_3t_rules, MCE_DEFAULT_TCP_3T_RULES,
                       SYS_VAR_TCP_3T_RULES, safe_mce_sys().tcp_3t_rules ? "Enabled " : "Disabled");
     VLOG_PARAM_STRING("UDP 3T rules", safe_mce_sys().udp_3t_rules, MCE_DEFAULT_UDP_3T_RULES,
@@ -649,15 +641,13 @@ void print_xlio_global_settings()
     VLOG_PARAM_STRING("Force Flowtag for MC", safe_mce_sys().mc_force_flowtag,
                       MCE_DEFAULT_MC_FORCE_FLOWTAG, SYS_VAR_MC_FORCE_FLOWTAG,
                       safe_mce_sys().mc_force_flowtag ? "Enabled " : "Disabled");
-    VLOG_STR_PARAM_STRING("Striding RQ", option_strq::to_str(safe_mce_sys().enable_strq_env),
-                          option_strq::to_str(MCE_DEFAULT_STRQ), SYS_VAR_STRQ,
-                          option_strq::to_str(safe_mce_sys().enable_strq_env));
+    VLOG_STR_PARAM_STRING("Striding RQ", option_3::to_str(safe_mce_sys().enable_strq_env),
+                          option_3::to_str(MCE_DEFAULT_STRQ), SYS_VAR_STRQ,
+                          option_3::to_str(safe_mce_sys().enable_strq_env));
     VLOG_PARAM_NUMBER("STRQ Strides per RWQE", safe_mce_sys().strq_stride_num_per_rwqe,
                       MCE_DEFAULT_STRQ_NUM_STRIDES, SYS_VAR_STRQ_NUM_STRIDES);
     VLOG_PARAM_NUMBER("STRQ Stride Size (Bytes)", safe_mce_sys().strq_stride_size_bytes,
                       MCE_DEFAULT_STRQ_STRIDE_SIZE_BYTES, SYS_VAR_STRQ_STRIDE_SIZE_BYTES);
-    VLOG_PARAM_NUMBER("STRQ Initial Strides Per Ring", safe_mce_sys().strq_strides_num_bufs,
-                      MCE_DEFAULT_STRQ_STRIDES_NUM_BUFS, SYS_VAR_STRQ_STRIDES_NUM_BUFS);
     VLOG_PARAM_NUMBER(
         "STRQ Strides Compensation Level", safe_mce_sys().strq_strides_compensation_level,
         MCE_DEFAULT_STRQ_STRIDES_COMPENSATION_LEVEL, SYS_VAR_STRQ_STRIDES_COMPENSATION_LEVEL);
@@ -724,9 +714,7 @@ void print_xlio_global_settings()
                       MCE_DEFAULT_CQ_KEEP_QP_FULL, SYS_VAR_CQ_KEEP_QP_FULL,
                       safe_mce_sys().cq_keep_qp_full ? "Enabled" : "Disabled");
     VLOG_PARAM_NUMBER("QP Compensation Level", safe_mce_sys().qp_compensation_level,
-                      (safe_mce_sys().enable_striding_rq ? MCE_DEFAULT_STRQ_COMPENSATION_LEVEL
-                                                         : MCE_DEFAULT_QP_COMPENSATION_LEVEL),
-                      SYS_VAR_QP_COMPENSATION_LEVEL);
+                      safe_mce_sys().rx_num_wr / 2U, SYS_VAR_QP_COMPENSATION_LEVEL);
     VLOG_PARAM_STRING("Offloaded Sockets", safe_mce_sys().offloaded_sockets,
                       MCE_DEFAULT_OFFLOADED_SOCKETS, SYS_VAR_OFFLOADED_SOCKETS,
                       safe_mce_sys().offloaded_sockets ? "Enabled" : "Disabled");
@@ -767,8 +755,6 @@ void print_xlio_global_settings()
     VLOG_PARAM_STRING("Internal Thread Arm CQ", safe_mce_sys().internal_thread_arm_cq_enabled,
                       MCE_DEFAULT_INTERNAL_THREAD_ARM_CQ_ENABLED, SYS_VAR_INTERNAL_THREAD_ARM_CQ,
                       safe_mce_sys().internal_thread_arm_cq_enabled ? "Enabled " : "Disabled");
-    VLOG_PARAM_STRING("Thread mode", safe_mce_sys().thread_mode, MCE_DEFAULT_THREAD_MODE,
-                      SYS_VAR_THREAD_MODE, thread_mode_str(safe_mce_sys().thread_mode));
     VLOG_PARAM_NUMSTR("Buffer batching mode", safe_mce_sys().buffer_batching_mode,
                       MCE_DEFAULT_BUFFER_BATCHING_MODE, SYS_VAR_BUFFER_BATCHING_MODE,
                       buffer_batching_mode_str(safe_mce_sys().buffer_batching_mode));
@@ -781,8 +767,8 @@ void print_xlio_global_settings()
     VLOG_PARAM_STRING("Memory limit (user allocator)", safe_mce_sys().memory_limit_user,
                       MCE_DEFAULT_MEMORY_LIMIT_USER, SYS_VAR_MEMORY_LIMIT_USER,
                       option_size::to_str(safe_mce_sys().memory_limit_user));
-    VLOG_PARAM_NUMBER("Hugepage log2", safe_mce_sys().hugepage_log2, MCE_DEFAULT_HUGEPAGE_LOG2,
-                      SYS_VAR_HUGEPAGE_LOG2);
+    VLOG_PARAM_STRING("Hugepage size", safe_mce_sys().hugepage_size, MCE_DEFAULT_HUGEPAGE_SIZE,
+                      SYS_VAR_HUGEPAGE_SIZE, option_size::to_str(safe_mce_sys().hugepage_size));
 
     VLOG_PARAM_NUMBER("Num of UC ARPs", safe_mce_sys().neigh_uc_arp_quata,
                       MCE_DEFAULT_NEIGH_UC_ARP_QUATA, SYS_VAR_NEIGH_UC_ARP_QUATA);
@@ -797,22 +783,24 @@ void print_xlio_global_settings()
     VLOG_STR_PARAM_STRING("TSO support", option_3::to_str(safe_mce_sys().enable_tso),
                           option_3::to_str(MCE_DEFAULT_TSO), SYS_VAR_TSO,
                           option_3::to_str(safe_mce_sys().enable_tso));
+    VLOG_PARAM_STRING("TSO max size", safe_mce_sys().max_tso_sz, MCE_DEFAULT_MAX_TSO_SIZE,
+                      SYS_VAR_MAX_TSO_SIZE, option_size::to_str(safe_mce_sys().max_tso_sz));
     VLOG_STR_PARAM_STRING("LRO support", option_3::to_str(safe_mce_sys().enable_lro),
                           option_3::to_str(MCE_DEFAULT_LRO), SYS_VAR_LRO,
                           option_3::to_str(safe_mce_sys().enable_lro));
-    VLOG_PARAM_STRING("BF (Blue Flame)", safe_mce_sys().handle_bf, MCE_DEFAULT_BF_FLAG, SYS_VAR_BF,
-                      safe_mce_sys().handle_bf ? "Enabled " : "Disabled");
 #ifdef DEFINED_UTLS
     VLOG_PARAM_STRING("UTLS RX support", safe_mce_sys().enable_utls_rx, MCE_DEFAULT_UTLS_RX,
                       SYS_VAR_UTLS_RX, safe_mce_sys().enable_utls_rx ? "Enabled " : "Disabled");
     VLOG_PARAM_STRING("UTLS TX support", safe_mce_sys().enable_utls_tx, MCE_DEFAULT_UTLS_TX,
                       SYS_VAR_UTLS_TX, safe_mce_sys().enable_utls_tx ? "Enabled " : "Disabled");
-    VLOG_PARAM_NUMBER(
-        "UTLS high watermark DEK cache size", safe_mce_sys().utls_high_wmark_dek_cache_size,
-        MCE_DEFAULT_UTLS_HIGH_WMARK_DEK_CACHE_SIZE, SYS_VAR_UTLS_HIGH_WMARK_DEK_CACHE_SIZE);
-    VLOG_PARAM_NUMBER(
-        "UTLS low watermark DEK cache size", safe_mce_sys().utls_low_wmark_dek_cache_size,
-        MCE_DEFAULT_UTLS_LOW_WMARK_DEK_CACHE_SIZE, SYS_VAR_UTLS_LOW_WMARK_DEK_CACHE_SIZE);
+    VLOG_PARAM_NUMBER("UTLS high watermark DEK cache size",
+                      static_cast<uint32_t>(safe_mce_sys().utls_high_wmark_dek_cache_size),
+                      MCE_DEFAULT_UTLS_HIGH_WMARK_DEK_CACHE_SIZE,
+                      SYS_VAR_UTLS_HIGH_WMARK_DEK_CACHE_SIZE);
+    VLOG_PARAM_NUMBER("UTLS low watermark DEK cache size",
+                      static_cast<uint32_t>(safe_mce_sys().utls_low_wmark_dek_cache_size),
+                      MCE_DEFAULT_UTLS_LOW_WMARK_DEK_CACHE_SIZE,
+                      SYS_VAR_UTLS_LOW_WMARK_DEK_CACHE_SIZE);
 #endif /* DEFINED_UTLS */
 #if defined(DEFINED_NGINX)
     VLOG_PARAM_NUMBER("Number of Nginx workers",
@@ -887,23 +875,8 @@ void print_xlio_global_settings()
                       SYS_VAR_MULTILOCK,
                       (safe_mce_sys().multilock == MULTILOCK_SPIN ? "Spin " : "Mutex"));
 
-#ifdef XLIO_TIME_MEASURE
-    VLOG_PARAM_NUMBER("Time Measure Num Samples", safe_mce_sys().xlio_time_measure_num_samples,
-                      MCE_DEFAULT_TIME_MEASURE_NUM_SAMPLES, SYS_VAR_TIME_MEASURE_NUM_SAMPLES);
-    VLOG_STR_PARAM_STRING("Time Measure Dump File", safe_mce_sys().xlio_time_measure_filename,
-                          MCE_DEFAULT_TIME_MEASURE_DUMP_FILE, SYS_VAR_TIME_MEASURE_DUMP_FILE,
-                          safe_mce_sys().xlio_time_measure_filename);
-#endif
-
     vlog_printf(VLOG_INFO,
                 "---------------------------------------------------------------------------\n");
-
-#if !defined(DEFINED_DPCP)
-    if (safe_mce_sys().mce_spec == MCE_SPEC_NVME_BF2) {
-        vlog_printf(VLOG_INFO, "XLIO '%s' spec is used without enabled DPCP!\n",
-                    xlio_spec::to_str(MCE_SPEC_NVME_BF2));
-    }
-#endif
 }
 
 void prepare_fork()
@@ -941,7 +914,7 @@ void register_handler_segv()
     act.sa_handler = handle_segfault;
     act.sa_flags = 0;
     sigemptyset(&act.sa_mask);
-    sigaction(SIGSEGV, &act, NULL);
+    sigaction(SIGSEGV, &act, nullptr);
     vlog_printf(VLOG_INFO, "Registered a SIGSEGV handler\n");
 }
 
@@ -957,24 +930,10 @@ extern "C" void sock_redirect_main(void)
     if (safe_mce_sys().handle_segfault) {
         register_handler_segv();
     }
-
-#ifdef RDTSC_MEASURE
-    init_rdtsc();
-#endif
-
-#ifdef XLIO_TIME_MEASURE
-    init_instrumentation();
-#endif
 }
 
 extern "C" void sock_redirect_exit(void)
 {
-#ifdef RDTSC_MEASURE
-    print_rdtsc_summary();
-#endif
-#ifdef XLIO_TIME_MEASURE
-    finit_instrumentation(safe_mce_sys().xlio_time_measure_filename);
-#endif
     vlog_printf(VLOG_DEBUG, "%s()\n", __FUNCTION__);
 
     if (g_init_global_ctors_done) {
@@ -1064,6 +1023,8 @@ static void do_global_ctors_helper()
     *g_p_vlogger_level = g_vlogger_level;
     *g_p_vlogger_details = g_vlogger_details;
 
+    sock_stats::init_instance(safe_mce_sys().stats_fd_num_max);
+
     g_global_stat_static.init();
     xlio_stats_instance_create_global_block(&g_global_stat_static);
 
@@ -1090,19 +1051,17 @@ static void do_global_ctors_helper()
         safe_mce_sys().rx_buf_size = 0;
     }
 
-    NEW_CTOR(
-        g_buffer_pool_rx_rwqe,
-        buffer_pool(BUFFER_POOL_RX, calc_rx_wqe_buff_size(), buffer_pool::free_rx_lwip_pbuf_custom,
-                    (safe_mce_sys().m_ioctl.user_alloc.flags & IOCTL_USER_ALLOC_RX
-                         ? safe_mce_sys().m_ioctl.user_alloc.memalloc
-                         : nullptr),
-                    (safe_mce_sys().m_ioctl.user_alloc.flags & IOCTL_USER_ALLOC_RX
-                         ? safe_mce_sys().m_ioctl.user_alloc.memfree
-                         : nullptr)));
+    NEW_CTOR(g_buffer_pool_rx_rwqe,
+             buffer_pool(BUFFER_POOL_RX, calc_rx_wqe_buff_size(),
+                         (safe_mce_sys().m_ioctl.user_alloc.flags & IOCTL_USER_ALLOC_RX
+                              ? safe_mce_sys().m_ioctl.user_alloc.memalloc
+                              : nullptr),
+                         (safe_mce_sys().m_ioctl.user_alloc.flags & IOCTL_USER_ALLOC_RX
+                              ? safe_mce_sys().m_ioctl.user_alloc.memfree
+                              : nullptr)));
 
     if (safe_mce_sys().enable_striding_rq) {
-        NEW_CTOR(g_buffer_pool_rx_stride,
-                 buffer_pool(BUFFER_POOL_RX, 0, buffer_pool::free_rx_lwip_pbuf_custom));
+        NEW_CTOR(g_buffer_pool_rx_stride, buffer_pool(BUFFER_POOL_RX, 0));
         g_buffer_pool_rx_ptr = g_buffer_pool_rx_stride;
     } else {
         g_buffer_pool_rx_ptr = g_buffer_pool_rx_rwqe;
@@ -1119,24 +1078,26 @@ static void do_global_ctors_helper()
                                          ? safe_mce_sys().tx_buf_size
                                          : get_lwip_tcp_mss(g_p_net_device_table_mgr->get_max_mtu(),
                                                             safe_mce_sys().lwip_mss)),
-                         buffer_pool::free_tx_lwip_pbuf_custom,
                          (safe_mce_sys().m_ioctl.user_alloc.flags & IOCTL_USER_ALLOC_TX
                               ? safe_mce_sys().m_ioctl.user_alloc.memalloc
-                              : NULL),
+                              : nullptr),
                          (safe_mce_sys().m_ioctl.user_alloc.flags & IOCTL_USER_ALLOC_TX
                               ? safe_mce_sys().m_ioctl.user_alloc.memfree
-                              : NULL)));
+                              : nullptr)));
 
-    NEW_CTOR(g_buffer_pool_zc,
-             buffer_pool(BUFFER_POOL_TX, 0, buffer_pool::free_tx_lwip_pbuf_custom));
+    NEW_CTOR(g_buffer_pool_zc, buffer_pool(BUFFER_POOL_TX, 0));
 
-    NEW_CTOR(g_tcp_seg_pool, tcp_seg_pool());
+    NEW_CTOR(g_tcp_seg_pool,
+             tcp_seg_pool("TCP segments", safe_mce_sys().tx_segs_pool_batch_tcp,
+                          g_global_stat_static.n_tcp_seg_pool_size,
+                          g_global_stat_static.n_tcp_seg_pool_no_segs));
+
+    NEW_CTOR(g_socketxtreme_ec_pool,
+             socketxtreme_ec_pool("Socketxtreme ec", 512, g_ec_pool_size, g_ec_pool_no_objs));
 
     // For delegated TCP timers the global collection is not used.
     if (safe_mce_sys().tcp_ctl_thread != option_tcp_ctl_thread::CTL_THREAD_DELEGATE_TCP_TIMERS) {
-        NEW_CTOR(g_tcp_timers_collection,
-                 tcp_timers_collection(safe_mce_sys().tcp_timer_resolution_msec,
-                                       safe_mce_sys().timer_resolution_msec));
+        NEW_CTOR(g_tcp_timers_collection, tcp_timers_collection());
     }
 
     NEW_CTOR(g_p_vlogger_timer_handler, vlogger_timer_handler());
@@ -1175,13 +1136,13 @@ static void do_global_ctors_helper()
 
         // Register netlink fd to the event_manager
         s_cmd_nl = new command_netlink(g_p_netlink_handler);
-        if (s_cmd_nl == NULL) {
+        if (!s_cmd_nl) {
             throw_xlio_exception("Failed allocating command_netlink\n");
         }
         BULLSEYE_EXCLUDE_BLOCK_END
         g_p_event_handler_manager->register_command_event(fd, s_cmd_nl);
         g_p_event_handler_manager->register_timer_event(safe_mce_sys().timer_netlink_update_msec,
-                                                        s_cmd_nl, PERIODIC_TIMER, NULL);
+                                                        s_cmd_nl, PERIODIC_TIMER, nullptr);
     }
 
 #ifdef DEFINED_UTLS
@@ -1208,29 +1169,30 @@ int do_global_ctors()
 
 void reset_globals()
 {
-    g_p_fd_collection = NULL;
-    g_p_ip_frag_manager = NULL;
-    g_zc_cache = NULL;
-    g_buffer_pool_rx_ptr = NULL;
-    g_buffer_pool_rx_stride = NULL;
-    g_buffer_pool_rx_rwqe = NULL;
-    g_buffer_pool_tx = NULL;
-    g_buffer_pool_zc = NULL;
-    g_tcp_seg_pool = NULL;
-    g_tcp_timers_collection = NULL;
-    g_p_vlogger_timer_handler = NULL;
-    g_p_event_handler_manager = NULL;
-    g_p_agent = NULL;
-    g_p_route_table_mgr = NULL;
-    g_bind_no_port = NULL;
-    g_p_rule_table_mgr = NULL;
-    g_stats_file = NULL;
-    g_p_net_device_table_mgr = NULL;
-    g_p_neigh_table_mgr = NULL;
-    g_p_lwip = NULL;
-    g_p_netlink_handler = NULL;
-    g_p_ib_ctx_handler_collection = NULL;
-    s_cmd_nl = NULL;
+    g_p_fd_collection = nullptr;
+    g_p_ip_frag_manager = nullptr;
+    g_zc_cache = nullptr;
+    g_buffer_pool_rx_ptr = nullptr;
+    g_buffer_pool_rx_stride = nullptr;
+    g_buffer_pool_rx_rwqe = nullptr;
+    g_buffer_pool_tx = nullptr;
+    g_buffer_pool_zc = nullptr;
+    g_tcp_seg_pool = nullptr;
+    g_socketxtreme_ec_pool = NULL;
+    g_tcp_timers_collection = nullptr;
+    g_p_vlogger_timer_handler = nullptr;
+    g_p_event_handler_manager = nullptr;
+    g_p_agent = nullptr;
+    g_p_route_table_mgr = nullptr;
+    g_bind_no_port = nullptr;
+    g_p_rule_table_mgr = nullptr;
+    g_stats_file = nullptr;
+    g_p_net_device_table_mgr = nullptr;
+    g_p_neigh_table_mgr = nullptr;
+    g_p_lwip = nullptr;
+    g_p_netlink_handler = nullptr;
+    g_p_ib_ctx_handler_collection = nullptr;
+    s_cmd_nl = nullptr;
     g_cpu_manager.reset();
 }
 
@@ -1252,9 +1214,9 @@ void check_netperf_flags()
     if (strcmp(command, "netserver")) {
         return;
     }
-    pch = strtok(NULL, " ");
+    pch = strtok(nullptr, " ");
 
-    while (pch != NULL) {
+    while (pch) {
         if (*pch == '-') {
             if (strchr(pch, 'D')) {
                 b_D_flag = true;
@@ -1266,7 +1228,7 @@ void check_netperf_flags()
         if (b_f_flag && b_D_flag) {
             break;
         }
-        pch = strtok(NULL, " ");
+        pch = strtok(nullptr, " ");
     }
     if (!b_D_flag || !b_f_flag) {
         vlog_printf(VLOG_WARNING, "Running netserver without flags: -D, -f can cause failure\n");
@@ -1307,7 +1269,9 @@ extern "C" int xlio_init(void)
 {
     PROFILE_FUNC
 
+#ifndef XLIO_STATIC_BUILD
     get_orig_funcs();
+#endif /* XLIO_STATIC_BUILD */
     safe_mce_sys();
 
     g_init_global_ctors_done = false;
