@@ -1,5 +1,7 @@
 /*
- * Copyright (c) 2001-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
+ * Copyright (c) 2001-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: GPL-2.0-only or BSD-3-Clause
  *
  * This software is available to you under a choice of one of two
  * licenses.  You may choose to be licensed under the terms of the GNU
@@ -71,7 +73,8 @@ ssize_t dst_entry_tcp::fast_send(const iovec *p_iov, const ssize_t sz_iov, xlio_
     void *p_pkt;
     void *p_ip_hdr;
     void *p_tcp_hdr;
-    tcp_iovec *p_tcp_iov = NULL;
+    tcp_iovec *p_tcp_iov = nullptr;
+    xlio_ibv_send_wr *p_send_wqe;
     size_t hdr_alignment_diff = 0;
 
     bool is_zerocopy = is_set(attr.flags, XLIO_TX_PACKET_ZEROCOPY);
@@ -151,7 +154,7 @@ ssize_t dst_entry_tcp::fast_send(const iovec *p_iov, const ssize_t sz_iov, xlio_
         tcp_hdr_len = (static_cast<tcphdr *>(p_tcp_hdr))->doff * 4;
 
         if (!is_zerocopy && (total_packet_len < m_max_inline) && (1 == sz_iov)) {
-            m_p_send_wqe = &m_inline_send_wqe;
+            p_send_wqe = &m_inline_send_wqe;
             p_tcp_iov[0].iovec.iov_base = (uint8_t *)p_pkt + hdr_alignment_diff;
             p_tcp_iov[0].iovec.iov_len = total_packet_len;
         } else if (is_set(attr.flags, (xlio_wr_tx_packet_attr)(XLIO_TX_PACKET_TSO))) {
@@ -165,18 +168,18 @@ ssize_t dst_entry_tcp::fast_send(const iovec *p_iov, const ssize_t sz_iov, xlio_
                 send_wqe_h.enable_tso(send_wqe, (void *)((uint8_t *)p_pkt + hdr_alignment_diff),
                                       m_header->m_total_hdr_len + tcp_hdr_len, 0);
             }
-            m_p_send_wqe = &send_wqe;
+            p_send_wqe = &send_wqe;
             if (!is_zerocopy) {
                 p_tcp_iov[0].iovec.iov_base = (uint8_t *)p_tcp_hdr + tcp_hdr_len;
                 p_tcp_iov[0].iovec.iov_len -= tcp_hdr_len;
             }
         } else {
-            m_p_send_wqe = &m_not_inline_send_wqe;
+            p_send_wqe = &m_not_inline_send_wqe;
             p_tcp_iov[0].iovec.iov_base = (uint8_t *)p_pkt + hdr_alignment_diff;
             p_tcp_iov[0].iovec.iov_len = total_packet_len;
         }
 
-        if (unlikely(p_tcp_iov[0].p_desc->lwip_pbuf.pbuf.ref > 1)) {
+        if (unlikely(p_tcp_iov[0].p_desc->lwip_pbuf.ref > 1)) {
             /*
              * First buffer in the vector is used for reference counting.
              * The reference is released after completion depending on
@@ -194,16 +197,16 @@ ssize_t dst_entry_tcp::fast_send(const iovec *p_iov, const ssize_t sz_iov, xlio_
              *
              * We don't change data, only pointer to buffer descriptor.
              */
-            pbuf_type type = (pbuf_type)p_tcp_iov[0].p_desc->lwip_pbuf.pbuf.type;
+            pbuf_type type = (pbuf_type)p_tcp_iov[0].p_desc->lwip_pbuf.type;
             mem_buf_desc_t *p_mem_buf_desc =
-                get_buffer(type, &(p_tcp_iov[0].p_desc->lwip_pbuf.pbuf.desc),
+                get_buffer(type, &(p_tcp_iov[0].p_desc->lwip_pbuf.desc),
                            is_set(attr.flags, XLIO_TX_PACKET_BLOCK));
             if (!p_mem_buf_desc) {
                 return -1;
             }
             p_tcp_iov[0].p_desc = p_mem_buf_desc;
         } else {
-            p_tcp_iov[0].p_desc->lwip_pbuf.pbuf.ref++;
+            p_tcp_iov[0].p_desc->lwip_pbuf.ref++;
         }
 
         /* save pointers to ip and tcp headers for software checksum calculation */
@@ -211,7 +214,7 @@ ssize_t dst_entry_tcp::fast_send(const iovec *p_iov, const ssize_t sz_iov, xlio_
         p_tcp_iov[0].p_desc->tx.p_tcp_h = static_cast<tcphdr *>(p_tcp_hdr);
 
         /* set wr_id as a pointer to memory descriptor */
-        m_p_send_wqe->wr_id = (uintptr_t)p_tcp_iov[0].p_desc;
+        p_send_wqe->wr_id = (uintptr_t)p_tcp_iov[0].p_desc;
 
         /* Update scatter gather element list
          * ref counter is incremented (above) for the first memory descriptor only because it is
@@ -222,14 +225,18 @@ ssize_t dst_entry_tcp::fast_send(const iovec *p_iov, const ssize_t sz_iov, xlio_
             m_sge[i].addr = (uintptr_t)p_tcp_iov[i].iovec.iov_base;
             m_sge[i].length = p_tcp_iov[i].iovec.iov_len;
             if (is_zerocopy) {
-                if (PBUF_DESC_MKEY == p_tcp_iov[i].p_desc->lwip_pbuf.pbuf.desc.attr) {
+                auto *p_desc = p_tcp_iov[i].p_desc;
+                auto &pbuf_descriptor = p_desc->lwip_pbuf.desc;
+                if (PBUF_DESC_EXPRESS == pbuf_descriptor.attr) {
+                    m_sge[i].lkey = pbuf_descriptor.mkey;
+                } else if (PBUF_DESC_MKEY == pbuf_descriptor.attr) {
                     /* PBUF_DESC_MKEY - value is provided by user */
-                    m_sge[i].lkey = p_tcp_iov[i].p_desc->lwip_pbuf.pbuf.desc.mkey;
-                } else if (PBUF_DESC_MDESC == p_tcp_iov[i].p_desc->lwip_pbuf.pbuf.desc.attr ||
-                           PBUF_DESC_NVME_TX == p_tcp_iov[i].p_desc->lwip_pbuf.pbuf.desc.attr) {
-                    mem_desc *mdesc = (mem_desc *)p_tcp_iov[i].p_desc->lwip_pbuf.pbuf.desc.mdesc;
-                    m_sge[i].lkey = mdesc->get_lkey(p_tcp_iov[i].p_desc, ib_ctx,
-                                                    (void *)m_sge[i].addr, m_sge[i].length);
+                    m_sge[i].lkey = pbuf_descriptor.mkey;
+                } else if (PBUF_DESC_MDESC == pbuf_descriptor.attr ||
+                           PBUF_DESC_NVME_TX == pbuf_descriptor.attr) {
+                    mem_desc *mdesc = (mem_desc *)pbuf_descriptor.mdesc;
+                    m_sge[i].lkey =
+                        mdesc->get_lkey(p_desc, ib_ctx, (void *)m_sge[i].addr, m_sge[i].length);
                     if (m_sge[i].lkey == LKEY_TX_DEFAULT) {
                         m_sge[i].lkey = m_p_ring->get_tx_lkey(m_id);
                     }
@@ -237,26 +244,24 @@ ssize_t dst_entry_tcp::fast_send(const iovec *p_iov, const ssize_t sz_iov, xlio_
                     /* Do not check desc.attr for specific type because
                      * PBUF_DESC_FD - is not possible for XLIO_TX_PACKET_ZEROCOPY
                      * PBUF_DESC_NONE - map should be initialized to NULL in
-                     * dst_entry_tcp::get_buffer() PBUF_DESC_MAP - map should point on mapping
-                     * object
+                     * dst_entry_tcp::get_buffer() object
                      */
                     masked_addr = (void *)((uint64_t)m_sge[i].addr & m_user_huge_page_mask);
                     m_sge[i].lkey =
-                        m_p_ring->get_tx_user_lkey(masked_addr, m_n_sysvar_user_huge_page_size,
-                                                   p_tcp_iov[i].p_desc->lwip_pbuf.pbuf.desc.map);
+                        m_p_ring->get_tx_user_lkey(masked_addr, m_n_sysvar_user_huge_page_size);
                 }
             } else {
                 m_sge[i].lkey = (i == 0 ? m_p_ring->get_tx_lkey(m_id) : m_sge[0].lkey);
             }
         }
 
-        ret = send_lwip_buffer(m_id, m_p_send_wqe, attr.flags, attr.tis);
+        ret = send_lwip_buffer(m_id, p_send_wqe, attr.flags, attr.tis);
     } else { // We don'nt support inline in this case, since we believe that this a very rare case
         mem_buf_desc_t *p_mem_buf_desc;
         size_t total_packet_len = 0;
 
-        p_mem_buf_desc = get_buffer(PBUF_RAM, NULL, is_set(attr.flags, XLIO_TX_PACKET_BLOCK));
-        if (p_mem_buf_desc == NULL) {
+        p_mem_buf_desc = get_buffer(PBUF_RAM, nullptr, is_set(attr.flags, XLIO_TX_PACKET_BLOCK));
+        if (!p_mem_buf_desc) {
             ret = -1;
             goto out;
         }
@@ -291,13 +296,13 @@ ssize_t dst_entry_tcp::fast_send(const iovec *p_iov, const ssize_t sz_iov, xlio_
         p_mem_buf_desc->tx.p_ip_h = p_ip_hdr;
         p_mem_buf_desc->tx.p_tcp_h = static_cast<tcphdr *>(p_tcp_hdr);
 
-        m_p_send_wqe = &m_not_inline_send_wqe;
-        m_p_send_wqe->wr_id = (uintptr_t)p_mem_buf_desc;
+        p_send_wqe = &m_not_inline_send_wqe;
+        p_send_wqe->wr_id = (uintptr_t)p_mem_buf_desc;
 
-        send_ring_buffer(m_id, m_p_send_wqe, attr.flags);
+        send_ring_buffer(m_id, p_send_wqe, attr.flags);
     }
 
-    if (unlikely(m_p_tx_mem_buf_desc_list == NULL)) {
+    if (unlikely(!m_p_tx_mem_buf_desc_list)) {
         m_p_tx_mem_buf_desc_list = m_p_ring->mem_buf_tx_get(
             m_id, is_set(attr.flags, XLIO_TX_PACKET_BLOCK), PBUF_RAM, m_n_sysvar_tx_bufs_batch_tcp);
     }
@@ -312,7 +317,7 @@ out:
 
 ssize_t dst_entry_tcp::slow_send(const iovec *p_iov, const ssize_t sz_iov, xlio_send_attr attr,
                                  struct xlio_rate_limit_t &rate_limit, int flags /*= 0*/,
-                                 socket_fd_api *sock /*= 0*/, tx_call_t call_type /*= 0*/)
+                                 sockinfo *sock /*= 0*/, tx_call_t call_type /*= 0*/)
 {
     ssize_t ret_val = -1;
 
@@ -364,12 +369,11 @@ void dst_entry_tcp::configure_headers()
     dst_entry::configure_headers();
 }
 
-ssize_t dst_entry_tcp::pass_buff_to_neigh(const iovec *p_iov, size_t sz_iov, uint32_t packet_id)
+ssize_t dst_entry_tcp::pass_buff_to_neigh(const iovec *p_iov, size_t sz_iov)
 {
-    NOT_IN_USE(packet_id);
     m_header_neigh->init();
     m_header_neigh->configure_tcp_ports(m_dst_port, m_src_port);
-    return (dst_entry::pass_buff_to_neigh(p_iov, sz_iov));
+    return pass_pkt_to_neigh(p_iov, sz_iov, 0U);
 }
 
 mem_buf_desc_t *dst_entry_tcp::get_buffer(pbuf_type type, pbuf_desc *desc,
@@ -382,41 +386,35 @@ mem_buf_desc_t *dst_entry_tcp::get_buffer(pbuf_type type, pbuf_desc *desc,
     p_desc_list = type == PBUF_ZEROCOPY ? &m_p_zc_mem_buf_desc_list : &m_p_tx_mem_buf_desc_list;
 
     // Get a bunch of tx buf descriptor and data buffers
-    if (unlikely(*p_desc_list == NULL)) {
+    if (unlikely(!*p_desc_list)) {
         *p_desc_list =
             m_p_ring->mem_buf_tx_get(m_id, b_blocked, type, m_n_sysvar_tx_bufs_batch_tcp);
     }
 
     mem_buf_desc_t *p_mem_buf_desc = *p_desc_list;
-    if (unlikely(p_mem_buf_desc == NULL)) {
+    if (unlikely(!p_mem_buf_desc)) {
         dst_tcp_logfunc("silent packet drop, no buffers!");
     } else {
         *p_desc_list = (*p_desc_list)->p_next_desc;
-        p_mem_buf_desc->p_next_desc = NULL;
+        p_mem_buf_desc->p_next_desc = nullptr;
         // for TX, set lwip payload to the data segment.
         // lwip will send it with payload pointing to the tcp header.
         if (p_mem_buf_desc->p_buffer) {
-            p_mem_buf_desc->lwip_pbuf.pbuf.payload = (u8_t *)p_mem_buf_desc->p_buffer +
+            p_mem_buf_desc->lwip_pbuf.payload = (u8_t *)p_mem_buf_desc->p_buffer +
                 m_header->m_aligned_l2_l3_len + sizeof(struct tcphdr);
         } else {
-            p_mem_buf_desc->lwip_pbuf.pbuf.payload = NULL;
+            p_mem_buf_desc->lwip_pbuf.payload = nullptr;
         }
 
         /* Initialize pbuf description */
-        memset(&p_mem_buf_desc->lwip_pbuf.pbuf.desc, 0,
-               sizeof(p_mem_buf_desc->lwip_pbuf.pbuf.desc));
-        p_mem_buf_desc->lwip_pbuf.pbuf.desc.attr = PBUF_DESC_NONE;
+        memset(&p_mem_buf_desc->lwip_pbuf.desc, 0, sizeof(p_mem_buf_desc->lwip_pbuf.desc));
+        p_mem_buf_desc->lwip_pbuf.desc.attr = PBUF_DESC_NONE;
         if (desc) {
-            memcpy(&p_mem_buf_desc->lwip_pbuf.pbuf.desc, desc,
-                   sizeof(p_mem_buf_desc->lwip_pbuf.pbuf.desc));
-            if (p_mem_buf_desc->lwip_pbuf.pbuf.desc.attr == PBUF_DESC_MDESC ||
-                p_mem_buf_desc->lwip_pbuf.pbuf.desc.attr == PBUF_DESC_NVME_TX) {
-                mem_desc *mdesc = (mem_desc *)p_mem_buf_desc->lwip_pbuf.pbuf.desc.mdesc;
+            memcpy(&p_mem_buf_desc->lwip_pbuf.desc, desc, sizeof(p_mem_buf_desc->lwip_pbuf.desc));
+            if (p_mem_buf_desc->lwip_pbuf.desc.attr == PBUF_DESC_MDESC ||
+                p_mem_buf_desc->lwip_pbuf.desc.attr == PBUF_DESC_NVME_TX) {
+                mem_desc *mdesc = (mem_desc *)p_mem_buf_desc->lwip_pbuf.desc.mdesc;
                 mdesc->get();
-            } else if (p_mem_buf_desc->lwip_pbuf.pbuf.type == PBUF_ZEROCOPY &&
-                       (p_mem_buf_desc->lwip_pbuf.pbuf.desc.attr == PBUF_DESC_MAP)) {
-                mapping_t *mapping = (mapping_t *)p_mem_buf_desc->lwip_pbuf.pbuf.desc.map;
-                mapping->get();
             }
         }
     }
@@ -431,7 +429,7 @@ void dst_entry_tcp::put_buffer(mem_buf_desc_t *p_desc)
 {
     // todo accumulate buffers?
 
-    if (unlikely(p_desc == NULL)) {
+    if (unlikely(!p_desc)) {
         return;
     }
 
@@ -440,26 +438,26 @@ void dst_entry_tcp::put_buffer(mem_buf_desc_t *p_desc)
     } else {
 
         // potential race, ref is protected here by tcp lock, and in ring by ring_tx lock
-        if (likely(p_desc->lwip_pbuf.pbuf.ref)) {
-            p_desc->lwip_pbuf.pbuf.ref--;
+        if (likely(p_desc->lwip_pbuf.ref)) {
+            p_desc->lwip_pbuf.ref--;
         } else {
             dst_tcp_logerr("ref count of %p is already zero, double free??", p_desc);
         }
 
-        if (p_desc->lwip_pbuf.pbuf.ref == 0) {
-            p_desc->p_next_desc = NULL;
-            buffer_pool::free_tx_lwip_pbuf_custom(&p_desc->lwip_pbuf.pbuf);
+        if (p_desc->lwip_pbuf.ref == 0) {
+            p_desc->p_next_desc = nullptr;
+            buffer_pool::free_tx_lwip_pbuf_custom(&p_desc->lwip_pbuf);
         }
     }
 }
 
 void dst_entry_tcp::put_zc_buffer(mem_buf_desc_t *p_desc)
 {
-    if (likely(p_desc->lwip_pbuf.pbuf.ref <= 1)) {
-        p_desc->lwip_pbuf.pbuf.ref = 1;
+    if (likely(p_desc->lwip_pbuf.ref <= 1)) {
+        p_desc->lwip_pbuf.ref = 1;
         p_desc->p_next_desc = m_p_zc_mem_buf_desc_list;
         m_p_zc_mem_buf_desc_list = p_desc;
     } else {
-        p_desc->lwip_pbuf.pbuf.ref--;
+        p_desc->lwip_pbuf.ref--;
     }
 }
