@@ -1,5 +1,7 @@
 /*
- * Copyright (c) 2001-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
+ * Copyright (c) 2001-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: GPL-2.0-only or BSD-3-Clause
  *
  * This software is available to you under a choice of one of two
  * licenses.  You may choose to be licensed under the terms of the GNU
@@ -60,6 +62,12 @@
 #define AGENT_DEFAULT_ALIVE (1) /* periodic time for alive check (in sec) */
 
 /* Force system call */
+#ifdef XLIO_STATIC_BUILD
+#define sys_call(_result, _func, ...)                                                              \
+    do {                                                                                           \
+        _result = ::_func(__VA_ARGS__);                                                            \
+    } while (0)
+#else /* XLIO_STATIC_BUILD */
 #define sys_call(_result, _func, ...)                                                              \
     do {                                                                                           \
         if (orig_os_api._func)                                                                     \
@@ -67,6 +75,7 @@
         else                                                                                       \
             _result = ::_func(__VA_ARGS__);                                                        \
     } while (0)
+#endif /* XLIO_STATIC_BUILD */
 
 /* Print user notification */
 #define output_fatal()                                                                             \
@@ -84,7 +93,7 @@
         vlog_printf(_level, "*************************************************************\n");    \
     } while (0)
 
-agent *g_p_agent = NULL;
+agent *g_p_agent = nullptr;
 
 agent::agent()
     : m_state(AGENT_CLOSED)
@@ -93,7 +102,7 @@ agent::agent()
     , m_msg_num(AGENT_DEFAULT_MSG_NUM)
 {
     int rc = 0;
-    agent_msg_t *msg = NULL;
+    agent_msg_t *msg = nullptr;
     int i = 0;
 
     INIT_LIST_HEAD(&m_cb_queue);
@@ -107,7 +116,7 @@ agent::agent()
     while (i--) {
         /* coverity[overwrite_var] */
         msg = (agent_msg_t *)calloc(1, sizeof(*msg));
-        if (NULL == msg) {
+        if (!msg) {
             rc = -ENOMEM;
             __log_dbg("failed queue creation (rc = %d)", rc);
             goto err;
@@ -207,8 +216,8 @@ err:
 
 agent::~agent()
 {
-    agent_msg_t *msg = NULL;
-    agent_callback_t *cb = NULL;
+    agent_msg_t *msg = nullptr;
+    agent_callback_t *cb = nullptr;
 
     if (AGENT_CLOSED == m_state) {
         return;
@@ -253,14 +262,14 @@ agent::~agent()
 
 void agent::register_cb(agent_cb_t fn, void *arg)
 {
-    agent_callback_t *cb = NULL;
-    struct list_head *entry = NULL;
+    agent_callback_t *cb = nullptr;
+    struct list_head *entry = nullptr;
 
     if (AGENT_CLOSED == m_state) {
         return;
     }
 
-    if (NULL == fn) {
+    if (!fn) {
         return;
     }
 
@@ -287,8 +296,8 @@ void agent::register_cb(agent_cb_t fn, void *arg)
 
 void agent::unregister_cb(agent_cb_t fn, void *arg)
 {
-    agent_callback_t *cb = NULL;
-    struct list_head *entry = NULL;
+    agent_callback_t *cb = nullptr;
+    struct list_head *entry = nullptr;
 
     if (AGENT_CLOSED == m_state) {
         return;
@@ -311,7 +320,7 @@ void agent::unregister_cb(agent_cb_t fn, void *arg)
 
 int agent::put(const void *data, size_t length, intptr_t tag)
 {
-    agent_msg_t *msg = NULL;
+    agent_msg_t *msg = nullptr;
     int i = 0;
 
     if (AGENT_CLOSED == m_state) {
@@ -338,7 +347,7 @@ int agent::put(const void *data, size_t length, intptr_t tag)
             for (i = 0; i < AGENT_DEFAULT_MSG_GROW; i++) {
                 /* coverity[overwrite_var] */
                 msg = (agent_msg_t *)malloc(sizeof(*msg));
-                if (NULL == msg) {
+                if (!msg) {
                     break;
                 }
                 msg->length = 0;
@@ -370,7 +379,7 @@ int agent::put(const void *data, size_t length, intptr_t tag)
 
 void agent::progress(void)
 {
-    agent_msg_t *msg = NULL;
+    agent_msg_t *msg = nullptr;
     struct timeval tv_now = TIMEVAL_INITIALIZER;
     static struct timeval tv_inactive_elapsed = TIMEVAL_INITIALIZER;
     static struct timeval tv_alive_elapsed = TIMEVAL_INITIALIZER;
@@ -424,8 +433,8 @@ go:
 
 void agent::progress_cb(void)
 {
-    agent_callback_t *cb = NULL;
-    struct list_head *entry = NULL;
+    agent_callback_t *cb = nullptr;
+    struct list_head *entry = nullptr;
 
     m_cb_lock.lock();
     list_for_each(entry, &m_cb_queue)
@@ -448,7 +457,7 @@ int agent::send(agent_msg_t *msg)
         return -EBADF;
     }
 
-    if (NULL == msg) {
+    if (!msg) {
         return -EINVAL;
     }
 
