@@ -1,5 +1,7 @@
 /*
- * Copyright (c) 2001-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
+ * Copyright (c) 2001-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: GPL-2.0-only or BSD-3-Clause
  *
  * This software is available to you under a choice of one of two
  * licenses.  You may choose to be licensed under the terms of the GNU
@@ -50,10 +52,9 @@ typedef enum {
     MCE_SPEC_NONE = 0,
     MCE_SPEC_SOCKPERF_ULTRA_LATENCY,
     MCE_SPEC_SOCKPERF_LATENCY,
-    MCE_SPEC_LL_MULTI_RING,
     MCE_SPEC_NGINX,
     MCE_SPEC_NGINX_DPU,
-    MCE_SPEC_NVME_BF2,
+    MCE_SPEC_NVME_BF3,
     MCE_SPEC_ALL /* last element */
 } xlio_spec_t;
 
@@ -111,14 +112,6 @@ static inline const char *ring_logic_str(ring_logic_t logic)
     return "unsupported";
 }
 
-typedef enum {
-    THREAD_MODE_SINGLE = 0,
-    THREAD_MODE_MULTI,
-    THREAD_MODE_MUTEX,
-    THREAD_MODE_PLENTY,
-    THREAD_MODE_LAST
-} thread_mode_t;
-
 typedef enum {
     BUFFER_BATCHING_NONE = 0,
     BUFFER_BATCHING_WITH_RECLAIM,
@@ -208,11 +201,6 @@ typedef enum { AUTO_ON_OFF_DEF } mode_t;
 OPTIONS_FROM_TO_STR_DEF;
 } // namespace option_3
 
-namespace option_strq {
-typedef enum { AUTO_ON_OFF_DEF, REGULAR_RQ = 2 } mode_t;
-OPTIONS_FROM_TO_STR_DEF;
-} // namespace option_strq
-
 namespace option_tcp_ctl_thread {
 typedef enum {
     CTL_THREAD_DISABLE = 0,
@@ -364,6 +352,7 @@ public:
     bool handle_sigintr;
     bool handle_segfault;
     uint32_t stats_fd_num_max;
+    uint32_t stats_fd_num_monitor;
 
     ring_logic_t ring_allocation_logic_tx;
     ring_logic_t ring_allocation_logic_rx;
@@ -374,9 +363,7 @@ public:
     int tcp_max_syn_rate;
 
     size_t zc_cache_threshold;
-    uint32_t tx_num_bufs;
     uint32_t tx_buf_size;
-    uint32_t zc_tx_size;
     uint32_t tcp_nodelay_treshold;
     uint32_t tx_num_wr;
     uint32_t tx_num_wr_to_signal;
@@ -388,7 +375,6 @@ public:
     uint32_t tx_bufs_batch_tcp;
     uint32_t tx_segs_batch_tcp;
 
-    uint32_t rx_num_bufs;
     uint32_t rx_buf_size;
     uint32_t rx_bufs_batch;
     uint32_t rx_num_wr;
@@ -408,14 +394,13 @@ public:
 
     uint32_t strq_stride_num_per_rwqe;
     uint32_t strq_stride_size_bytes;
-    uint32_t strq_strides_num_bufs;
     uint32_t strq_strides_compensation_level;
 
     uint32_t gro_streams_max;
     bool disable_flow_tag;
 
     bool enable_striding_rq;
-    bool enable_dpcp_rq;
+    bool tcp_2t_rules;
     bool tcp_3t_rules;
     bool udp_3t_rules;
     bool eth_mc_l2_only_rules;
@@ -440,6 +425,7 @@ public:
     uint32_t progress_engine_wce_max;
     bool cq_keep_qp_full;
     uint32_t qp_compensation_level;
+    uint32_t max_tso_sz;
     size_t user_huge_page_size;
 
     bool offloaded_sockets;
@@ -454,13 +440,12 @@ public:
     bool avoid_sys_calls_on_tcp_fd;
     bool allow_privileged_sock_opt;
     uint32_t wait_after_join_msec;
-    thread_mode_t thread_mode;
     buffer_batching_mode_t buffer_batching_mode;
     option_alloc_type::mode_t mem_alloc_type;
     size_t memory_limit;
     size_t memory_limit_user;
     size_t heap_metadata_block;
-    uint8_t hugepage_log2;
+    size_t hugepage_size;
     bool handle_fork;
     bool close_on_dup2;
     uint32_t mtu; /* effective MTU. If mtu==0 then auto calculate the MTU */
@@ -470,14 +455,13 @@ public:
     char internal_thread_affinity_str[FILENAME_MAX];
     cpu_set_t internal_thread_affinity;
     bool internal_thread_arm_cq_enabled;
-    bool handle_bf;
     skip_poll_in_rx_t skip_poll_in_rx;
     multilock_t multilock;
 
     bool enable_socketxtreme;
     option_3::mode_t enable_tso;
     option_3::mode_t enable_lro;
-    option_strq::mode_t enable_strq_env;
+    option_3::mode_t enable_strq_env;
 #ifdef DEFINED_UTLS
     bool enable_utls_rx;
     bool enable_utls_tx;
@@ -494,8 +478,6 @@ public:
     uint32_t neigh_wait_till_send_arp_msec;
     uint32_t neigh_num_err_retries;
 
-    uint32_t xlio_time_measure_num_samples;
-    char xlio_time_measure_filename[PATH_MAX];
     sysctl_reader_t &sysctl_reader;
     // Workaround for #3440429: postpone close(2) to the socket destructor, so the sockfd is closed
     // after the rfs rule is destroyed. Otherwise, flow_tag or TCP port can be reused too early.
@@ -546,6 +528,8 @@ private:
     // prevent unautothrized creation of objects
     mce_sys_var()
         : sysctl_reader(sysctl_reader_t::instance())
+        , stats_file(nullptr)
+        , m_ioctl {}
     {
         // coverity[uninit_member]
         get_env_params();
@@ -583,9 +567,7 @@ extern mce_sys_var &safe_mce_sys();
 #define SYS_VAR_RING_DEV_MEM_TX          "XLIO_RING_DEV_MEM_TX"
 
 #define SYS_VAR_ZC_CACHE_THRESHOLD    "XLIO_ZC_CACHE_THRESHOLD"
-#define SYS_VAR_TX_NUM_BUFS           "XLIO_TX_BUFS"
 #define SYS_VAR_TX_BUF_SIZE           "XLIO_TX_BUF_SIZE"
-#define SYS_VAR_ZC_TX_SIZE            "XLIO_ZC_TX_SIZE"
 #define SYS_VAR_TCP_NODELAY_TRESHOLD  "XLIO_TCP_NODELAY_TRESHOLD"
 #define SYS_VAR_TX_NUM_WRE            "XLIO_TX_WRE"
 #define SYS_VAR_TX_NUM_WRE_TO_SIGNAL  "XLIO_TX_WRE_BATCHING"
@@ -599,20 +581,15 @@ extern mce_sys_var &safe_mce_sys();
 #define SYS_VAR_STRQ                            "XLIO_STRQ"
 #define SYS_VAR_STRQ_NUM_STRIDES                "XLIO_STRQ_NUM_STRIDES"
 #define SYS_VAR_STRQ_STRIDE_SIZE_BYTES          "XLIO_STRQ_STRIDE_SIZE_BYTES"
-#define SYS_VAR_STRQ_STRIDES_NUM_BUFS           "XLIO_STRQ_STRIDES_NUM_BUFS"
 #define SYS_VAR_STRQ_STRIDES_COMPENSATION_LEVEL "XLIO_STRQ_STRIDES_COMPENSATION_LEVEL"
 
-#define SYS_VAR_RX_NUM_BUFS             "XLIO_RX_BUFS"
-#define SYS_VAR_RX_BUF_SIZE             "XLIO_RX_BUF_SIZE"
-#define SYS_VAR_RX_NUM_WRE              "XLIO_RX_WRE"
-#define SYS_VAR_RX_NUM_WRE_TO_POST_RECV "XLIO_RX_WRE_BATCHING"
-#define SYS_VAR_RX_NUM_POLLS            "XLIO_RX_POLL"
-#define SYS_VAR_RX_NUM_POLLS_INIT       "XLIO_RX_POLL_INIT"
-#define SYS_VAR_RX_UDP_POLL_OS_RATIO    "XLIO_RX_UDP_POLL_OS_RATIO"
-#define SYS_VAR_HW_TS_CONVERSION_MODE   "XLIO_HW_TS_CONVERSION"
-// The following 2 params were replaced by XLIO_RX_UDP_POLL_OS_RATIO
-#define SYS_VAR_RX_POLL_OS_RATIO              "XLIO_RX_POLL_OS_RATIO"
-#define SYS_VAR_RX_SKIP_OS                    "XLIO_RX_SKIP_OS"
+#define SYS_VAR_RX_BUF_SIZE                   "XLIO_RX_BUF_SIZE"
+#define SYS_VAR_RX_NUM_WRE                    "XLIO_RX_WRE"
+#define SYS_VAR_RX_NUM_WRE_TO_POST_RECV       "XLIO_RX_WRE_BATCHING"
+#define SYS_VAR_RX_NUM_POLLS                  "XLIO_RX_POLL"
+#define SYS_VAR_RX_NUM_POLLS_INIT             "XLIO_RX_POLL_INIT"
+#define SYS_VAR_RX_UDP_POLL_OS_RATIO          "XLIO_RX_UDP_POLL_OS_RATIO"
+#define SYS_VAR_HW_TS_CONVERSION_MODE         "XLIO_HW_TS_CONVERSION"
 #define SYS_VAR_RX_POLL_YIELD                 "XLIO_RX_POLL_YIELD"
 #define SYS_VAR_RX_BYTE_MIN_LIMIT             "XLIO_RX_BYTES_MIN"
 #define SYS_VAR_RX_PREFETCH_BYTES             "XLIO_RX_PREFETCH_BYTES"
@@ -620,6 +597,7 @@ extern mce_sys_var &safe_mce_sys();
 #define SYS_VAR_RX_CQ_DRAIN_RATE_NSEC         "XLIO_RX_CQ_DRAIN_RATE_NSEC"
 #define SYS_VAR_GRO_STREAMS_MAX               "XLIO_GRO_STREAMS_MAX"
 #define SYS_VAR_DISABLE_FLOW_TAG              "XLIO_DISABLE_FLOW_TAG"
+#define SYS_VAR_TCP_2T_RULES                  "XLIO_TCP_2T_RULES"
 #define SYS_VAR_TCP_3T_RULES                  "XLIO_TCP_3T_RULES"
 #define SYS_VAR_UDP_3T_RULES                  "XLIO_UDP_3T_RULES"
 #define SYS_VAR_ETH_MC_L2_ONLY_RULES          "XLIO_ETH_MC_L2_ONLY_RULES"
@@ -646,6 +624,7 @@ extern mce_sys_var &safe_mce_sys();
 #define SYS_VAR_PROGRESS_ENGINE_WCE_MAX   "XLIO_PROGRESS_ENGINE_WCE_MAX"
 #define SYS_VAR_CQ_KEEP_QP_FULL           "XLIO_CQ_KEEP_QP_FULL"
 #define SYS_VAR_QP_COMPENSATION_LEVEL     "XLIO_QP_COMPENSATION_LEVEL"
+#define SYS_VAR_MAX_TSO_SIZE              "XLIO_MAX_TSO_SIZE"
 #define SYS_VAR_USER_HUGE_PAGE_SIZE       "XLIO_USER_HUGE_PAGE_SIZE"
 #define SYS_VAR_OFFLOADED_SOCKETS         "XLIO_OFFLOADED_SOCKETS"
 #define SYS_VAR_TIMER_RESOLUTION_MSEC     "XLIO_TIMER_RESOLUTION_MSEC"
@@ -658,15 +637,13 @@ extern mce_sys_var &safe_mce_sys();
 #define SYS_VAR_AVOID_SYS_CALLS_ON_TCP_FD "XLIO_AVOID_SYS_CALLS_ON_TCP_FD"
 #define SYS_VAR_ALLOW_PRIVILEGED_SOCK_OPT "XLIO_ALLOW_PRIVILEGED_SOCK_OPT"
 #define SYS_VAR_WAIT_AFTER_JOIN_MSEC      "XLIO_WAIT_AFTER_JOIN_MSEC"
-#define SYS_VAR_THREAD_MODE               "XLIO_THREAD_MODE"
 #define SYS_VAR_BUFFER_BATCHING_MODE      "XLIO_BUFFER_BATCHING_MODE"
 #define SYS_VAR_MEM_ALLOC_TYPE            "XLIO_MEM_ALLOC_TYPE"
 #define SYS_VAR_MEMORY_LIMIT              "XLIO_MEMORY_LIMIT"
 #define SYS_VAR_MEMORY_LIMIT_USER         "XLIO_MEMORY_LIMIT_USER"
 #define SYS_VAR_HEAP_METADATA_BLOCK       "XLIO_HEAP_METADATA_BLOCK"
-#define SYS_VAR_HUGEPAGE_LOG2             "XLIO_HUGEPAGE_LOG2"
+#define SYS_VAR_HUGEPAGE_SIZE             "XLIO_HUGEPAGE_SIZE"
 #define SYS_VAR_FORK                      "XLIO_FORK"
-#define SYS_VAR_BF                        "XLIO_BF"
 #define SYS_VAR_CLOSE_ON_DUP2             "XLIO_CLOSE_ON_DUP2"
 #define SYS_VAR_MTU                       "XLIO_MTU"
 #if defined(DEFINED_NGINX)
@@ -707,8 +684,6 @@ extern mce_sys_var &safe_mce_sys();
 #define SYS_VAR_NEIGH_UC_ARP_DELAY_MSEC "XLIO_NEIGH_UC_ARP_DELAY_MSEC"
 #define SYS_VAR_NEIGH_NUM_ERR_RETRIES   "XLIO_NEIGH_NUM_ERR_RETRIES"
 
-#define SYS_VAR_TIME_MEASURE_NUM_SAMPLES       "XLIO_TIME_MEASURE_NUM_SAMPLES"
-#define SYS_VAR_TIME_MEASURE_DUMP_FILE         "XLIO_TIME_MEASURE_DUMP_FILE"
 #define SYS_VAR_DEFERRED_CLOSE                 "XLIO_DEFERRED_CLOSE"
 #define SYS_VAR_TCP_ABORT_ON_CLOSE             "XLIO_TCP_ABORT_ON_CLOSE"
 #define SYS_VAR_RX_POLL_ON_TX_TCP              "XLIO_RX_POLL_ON_TX_TCP"
@@ -735,18 +710,16 @@ extern mce_sys_var &safe_mce_sys();
 #define MCE_DEFAULT_APP_ID                   ("XLIO_DEFAULT_APPLICATION_ID")
 #define MCE_DEFAULT_HANDLE_SIGINTR           (true)
 #define MCE_DEFAULT_HANDLE_SIGFAULT          (false)
-#define MCE_DEFAULT_STATS_FD_NUM             100
-#define MCE_DEFAULT_RING_ALLOCATION_LOGIC_TX (RING_LOGIC_PER_INTERFACE)
-#define MCE_DEFAULT_RING_ALLOCATION_LOGIC_RX (RING_LOGIC_PER_INTERFACE)
+#define MCE_DEFAULT_STATS_FD_NUM             0
+#define MCE_DEFAULT_RING_ALLOCATION_LOGIC_TX (RING_LOGIC_PER_THREAD)
+#define MCE_DEFAULT_RING_ALLOCATION_LOGIC_RX (RING_LOGIC_PER_THREAD)
 #define MCE_DEFAULT_RING_MIGRATION_RATIO_TX  (-1)
 #define MCE_DEFAULT_RING_MIGRATION_RATIO_RX  (-1)
 #define MCE_DEFAULT_RING_LIMIT_PER_INTERFACE (0)
 #define MCE_DEFAULT_RING_DEV_MEM_TX          (0)
 #define MCE_DEFAULT_TCP_MAX_SYN_RATE         (0)
-#define MCE_DEFAULT_ZC_TX_SIZE               (32768)
 #define MCE_DEFAULT_TCP_NODELAY_TRESHOLD     (0)
 #define MCE_DEFAULT_ZC_CACHE_THRESHOLD       (10LU * 1024 * 1024 * 1024) // 10GB
-#define MCE_DEFAULT_TX_NUM_BUFS              (200000)
 #define MCE_DEFAULT_TX_BUF_SIZE              (0)
 #define MCE_DEFAULT_TX_NUM_WRE               (32768)
 #define MCE_DEFAULT_TX_NUM_WRE_TO_SIGNAL     (64)
@@ -762,27 +735,17 @@ extern mce_sys_var &safe_mce_sys();
 #define MCE_DEFAULT_TX_SEGS_POOL_BATCH_TCP   (16384)
 #define MCE_DEFAULT_TX_NUM_SGE               (4)
 
-#if defined(DEFINED_DPCP)
-#define MCE_DEFAULT_STRQ (option_strq::ON)
-#else
-#define MCE_DEFAULT_STRQ (option_strq::OFF)
-#endif
-
-#define MCE_DEFAULT_STRQ_NUM_STRIDES                (16384)
-#define MCE_DEFAULT_STRQ_STRIDE_SIZE_BYTES          (512)
-#define MCE_DEFAULT_STRQ_NUM_BUFS                   (64)
-#define MCE_DEFAULT_STRQ_NUM_WRE                    (8)
+#define MCE_DEFAULT_STRQ                            (option_3::ON)
+#define MCE_DEFAULT_STRQ_NUM_STRIDES                (2048)
+#define MCE_DEFAULT_STRQ_STRIDE_SIZE_BYTES          (64)
+#define MCE_DEFAULT_STRQ_NUM_WRE                    (128)
 #define MCE_DEFAULT_STRQ_NUM_WRE_TO_POST_RECV       (1)
-#define MCE_DEFAULT_STRQ_COMPENSATION_LEVEL         (1)
-#define MCE_DEFAULT_STRQ_STRIDES_NUM_BUFS           (262144)
-#define MCE_DEFAULT_STRQ_STRIDES_COMPENSATION_LEVEL (16384)
+#define MCE_DEFAULT_STRQ_STRIDES_COMPENSATION_LEVEL (32768)
 
-#define MCE_DEFAULT_RX_NUM_BUFS                   (200000)
 #define MCE_DEFAULT_RX_BUF_SIZE                   (0)
 #define MCE_DEFAULT_RX_BUFS_BATCH                 (64)
-#define MCE_DEFAULT_RX_NUM_WRE                    (16000)
+#define MCE_DEFAULT_RX_NUM_WRE                    (32768)
 #define MCE_DEFAULT_RX_NUM_WRE_TO_POST_RECV       (1024)
-#define MCE_DEFAULT_RX_NUM_SGE                    (1)
 #define MCE_DEFAULT_RX_NUM_POLLS                  (100000)
 #define MCE_DEFAULT_RX_NUM_POLLS_INIT             (0)
 #define MCE_DEFAULT_RX_UDP_POLL_OS_RATIO          (100)
@@ -794,6 +757,7 @@ extern mce_sys_var &safe_mce_sys();
 #define MCE_DEFAULT_RX_CQ_DRAIN_RATE              (MCE_RX_CQ_DRAIN_RATE_DISABLED)
 #define MCE_DEFAULT_GRO_STREAMS_MAX               (32)
 #define MCE_DEFAULT_DISABLE_FLOW_TAG              (false)
+#define MCE_DEFAULT_TCP_2T_RULES                  (false)
 #define MCE_DEFAULT_TCP_3T_RULES                  (false)
 #define MCE_DEFAULT_UDP_3T_RULES                  (true)
 #define MCE_DEFAULT_ETH_MC_L2_ONLY_RULES          (false)
@@ -810,15 +774,14 @@ extern mce_sys_var &safe_mce_sys();
 #endif
 #define MCE_DEFAULT_CQ_MODERATION_COUNT            (48)
 #define MCE_DEFAULT_CQ_MODERATION_PERIOD_USEC      (50)
-#define MCE_DEFAULT_CQ_AIM_MAX_COUNT               (560)
-#define MCE_DEFAULT_CQ_AIM_MAX_PERIOD_USEC         (250)
-#define MCE_DEFAULT_CQ_AIM_INTERVAL_MSEC           (250)
-#define MCE_DEFAULT_CQ_AIM_INTERRUPTS_RATE_PER_SEC (5000)
+#define MCE_DEFAULT_CQ_AIM_MAX_COUNT               (500)
+#define MCE_DEFAULT_CQ_AIM_MAX_PERIOD_USEC         (1000)
+#define MCE_DEFAULT_CQ_AIM_INTERVAL_MSEC           (1000)
+#define MCE_DEFAULT_CQ_AIM_INTERRUPTS_RATE_PER_SEC (1000)
 #define MCE_DEFAULT_CQ_POLL_BATCH                  (16)
 #define MCE_DEFAULT_PROGRESS_ENGINE_INTERVAL_MSEC  (10)
 #define MCE_DEFAULT_PROGRESS_ENGINE_WCE_MAX        (10000)
 #define MCE_DEFAULT_CQ_KEEP_QP_FULL                (true)
-#define MCE_DEFAULT_QP_COMPENSATION_LEVEL          (256)
 #define MCE_DEFAULT_USER_HUGE_PAGE_SIZE            (2 * 1024 * 1024)
 #define MCE_DEFAULT_INTERNAL_THREAD_ARM_CQ_ENABLED (false)
 #define MCE_DEFAULT_QP_FORCE_MC_ATTACH             (false)
@@ -833,15 +796,14 @@ extern mce_sys_var &safe_mce_sys();
 #define MCE_DEFAULT_AVOID_SYS_CALLS_ON_TCP_FD      (false)
 #define MCE_DEFAULT_ALLOW_PRIVILEGED_SOCK_OPT      (true)
 #define MCE_DEFAULT_WAIT_AFTER_JOIN_MSEC           (0)
-#define MCE_DEFAULT_THREAD_MODE                    (THREAD_MODE_MULTI)
 #define MCE_DEFAULT_BUFFER_BATCHING_MODE           (BUFFER_BATCHING_WITH_RECLAIM)
 #define MCE_DEFAULT_MEM_ALLOC_TYPE                 (option_alloc_type::HUGE)
 #define MCE_DEFAULT_MEMORY_LIMIT                   (2LU * 1024 * 1024 * 1024)
 #define MCE_DEFAULT_MEMORY_LIMIT_USER              (0)
 #define MCE_DEFAULT_HEAP_METADATA_BLOCK            (32LU * 1024 * 1024)
-#define MCE_DEFAULT_HUGEPAGE_LOG2                  (0)
+#define MCE_DEFAULT_HUGEPAGE_SIZE                  (0)
+#define MCE_MAX_HUGEPAGE_SIZE                      (1ULL << 63ULL)
 #define MCE_DEFAULT_FORK_SUPPORT                   (true)
-#define MCE_DEFAULT_BF_FLAG                        (true)
 #define MCE_DEFAULT_CLOSE_ON_DUP2                  (true)
 #define MCE_DEFAULT_MTU                            (0)
 #if defined(DEFINED_NGINX)
@@ -864,23 +826,20 @@ extern mce_sys_var &safe_mce_sys();
 #define MCE_DEFAULT_NEIGH_UC_ARP_DELAY_MSEC 10000
 #define MCE_DEFAULT_NEIGH_NUM_ERR_RETRIES   1
 
-#define MCE_DEFAULT_TIME_MEASURE_NUM_SAMPLES (10000)
-#define MCE_DEFAULT_TIME_MEASURE_DUMP_FILE   "/tmp/xlio_inst.dump"
-
 #define MCE_MIN_NUM_SGE                     (1)
 #define MCE_MAX_NUM_SGE                     (32)
 #define MCE_MIN_RX_NUM_POLLS                (-1)
 #define MCE_MAX_RX_NUM_POLLS                (100000000)
 #define MCE_MIN_RX_PREFETCH_BYTES           (32) /* Just enough for headers (IPoIB+IP+UDP)*/
 #define MCE_MAX_RX_PREFETCH_BYTES           (2044)
-#define MCE_MAX_ZC_TX_SIZE                  (65535)
 #define MCE_RX_CQ_DRAIN_RATE_DISABLED       (0)
 #define MCE_CQ_DRAIN_INTERVAL_DISABLED      (0)
 #define MCE_CQ_ADAPTIVE_MODERATION_DISABLED (0)
 #define MCE_MIN_CQ_POLL_BATCH               (1)
-#define MCE_MAX_CQ_POLL_BATCH               (128)
+#define MCE_MAX_CQ_POLL_BATCH               (32768)
 #define MCE_DEFAULT_SOCKETXTREME            (false)
 #define MCE_DEFAULT_TSO                     (option_3::AUTO)
+#define MCE_DEFAULT_MAX_TSO_SIZE            (256 * 1024)
 #ifdef DEFINED_UTLS
 #define MCE_DEFAULT_UTLS_RX                        (false)
 #define MCE_DEFAULT_UTLS_TX                        (true)
@@ -902,8 +861,9 @@ extern mce_sys_var &safe_mce_sys();
  * This block consists of auxiliary constants
  */
 #define RX_BUF_SIZE(mtu) (mtu)
-#define TX_BUF_SIZE(mtu)                                                                           \
-    ((mtu) + 92) // Tx buffers are larger in Ethernet (they include L2 for RAW QP)
+// Add room for the L2-L4 headers
+#define TX_BUF_SIZE(mtu) ((mtu) + 92)
+
 #define NUM_TX_WRE_TO_SIGNAL_MAX            64
 #define NUM_RX_WRE_TO_POST_RECV_MAX         1024
 #define MAX_MLX5_CQ_SIZE_ITEMS              4194304
@@ -935,7 +895,7 @@ extern mce_sys_var &safe_mce_sys();
 #define NETVSC_DEVICE_UPPER_FILE "/sys/class/net/%s/upper_%s/ifindex"
 #define NETVSC_ID                "{f8615163-df3e-46c5-913f-f2d2f965ed0e}\n"
 
-#define MAX_STATS_FD_NUM   1024
+#define MAX_STATS_FD_NUM   1024U
 #define MAX_WINDOW_SCALING 14
 
 #define STRQ_MIN_STRIDES_NUM       512
