@@ -1,5 +1,7 @@
 /*
- * Copyright (c) 2001-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
+ * Copyright (c) 2001-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: GPL-2.0-only or BSD-3-Clause
  *
  * This software is available to you under a choice of one of two
  * licenses.  You may choose to be licensed under the terms of the GNU
@@ -192,10 +194,10 @@ inline int priv_safe_try_read_file(const char *path, char *buf, size_t size)
 
 /**
  * Read content of file detailed in 'path' (usually a sysfs file)
- * upon failure print error
+ * upon failure print warning
  * @return int value (atoi) of the file content, or 'default_value' upon failure
  */
-int read_file_to_int(const char *path, int default_value, vlog_levels_t log_level = VLOG_ERROR);
+int read_file_to_int(const char *path, int default_value, vlog_levels_t log_level = VLOG_WARNING);
 
 /**
  * Get port number from interface name
@@ -279,8 +281,8 @@ size_t get_vlan_base_name_from_ifname(const char *ifname, char *base_ifname, siz
 size_t get_local_ll_addr(const char *ifname, unsigned char *addr, int addr_len, bool is_broadcast);
 
 /* Print warning while RoCE Lag is enabled */
-void print_roce_lag_warnings(const char *interface, char *disable_path = NULL,
-                             const char *port1 = NULL, const char *port2 = NULL);
+void print_roce_lag_warnings(const char *interface, char *disable_path = nullptr,
+                             const char *port1 = nullptr, const char *port2 = nullptr);
 
 /*Print a warning to the user when there was an error registering memory*/
 void print_warning_rlimit_memlock(size_t length, int error);
@@ -323,10 +325,10 @@ static inline int get_procname(int pid, char *proc, size_t size)
 {
     char app_full_name[PATH_MAX] = {0};
     char proccess_proc_dir[FILE_NAME_MAX_SIZE] = {0};
-    char *app_base_name = NULL;
+    char *app_base_name = nullptr;
     int n = -1;
 
-    if (NULL == proc) {
+    if (!proc) {
         return -1;
     }
 
@@ -351,7 +353,7 @@ static inline int get_procname(int pid, char *proc, size_t size)
 inline void create_multicast_mac_from_ip(unsigned char *mc_mac, const ip_address &addr,
                                          sa_family_t family)
 {
-    if (mc_mac == NULL) {
+    if (!mc_mac) {
         return;
     }
 
