@@ -385,7 +385,7 @@ enum ctx_device {
 	ERDMA			= 29,
 	HNS			= 30,
 	CONNECTX8		= 31,
-	INTEL_GEN2		= 32,
+  INTEL_GEN2		= 32,
 };
 
 /* Units for rate limiter */
@@ -572,7 +572,6 @@ struct perftest_parameters {
 	int				cuda_device_id;
 	char				*cuda_device_bus_id;
 	int				use_cuda_dmabuf;
-	int				use_data_direct;
 	int				rocm_device_id;
 	int				neuron_core_id;
 	int				use_neuron_dmabuf;
@@ -641,8 +640,8 @@ struct perftest_parameters {
 	int 			ah_allocated;
 	int				use_write_with_imm;
 	int				use_unsolicited_write;
-	int				use_ddp;
-	int				no_ddp;
+	uint16_t			cqe_poll;
+	int				use_cqe_poll;
 };
 
 struct report_options {
