@@ -55,6 +55,9 @@
 #if defined(HAVE_MLX5DV)
 #include <infiniband/mlx5dv.h>
 #endif
+#if defined(HAVE_HNSDV)
+#include <infiniband/hnsdv.h>
+#endif
 #include <rdma/rdma_cma.h>
 #include <stdint.h>
 #if defined(__FreeBSD__)
@@ -178,6 +181,10 @@ struct pingpong_context {
 	struct ibv_comp_channel			*recv_channel;
 	struct ibv_comp_channel			*send_channel;
 	struct ibv_pd				*pd;
+	#ifdef HAVE_TD_API
+	struct ibv_td				*td;
+	struct ibv_pd				*pad;
+	#endif
 	struct ibv_mr				**mr;
 	struct ibv_mr				*null_mr;
 	struct ibv_cq				*send_cq;
@@ -520,7 +527,8 @@ void ctx_set_send_wqes(struct pingpong_context *ctx,
  *
  * Description :
  *
- *	Prepare the receives work request templates for all QPs in SEND receive test.
+ *	Prepare the receives work request templates for all QPs in SEND and
+ *	WRITE_IMM receive test.
  *
  * Parameters :
  *
@@ -650,6 +658,19 @@ int run_iter_bi(struct pingpong_context *ctx,struct perftest_parameters *user_pa
  */
 int run_iter_lat_write(struct pingpong_context *ctx,struct perftest_parameters *user_param);
 
+/* run_iter_lat_write_imm
+ *
+ * Description :
+ *
+ *  This is the latency test function for WRITE_IMM verb.
+ *
+ * Parameters :
+ *
+ *	ctx     - Test Context.
+ *	user_param  - user_parameters struct for this test.
+ */
+int run_iter_lat_write_imm(struct pingpong_context *ctx,struct perftest_parameters *user_param);
+
 /* run_iter_lat
  *
  * Description :
@@ -790,13 +811,13 @@ static __inline int ctx_notify_send_recv_events(struct pingpong_context *ctx)
 
 	if (FD_ISSET(ctx->recv_channel->fd, &rfds) &&
 	    ctx_notify_events(ctx->recv_channel)) {
-		fprintf(stderr,"Failed to notify receive events to CQ");
+		fprintf(stderr,"Failed to notify receive events to CQ\n");
 		return FAILURE;
 	}
 
 	if (FD_ISSET(ctx->send_channel->fd, &rfds) &&
 	    ctx_notify_events(ctx->send_channel)) {
-		fprintf(stderr,"Failed to notify send events to CQ");
+		fprintf(stderr,"Failed to notify send events to CQ\n");
 		return FAILURE;
 	}
 
