@@ -95,13 +95,8 @@ u16 nvmet_report_invalid_opcode(struct nvmet_req *req)
 static struct nvmet_subsys *nvmet_find_get_subsys(struct nvmet_port *port,
 		const char *subsysnqn);
 
-#ifdef CONFIG_COMPAT_SCATTERLIST_SG_PCOPY_TO_BUFFER
 u16 nvmet_copy_to_sgl(struct nvmet_req *req, off_t off, const void *buf,
 		size_t len)
-#else
-u16 nvmet_copy_to_sgl(struct nvmet_req *req, off_t off, void *buf,
-		size_t len)
-#endif
 {
 	if (sg_pcopy_from_buffer(req->sg, req->sg_cnt, buf, len, off) != len) {
 		req->error_loc = offsetof(struct nvme_common_command, dptr);
@@ -256,7 +251,7 @@ void nvmet_ns_changed(struct nvmet_subsys *subsys, u32 nsid)
 		nvmet_add_to_changed_ns_log(ctrl, cpu_to_le32(nsid));
 		if (nvmet_aen_bit_disabled(ctrl, NVME_AEN_BIT_NS_ATTR))
 			continue;
-		nvmet_add_async_event(ctrl, NVME_AER_TYPE_NOTICE,
+		nvmet_add_async_event(ctrl, NVME_AER_NOTICE,
 				NVME_AER_NOTICE_NS_CHANGED,
 				NVME_LOG_CHANGED_NS);
 	}
@@ -273,7 +268,7 @@ void nvmet_send_ana_event(struct nvmet_subsys *subsys,
 			continue;
 		if (nvmet_aen_bit_disabled(ctrl, NVME_AEN_BIT_ANA_CHANGE))
 			continue;
-		nvmet_add_async_event(ctrl, NVME_AER_TYPE_NOTICE,
+		nvmet_add_async_event(ctrl, NVME_AER_NOTICE,
 				NVME_AER_NOTICE_ANA, NVME_LOG_ANA);
 	}
 	mutex_unlock(&subsys->lock);
@@ -349,7 +344,8 @@ static bool nvmet_peer_to_peer_capable(struct nvmet_port *port)
 	    ops->offload_ns_error_cmds &&
 	    ops->offload_ns_backend_error_cmds &&
 	    ops->offload_query_counters &&
-	    ops->check_subsys_match_offload_port)
+	    ops->check_subsys_match_offload_port &&
+	    ops->offload_ns_is_active)
 		return ops->peer_to_peer_capable(port);
 
 	return false;
@@ -471,12 +467,23 @@ int nvmet_enable_port(struct nvmet_port *port, struct nvmet_subsys *subsys)
 	if (port->inline_data_size < 0)
 		port->inline_data_size = 0;
 
+	/*
+	 * If the transport didn't set the max_queue_size properly, then clamp
+	 * it to the target limits. Also set default values in case the
+	 * transport didn't set it at all.
+	 */
+	if (port->max_queue_size < 0)
+		port->max_queue_size = NVMET_MAX_QUEUE_SIZE;
+	else
+		port->max_queue_size = clamp_t(int, port->max_queue_size,
+					       NVMET_MIN_QUEUE_SIZE,
+					       NVMET_MAX_QUEUE_SIZE);
+
 	port->enabled = true;
 	port->offload = subsys->offloadble;
 	port->tr_ops = ops;
 	return 0;
 
-
 out_remove_port:
 	ops->remove_port(port);
 out_module_put:
@@ -549,10 +556,13 @@ void nvmet_stop_keep_alive_timer(struct nvmet_ctrl *ctrl)
 u16 nvmet_req_find_ns(struct nvmet_req *req)
 {
 	u32 nsid = le32_to_cpu(req->cmd->common.nsid);
+	struct nvmet_subsys *subsys = nvmet_req_subsys(req);
 
-	req->ns = xa_load(&nvmet_req_subsys(req)->namespaces, nsid);
+	req->ns = xa_load(&subsys->namespaces, nsid);
 	if (unlikely(!req->ns)) {
 		req->error_loc = offsetof(struct nvme_common_command, nsid);
+		if (nvmet_subsys_nsid_exists(subsys, nsid))
+			return NVME_SC_INTERNAL_PATH_ERROR;
 		return NVME_SC_INVALID_NS | NVME_SC_DNR;
 	}
 
@@ -575,9 +585,7 @@ void nvmet_put_namespace(struct nvmet_ns *ns)
 static void nvmet_ns_dev_disable(struct nvmet_ns *ns)
 {
 	nvmet_bdev_ns_disable(ns);
-#ifdef HAVE_FS_HAS_KIOCB
 	nvmet_file_ns_disable(ns);
-#endif
 }
 
 static int nvmet_p2pmem_ns_enable(struct nvmet_ns *ns)
@@ -669,11 +677,8 @@ bool nvmet_ns_revalidate(struct nvmet_ns *ns)
 
 	if (ns->bdev)
 		nvmet_bdev_ns_revalidate(ns);
-#ifdef HAVE_FS_HAS_KIOCB
 	else
 		nvmet_file_ns_revalidate(ns);
-#endif
-
 	return oldsize != ns->size;
 }
 
@@ -728,10 +733,8 @@ int nvmet_ns_enable(struct nvmet_ns *ns)
 		goto out_unlock;
 
 	ret = nvmet_bdev_ns_enable(ns);
-#ifdef HAVE_FS_HAS_KIOCB
 	if (ret == -ENOTBLK)
 		ret = nvmet_file_ns_enable(ns);
-#endif
 	if (ret)
 		goto out_unlock;
 
@@ -1126,10 +1129,8 @@ static u16 nvmet_parse_io_cmd(struct nvmet_req *req)
 
 	switch (req->ns->csi) {
 	case NVME_CSI_NVM:
-#ifdef HAVE_FS_HAS_KIOCB
 		if (req->ns->file)
 			return nvmet_file_parse_io_cmd(req);
-#endif
 		return nvmet_bdev_parse_io_cmd(req);
 	case NVME_CSI_ZNS:
 #ifdef HAVE_BIO_ADD_ZONE_APPEND_PAGE
@@ -1278,63 +1279,6 @@ static struct pci_dev *nvmet_req_find_p2p_dev(struct nvmet_req *req)
 	return radix_tree_lookup(&req->sq->ctrl->p2p_ns_map, req->ns->nsid);
 }
 
-#ifndef HAVE_SGL_FREE
-static void nvmet_free_sgl(struct scatterlist *sgl, unsigned int nents)
-{
-	struct scatterlist *sg;
-	int count;
-
-	if (!sgl || !nents)
-		return;
-
-	for_each_sg(sgl, sg, nents, count)
-		__free_page(sg_page(sg));
-	kfree(sgl);
-}
-#endif
-
-#ifndef HAVE_SGL_ALLOC
-static int nvmet_alloc_sgl(struct scatterlist **sgl, unsigned int *nents,
-               u32 length)
-{
-	struct scatterlist *sg;
-	struct page *page;
-	unsigned int nent;
-	int i = 0;
-
-	nent = DIV_ROUND_UP(length, PAGE_SIZE);
-	sg = kmalloc_array(nent, sizeof(struct scatterlist), GFP_KERNEL);
-	if (!sg)
-		goto out;
-
-	sg_init_table(sg, nent);
-
-	while (length) {
-		u32 page_len = min_t(u32, length, PAGE_SIZE);
-
-	        page = alloc_page(GFP_KERNEL);
-		if (!page)
-	                goto out_free_pages;
-
-	        sg_set_page(&sg[i], page, page_len, 0);
-		length -= page_len;
-		i++;
-	}
-	*sgl = sg;
-	*nents = nent;
-	return 0;
-
-out_free_pages:
-	while (i > 0) {
-		i--;
-		__free_page(sg_page(&sg[i]));
-	}
-	kfree(sg);
-out:
-	return NVME_SC_INTERNAL;
-}
-#endif
-
 int nvmet_req_alloc_sgls(struct nvmet_req *req)
 {
 	struct pci_dev *p2p_dev = nvmet_req_find_p2p_dev(req);
@@ -1342,37 +1286,21 @@ int nvmet_req_alloc_sgls(struct nvmet_req *req)
 	if (p2p_dev && !nvmet_req_alloc_p2pmem_sgls(p2p_dev, req))
 		return 0;
 
-#ifdef HAVE_SGL_ALLOC
 	req->sg = sgl_alloc(nvmet_data_transfer_len(req), GFP_KERNEL,
 			    &req->sg_cnt);
 	if (unlikely(!req->sg))
 		goto out;
-#else
-	if (nvmet_alloc_sgl(&req->sg, &req->sg_cnt,
-			    nvmet_data_transfer_len(req)))
-		goto out;
-#endif
 
 	if (req->metadata_len) {
-#ifdef HAVE_SGL_ALLOC
 		req->metadata_sg = sgl_alloc(req->metadata_len, GFP_KERNEL,
 					     &req->metadata_sg_cnt);
 		if (unlikely(!req->metadata_sg))
 			goto out_free;
-#else
-		if (nvmet_alloc_sgl(&req->metadata_sg, &req->metadata_sg_cnt,
-				    req->metadata_len))
-			goto out_free;
-#endif
 	}
 
 	return 0;
 out_free:
-#ifdef HAVE_SGL_FREE
 	sgl_free(req->sg);
-#else
-	nvmet_free_sgl(req->sg, req->sg_cnt);
-#endif
 out:
 	return -ENOMEM;
 }
@@ -1386,15 +1314,9 @@ void nvmet_req_free_sgls(struct nvmet_req *req)
 			pci_p2pmem_free_sgl(req->p2p_dev, req->metadata_sg);
 		req->p2p_dev = NULL;
 	} else {
-#ifdef HAVE_SGL_FREE
 		sgl_free(req->sg);
 		if (req->metadata_sg)
 			sgl_free(req->metadata_sg);
-#else
-		nvmet_free_sgl(req->sg, req->sg_cnt);
-		if (req->metadata_sg)
-			nvmet_free_sgl(req->metadata_sg, req->metadata_sg_cnt);
-#endif
 	}
 
 	req->sg = NULL;
@@ -1526,9 +1448,10 @@ static void nvmet_init_cap(struct nvmet_ctrl *ctrl)
 	ctrl->cap |= (15ULL << 24);
 	/* maximum queue entries supported: */
 	if (ctrl->ops->get_max_queue_size)
-		ctrl->cap |= ctrl->ops->get_max_queue_size(ctrl) - 1;
+		ctrl->cap |= min_t(u16, ctrl->ops->get_max_queue_size(ctrl),
+				   ctrl->port->max_queue_size) - 1;
 	else
-		ctrl->cap |= NVMET_QUEUE_SIZE - 1;
+		ctrl->cap |= ctrl->port->max_queue_size - 1;
 
 	if (nvmet_is_passthru_subsys(ctrl->subsys))
 		nvmet_passthrough_override_cap(ctrl);
@@ -1714,6 +1637,7 @@ u16 nvmet_alloc_ctrl(const char *subsysnqn, const char *hostnqn,
 
 	kref_init(&ctrl->ref);
 	ctrl->subsys = subsys;
+	ctrl->pi_support = ctrl->port->pi_enable && ctrl->subsys->pi_support;
 	nvmet_init_cap(ctrl);
 	WRITE_ONCE(ctrl->aen_enabled, NVMET_AEN_CFG_OPTIONAL);
 
@@ -1728,9 +1652,6 @@ u16 nvmet_alloc_ctrl(const char *subsysnqn, const char *hostnqn,
 	if (!ctrl->sqs)
 		goto out_free_changed_ns_list;
 
-	if (subsys->cntlid_min > subsys->cntlid_max)
-		goto out_free_sqs;
-
 #ifdef HAVE_IDA_ALLOC_RANGE
 	ret = ida_alloc_range(&cntlid_ida,
 #else
@@ -1984,7 +1905,8 @@ static int __init nvmet_init(void)
 	if (!buffered_io_wq)
 		goto out_free_zbd_work_queue;
 
-	nvmet_wq = alloc_workqueue("nvmet-wq", WQ_MEM_RECLAIM, 0);
+	nvmet_wq = alloc_workqueue("nvmet-wq",
+			WQ_MEM_RECLAIM | WQ_UNBOUND, 0);
 	if (!nvmet_wq)
 		goto out_free_buffered_work_queue;
 
@@ -2027,6 +1949,7 @@ static void __exit nvmet_exit(void)
 module_init(nvmet_init);
 module_exit(nvmet_exit);
 
+MODULE_DESCRIPTION("NVMe target core framework");
 MODULE_LICENSE("GPL v2");
 #ifdef RETPOLINE_MLNX
 MODULE_INFO(retpoline, "Y");
